@@ -1,16 +1,24 @@
-/***************************************************************************
- *   Copyright (C) 2004 Frederik Holljen <fh@ez.no>                        *
- *             (C) 2004, 2005 Max Howell <max.howell@methylblue.com>       *
- *             (C) 2004-2008 Mark Kretschmann <kretschmann@kde.org>        *
- *             (C) 2006, 2008 Ian Monroe <ian@monroe.nu>                   *
- *             (C) 2008 Jason A. Donenfeld <Jason@zx2c4.com>               *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2004 Frederik Holljen <fh@ez.no>                                       *
+ * Copyright (c) 2004,2005 Max Howell <max.howell@methylblue.com>                       *
+ * Copyright (c) 2004-2008 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2006,2008 Ian Monroe <ian@monroe.nu>                                   *
+ * Copyright (c) 2008 Jason A. Donenfeld <Jason@zx2c4.com>                              *
+ * Copyright (c) 2009 Nikolaj Hald Nielsen <nhn@kde.org>                                *
+ * Copyright (c) 2009 Artur Szymiec <artur.szymiec@gmail.com>                           *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #define DEBUG_PREFIX "EngineController"
 
@@ -19,15 +27,17 @@
 #include "Amarok.h"
 #include "amarokconfig.h"
 #include "collection/CollectionManager.h"
+#include "Components.h"
 #include "statusbar/StatusBar.h"
 #include "Debug.h"
 #include "MainWindow.h"
+#include "MediaDeviceMonitor.h"
 #include "meta/Meta.h"
 #include "meta/MetaConstants.h"
 #include "meta/capabilities/MultiPlayableCapability.h"
 #include "meta/capabilities/MultiSourceCapability.h"
 #include "playlist/PlaylistActions.h"
-#include "playlistmanager/PlaylistManager.h"
+#include "PlaylistFileSupport.h"
 #include "PluginManager.h"
 
 #include <KFileItem>
@@ -48,75 +58,102 @@ namespace The {
     EngineController* engineController() { return EngineController::instance(); }
 }
 
-EngineController* EngineController::s_instance = 0;
-
 EngineController*
 EngineController::instance()
 {
-    return s_instance ? s_instance : new EngineController();
+    return Amarok::Components::engineController();
 }
 
 void
 EngineController::destroy()
 {
-    delete s_instance;
-    s_instance = 0;
+    //nothing to do?
 }
 
 EngineController::EngineController()
     : m_playWhenFetched( true )
     , m_fadeoutTimer( new QTimer( this ) )
     , m_volume( 0 )
+    , m_currentIsAudioCd( false )
 {
     DEBUG_BLOCK
 
-    initializePhonon();
-
     m_fadeoutTimer->setSingleShot( true );
 
     connect( m_fadeoutTimer, SIGNAL( timeout() ), SLOT( slotStopFadeout() ) );
-
-    s_instance = this;
 }
 
 EngineController::~EngineController()
 {
     DEBUG_BLOCK //we like to know when singletons are destroyed
 
+    // don't do any of the after-processing that normally happens when
+    // the media is stopped - that's what endSession() is for
+    m_media->blockSignals(true);
     m_media->stop();
 
     delete m_media;
     delete m_audio;
 }
 
+void
+EngineController::createFadeoutEffect()
+{
+    m_fader = new Phonon::VolumeFaderEffect( this );
+    m_path.insertEffect( m_fader );
+    m_fader->setFadeCurve( Phonon::VolumeFaderEffect::Fade9Decibel );
+}
+
 void
 EngineController::initializePhonon()
 {
     DEBUG_BLOCK
 
+    m_path.disconnect();
     delete m_media;
+    delete m_controller;
     delete m_audio;
     delete m_preamp;
+    delete m_equalizer;
+    delete m_fader;
 
     PERF_LOG( "EngineController: loading phonon objects" )
     m_media = new Phonon::MediaObject( this );
     m_audio = new Phonon::AudioOutput( Phonon::MusicCategory, this );
 
     m_path = Phonon::createPath( m_media, m_audio );
-    
+
+    m_controller = new Phonon::MediaController( m_media );
+
+    //Add an equalizer effect if available
+    QList<Phonon::EffectDescription> mEffectDescriptions = Phonon::BackendCapabilities::availableAudioEffects();
+    foreach ( const Phonon::EffectDescription &mDescr, mEffectDescriptions ) {
+        if ( mDescr.name() == QLatin1String( "KEqualizer" ) ) {
+            m_equalizer = new Phonon::Effect( mDescr );
+            eqUpdate();
+            }
+    }
+
     // HACK we turn off replaygain manually on OSX, until the phonon coreaudio backend is fixed.
     // as the default is specified in the .cfg file, we can't just tell it to be a different default on OSX
 #ifdef Q_WS_MAC
     AmarokConfig::setReplayGainMode( AmarokConfig::EnumReplayGainMode::Off );
+    AmarokConfig::setFadeout( false );
 #endif
 
-    // only create pre-amp if we have replaygain on, preamp can cause phonon issues
+    // only create pre-amp if we have replaygain on, VolumeFaderEffect can cause phonon issues
     if( AmarokConfig::replayGainMode() != AmarokConfig::EnumReplayGainMode::Off )
-    {   
+    {
         m_preamp = new Phonon::VolumeFaderEffect( this );
         m_path.insertEffect( m_preamp );
     }
 
+    // only create fader if we have fadeout on, VolumeFaderEffect can cause phonon issues
+    if( AmarokConfig::fadeout() && AmarokConfig::fadeoutLength() )
+    {
+        createFadeoutEffect();
+    }
+
     m_media->setTickInterval( 100 );
     debug() << "Tick Interval (actual): " << m_media->tickInterval();
     PERF_LOG( "EngineController: loaded phonon objects" )
@@ -132,14 +169,13 @@ EngineController::initializePhonon()
     connect( m_media, SIGNAL( totalTimeChanged( qint64 ) ), SLOT( slotTrackLengthChanged( qint64 ) ) );
     connect( m_media, SIGNAL( currentSourceChanged( const Phonon::MediaSource & ) ), SLOT( slotNewTrackPlaying( const Phonon::MediaSource & ) ) );
 
-    
-    //TODO: The xine engine does not support crossfading. Cannot get the gstreamer engine to work, will test this once I do.
-#if 0
+    connect( m_controller, SIGNAL( titleChanged( int ) ), SLOT( slotTitleChanged( int ) ) );
+
+
     if( AmarokConfig::trackDelayLength() > -1 )
         m_media->setTransitionTime( AmarokConfig::trackDelayLength() ); // Also Handles gapless.
     else if( AmarokConfig::crossfadeLength() > 0 )  // TODO: Handle the possible options on when to crossfade.. the values are not documented anywhere however
         m_media->setTransitionTime( -AmarokConfig::crossfadeLength() );
-#endif
 }
 
 
@@ -153,7 +189,7 @@ EngineController::canDecode( const KUrl &url ) //static
    //NOTE this function must be thread-safe
 
     // We can't use playlists in the engine
-    if( PlaylistManager::isPlaylist( url ) )
+    if( Meta::isPlaylist( url ) )
         return false;
 
     KFileItem item( KFileItem::Unknown, KFileItem::Unknown, url );
@@ -161,7 +197,7 @@ EngineController::canDecode( const KUrl &url ) //static
     if( !item.size() )
         return false;
 
-    // We can't play directories, reguardless of what the engine says.
+    // We can't play directories, regardless of what the engine says.
     if( item.isDir() )
         return false;
 
@@ -169,21 +205,46 @@ EngineController::canDecode( const KUrl &url ) //static
     if( !item.isLocalFile() )
         return true;
 
+    // Filter the available mime types to only include audio and video, as amarok does not intend to play photos
+    static QStringList mimeTable = supportedMimeTypes();
+
+    const KMimeType::Ptr mimeType = item.mimeTypePtr();
+    
+    bool valid = false;
+    foreach( const QString &type, mimeTable )
+    {
+        if( mimeType->is( type ) )
+        {
+            valid = true;
+            break;
+        }
+    }
+
+    return valid;
+}
+
+QStringList
+EngineController::supportedMimeTypes()
+{
+    //NOTE this function must be thread-safe
     // Filter the available mime types to only include audio and video, as amarok does not intend to play photos
     static QStringList mimeTable = Phonon::BackendCapabilities::availableMimeTypes().filter( "audio/", Qt::CaseInsensitive ) +
                                    Phonon::BackendCapabilities::availableMimeTypes().filter( "video/", Qt::CaseInsensitive );
 
-    const QString mimeType = item.mimetype();
-    const bool valid = mimeTable.contains( mimeType, Qt::CaseInsensitive );
+    // Add whitelist hacks
+    mimeTable << "audio/x-m4b"; // MP4 Audio Books have a different extension that KFileItem/Phonon don't grok
 
-    //we special case this as otherwise users hate us
-    if ( !valid && ( mimeType == "audio/mp3" || mimeType == "audio/x-mp3" ) && !installDistroCodec() )
+    // We special case this, as otherwise the users would hate us
+    if( ( !mimeTable.contains( "audio/mp3" ) || !mimeTable.contains( "audio/x-mp3" ) ) && !installDistroCodec() )
+    {
         The::statusBar()->longMessage(
                 i18n( "<p>Phonon claims it <b>cannot</b> play MP3 files. You may want to examine "
                       "the installation of the backend that phonon uses.</p>"
                       "<p>You may find useful information in the <i>FAQ</i> section of the <i>Amarok Handbook</i>.</p>" ), StatusBar::Error );
+        mimeTable << "audio/mp3" << "audio/x-mp3";
+    }
 
-    return valid;
+    return mimeTable;
 }
 
 bool
@@ -241,8 +302,8 @@ EngineController::endSession()
     //only update song stats, when we're not going to resume it
     if ( !AmarokConfig::resumePlayback() && m_currentTrack )
     {
-        playbackEnded( trackPosition(), m_currentTrack->length(), EngineObserver::EndedQuit );
-        emit trackChanged( Meta::TrackPtr( 0 ) );
+        playbackEnded( trackPositionMs(), m_currentTrack->length(), EngineObserver::EndedQuit );
+        trackChangedNotify( Meta::TrackPtr( 0 ) );
     }
 }
 
@@ -255,13 +316,13 @@ EngineController::play() //SLOT
 {
     DEBUG_BLOCK
 
-    if( m_media->state() == Phonon::PlayingState )
+    // FIXME: what should we do in buffering state?
+    if( state() == Phonon::PlayingState )
         return;
 
-    if( m_fader )
-        m_fader->deleteLater();
+    resetFadeout();
 
-    if ( m_media->state() == Phonon::PausedState )
+    if ( state() == Phonon::PausedState )
     {
         m_media->play();
     }
@@ -280,11 +341,15 @@ EngineController::play( const Meta::TrackPtr& track, uint offset )
         return;
 
     m_currentTrack = track;
+    m_currentIsAudioCd = false;
+    delete m_boundedPlayback;
     delete m_multiPlayback;
     delete m_multiSource;
+    m_boundedPlayback = m_currentTrack->create<Meta::BoundedPlaybackCapability>();
     m_multiPlayback = m_currentTrack->create<Meta::MultiPlayableCapability>();
     m_multiSource  = m_currentTrack->create<Meta::MultiSourceCapability>();
 
+
     m_nextTrack.clear();
     m_nextUrl.clear();
     m_media->clearQueue();
@@ -304,8 +369,14 @@ EngineController::play( const Meta::TrackPtr& track, uint offset )
         connect( m_multiSource, SIGNAL( urlChanged( const KUrl & ) ), this, SLOT( slotPlayableUrlFetched( const KUrl & ) ) );
         playUrl( m_currentTrack->playableUrl(), 0 );
     }
+    else if ( m_boundedPlayback )
+    {
+        debug() << "Starting bounded playback of url " << m_currentTrack->playableUrl() << " at position " << m_boundedPlayback->startPosition();
+        playUrl( m_currentTrack->playableUrl(), m_boundedPlayback->startPosition() );
+    }
     else
     {
+        debug() << "Just a normal, boring track... :-P";
         playUrl( m_currentTrack->playableUrl(), offset );
     }
 }
@@ -315,10 +386,78 @@ EngineController::playUrl( const KUrl &url, uint offset )
 {
     DEBUG_BLOCK
 
-    slotStopFadeout();
+    m_media->stop();
+    resetFadeout();
 
     debug() << "URL: " << url.url();
-    m_media->setCurrentSource( url );
+    debug() << "offset: " << offset;
+
+    if ( url.url().startsWith( "audiocd:/" ) )
+    {
+
+        m_currentIsAudioCd = true;
+        //disconnect this signal for now or it will cause a loop that will cause a mutex lockup
+        disconnect( m_controller, SIGNAL( titleChanged( int ) ), this, SLOT( slotTitleChanged( int ) ) );
+
+        debug() << "play track from cd";
+        QString trackNumberString = url.url();
+        trackNumberString = trackNumberString.remove( "audiocd:/" );
+
+        QStringList parts = trackNumberString.split( '/' );
+
+        if ( parts.count() != 2 )
+            return;
+
+        QString discId = parts.at( 0 );
+
+        //we really only want to play it if it is the disc that is currently present.
+        //In the case of CDs for which we don't have any id, any "unknown" CDs will
+        //be considered equal.
+
+
+        //FIXME:
+        //if ( MediaDeviceMonitor::instance()->currentCdId() != discId )
+        //    return;
+
+
+        int trackNumber = parts.at( 1 ).toInt();
+
+        debug() << "3.2.1...";
+
+        Phonon::MediaSource::Type type = m_media->currentSource().type();
+        if( type != Phonon::MediaSource::Disc )
+        {
+            m_media->clear();
+            m_media->setCurrentSource( Phonon::Cd );
+        }
+
+        debug() << "boom?";
+        m_controller->setCurrentTitle( trackNumber );
+        debug() << "no boom?";
+
+        if( type == Phonon::MediaSource::Disc )
+        {
+            // The track has changed but the slot will not be called,
+            // because it's still the same media source, which means
+            // we need to do it explicitly.
+            slotNewTrackPlaying( Phonon::Cd );
+        }
+
+        //reconnect it
+        connect( m_controller, SIGNAL( titleChanged( int ) ), SLOT( slotTitleChanged( int ) ) );
+
+    }
+    else
+    {
+        if ( url.toLocalFile().isEmpty() )
+        {
+            m_media->setCurrentSource( url );
+        }
+        else
+        {
+            m_media->setCurrentSource( url.toLocalFile() );
+        }
+    }
 
     m_nextTrack.clear();
     m_nextUrl.clear();
@@ -326,10 +465,15 @@ EngineController::playUrl( const KUrl &url, uint offset )
 
     if( offset )
     {
+        debug() << "seeking to " << offset;
         m_media->pause();
         m_media->seek( offset );
     }
     m_media->play();
+
+    debug() << "track pos after play: " << trackPositionMs();
+
+
 }
 
 void
@@ -343,6 +487,7 @@ EngineController::stop( bool forceInstant ) //SLOT
 {
     DEBUG_BLOCK
 
+    m_currentIsAudioCd = false;
     // need to get a new instance of multi if played again
     delete m_multiPlayback;
     delete m_multiSource;
@@ -357,41 +502,44 @@ EngineController::stop( bool forceInstant ) //SLOT
     if( m_currentTrack )
     {
         debug() << "m_currentTrack != 0";
-        const int pos = trackPosition();
-        const int length = m_currentTrack->length();
+        const qint64 pos = trackPositionMs();
+        const qint64 length = m_currentTrack->length();
         m_currentTrack->finishedPlaying( double(pos)/double(length) );
         playbackEnded( pos, length, EngineObserver::EndedStopped );
-        emit trackChanged( Meta::TrackPtr( 0 ) );
+        trackChangedNotify( Meta::TrackPtr( 0 ) );
     }
 
-    // Stop instantly if fadeout is already running, or the media is paused (i.e. pressing Stop twice)
-    if( m_fader || m_media->state() == Phonon::PausedState )
+    // Stop instantly if fadeout is already running, or the media is not playing
+    if( m_fadeoutTimer->isActive() || m_media->state() != Phonon::PlayingState )
     {
         forceInstant = true;
     }
 
     if( AmarokConfig::fadeout() && AmarokConfig::fadeoutLength() && !forceInstant )
     {
-        stateChangedNotify( Phonon::StoppedState, Phonon::PlayingState ); //immediately disable Stop action
+        // WARNING: this can cause a gap in playback in GStreamer
+        if (! m_fader )
+            createFadeoutEffect();
 
-        m_fader = new Phonon::VolumeFaderEffect( this );
-        m_path.insertEffect( m_fader );
-        m_fader->setFadeCurve( Phonon::VolumeFaderEffect::Fade9Decibel );
         m_fader->fadeOut( AmarokConfig::fadeoutLength() );
 
         m_fadeoutTimer->start( AmarokConfig::fadeoutLength() + 1000 ); //add 1s for good measure, otherwise seems to cut off early (buffering..)
+
+        stateChangedNotify( Phonon::StoppedState, m_media->state() ); //immediately disable Stop action
     }
     else
+    {
         m_media->stop();
+        m_media->setCurrentSource( Phonon::MediaSource() );
+    }
 
-    emit trackFinished();
     m_currentTrack = 0;
 }
 
 bool
 EngineController::isPaused() const
 {
-    return m_media->state() == Phonon::PausedState;
+    return state() == Phonon::PausedState;
 }
 
 void
@@ -400,14 +548,19 @@ EngineController::playPause() //SLOT
     DEBUG_BLOCK
 
     //this is used by the TrayIcon, PlayPauseAction and DBus
-    debug() << "PlayPause: phonon state" << m_media->state();
+    debug() << "PlayPause: EngineController state" << state();
 
-    if( m_media->state() == Phonon::PausedState ||
-        m_media->state() == Phonon::StoppedState ||
-        m_media->state() == Phonon::LoadingState )
-        play();
-    else
-        pause();
+    switch ( state() )
+    {
+        case Phonon::PausedState:
+        case Phonon::StoppedState:
+        case Phonon::LoadingState:
+            play();
+            break;
+        default:
+            pause();
+            break;
+    }
 }
 
 void
@@ -417,9 +570,24 @@ EngineController::seek( int ms ) //SLOT
 
     if( m_media->isSeekable() )
     {
-        m_media->seek( static_cast<qint64>( ms ) );
-        trackPositionChangedNotify( ms, true ); /* User seek */
-        emit trackSeeked( ms );
+
+        debug() << "seek to: " << ms;
+        int seekTo;
+
+        if ( m_boundedPlayback )
+        {
+            seekTo = m_boundedPlayback->startPosition() + ms;
+            if( seekTo < m_boundedPlayback->startPosition() )
+                seekTo = m_boundedPlayback->startPosition();
+            else if( seekTo > m_boundedPlayback->startPosition() + trackLength() )
+                seekTo = m_boundedPlayback->startPosition() + trackLength();
+        }
+        else
+            seekTo = ms;
+
+        m_media->seek( static_cast<qint64>( seekTo ) );
+        // FIXME: is this correct for bounded playback?
+        trackPositionChangedNotify( seekTo, true ); /* User seek */
     }
     else
         debug() << "Stream is not seekable.";
@@ -460,17 +628,16 @@ EngineController::decreaseVolume( int ticks ) //SLOT
 int
 EngineController::setVolume( int percent ) //SLOT
 {
-    percent = qBound( 0, percent, 100 );
-    m_volume = percent; 
-    
-    // Phonon stays completely mute if the volume is lower than 0.05, so we shift and limit the range 
+    percent = qBound<qreal>( 0, percent, 100 );
+    m_volume = percent;
+
+    // Phonon stays completely mute if the volume is lower than 0.05, so we shift and limit the range
     qreal newVolume =  ( percent + 4 ) / 100.0;
-    newVolume = qBound( 0.04, newVolume, 1.0 );
+    newVolume = qBound<qreal>( 0.04, newVolume, 1.0 );
     m_audio->setVolume( newVolume );
-    
+
     AmarokConfig::setMasterVolume( percent );
     volumeChangedNotify( percent );
-    emit volumeChanged( percent );
 
     return percent;
 }
@@ -494,8 +661,6 @@ EngineController::setMuted( bool mute ) //SLOT
 
     AmarokConfig::setMuteState( mute );
     muteStateChangedNotify( mute );
-
-    emit muteStateChanged( mute );
 }
 
 void
@@ -510,7 +675,7 @@ EngineController::currentTrack() const
     return m_currentTrack;
 }
 
-int
+qint64
 EngineController::trackLength() const
 {
     const qint64 phononLength = m_media->totalTime(); //may return -1
@@ -518,13 +683,17 @@ EngineController::trackLength() const
     if( m_currentTrack && m_currentTrack->length() > 0 )   //When starting a last.fm stream, Phonon still shows the old track's length--trust Meta::Track over Phonon
         return m_currentTrack->length();
     else
-        return static_cast<int>( phononLength / 1000 );
+        return phononLength;
 }
 
 void
 EngineController::setNextTrack( Meta::TrackPtr track )
 {
+    DEBUG_BLOCK
+
+    debug() << "locking mutex";
     QMutexLocker locker( &m_mutex );
+    debug() << "locked!";
 
     if( !track )
         return;
@@ -533,8 +702,8 @@ EngineController::setNextTrack( Meta::TrackPtr track )
     if( track->playableUrl().isEmpty() )
         return;
 
-    if( m_media->state() == Phonon::PlayingState ||
-        m_media->state() == Phonon::BufferingState )
+    if( state() == Phonon::PlayingState ||
+        state() == Phonon::BufferingState )
     {
         m_media->clearQueue();
         if( track->playableUrl().isLocalFile() )
@@ -548,14 +717,13 @@ EngineController::setNextTrack( Meta::TrackPtr track )
     }
 }
 
-
-bool
-EngineController::getAudioCDContents(const QString &device, KUrl::List &urls)
+Phonon::State
+EngineController::state() const
 {
-    Q_UNUSED( device ); Q_UNUSED( urls );
-//since Phonon doesn't know anything about CD listings, there's actually no reason for this functionality to be here
-//kept to keep things compiling, probably should be in its own class.
-    return false;
+    if ( m_fadeoutTimer->isActive() )
+        return Phonon::StoppedState;
+    else
+        return phononMediaObject()->state();
 }
 
 bool
@@ -582,6 +750,105 @@ EngineController::trackPositionMs() const
     return m_media->currentTime();
 }
 
+bool
+EngineController::isEqSupported() const
+{
+    // If effect was created it means we have equalizer support
+    return ( !m_equalizer.isNull() );
+}
+
+double
+EngineController::eqMaxGain() const
+{
+   if( m_equalizer.isNull() )
+       return 100;
+   QList<Phonon::EffectParameter> mEqPar = m_equalizer->parameters();
+   if( mEqPar.isEmpty() )
+       return 100.0;
+   double mScale;
+   mScale = ( fabs(mEqPar.at(0).maximumValue().toDouble() ) +  fabs( mEqPar.at(0).minimumValue().toDouble() ) );
+   mScale /= 2.0;
+   return mScale;
+}
+
+QStringList
+EngineController::eqBandsFreq() const
+{
+    // This will extract the bands frequency values from effect parameter name
+    // as long as they follow the rules:
+    // eq-preamp parameter will contain 'pre-amp' string
+    // bands parameters are described using schema 'xxxHz'
+    QStringList mBandsFreq;
+    if( m_equalizer.isNull() )
+       return mBandsFreq;
+    QList<Phonon::EffectParameter> mEqPar = m_equalizer->parameters();
+    if( mEqPar.isEmpty() )
+       return mBandsFreq;
+    QRegExp rx( "\\d+(?=Hz)" );
+    foreach( const Phonon::EffectParameter &mParam, mEqPar )
+    {
+        if( mParam.name().contains( QString( "pre-amp" ) ) )
+        {
+            mBandsFreq << i18n( "Preamp" );
+        }
+        else if ( mParam.name().contains( rx ) )
+        {
+            if( rx.cap( 0 ).toInt() < 1000 )
+            {
+                mBandsFreq << QString( rx.cap( 0 )).append( "\nHz" );
+            }
+            else
+            {
+                mBandsFreq << QString::number( rx.cap( 0 ).toInt()/1000 ).append( "\nkHz" );
+            }
+        }
+    }
+    return mBandsFreq;
+}
+
+void
+EngineController::eqUpdate() //SLOT
+{
+    // if equalizer not present simply return
+    if( m_equalizer.isNull() )
+        return;
+    // check if equalizer should be disabled ??
+    if( AmarokConfig::equalizerMode() <= 0 )
+    {
+        // Remove effect from path
+        if( m_path.effects().indexOf( m_equalizer ) != -1 )
+            m_path.removeEffect( m_equalizer );
+    }
+    else
+    {
+        // Set equalizer parameter according to the gains from settings
+        QList<Phonon::EffectParameter> mEqPar = m_equalizer->parameters();
+        QList<int> mEqParCfg = AmarokConfig::equalizerGains();
+
+        QListIterator<int> mEqParNewIt( mEqParCfg );
+        double scaledVal; // Scaled value to set from universal -100 - 100 range to plugin scale
+        foreach( const Phonon::EffectParameter &mParam, mEqPar )
+        {
+            scaledVal = mEqParNewIt.hasNext() ? mEqParNewIt.next() : 0;
+            scaledVal *= ( fabs(mParam.maximumValue().toDouble() ) +  fabs( mParam.minimumValue().toDouble() ) );
+            scaledVal /= 200.0;
+            m_equalizer->setParameterValue( mParam, scaledVal );
+        }
+        // Insert effect into path if needed
+        if( m_path.effects().indexOf( m_equalizer ) == -1 )
+        {
+            if( !m_path.effects().isEmpty() )
+            {
+                m_path.insertEffect( m_equalizer, m_path.effects().first() );
+            }
+            else
+            {
+                m_path.insertEffect( m_equalizer );
+            }
+        }
+    }
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////
 // PRIVATE SLOTS
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -589,15 +856,28 @@ EngineController::trackPositionMs() const
 void
 EngineController::slotTick( qint64 position )
 {
-    trackPositionChangedNotify( static_cast<long>( position ), false ); //it expects milliseconds
-}
+    if ( m_boundedPlayback )
+    {
+        trackPositionChangedNotify( static_cast<long>( position - m_boundedPlayback->startPosition() ), false );
 
+        //don't go beyond the stop point
+        if ( position >= m_boundedPlayback->endPosition() )
+        {
+            slotAboutToFinish();
+        }
+    }
+    else
+    {
+        trackPositionChangedNotify( static_cast<long>( position ), false ); //it expects milliseconds
+    }
+}
 
 void
 EngineController::slotAboutToFinish()
 {
     DEBUG_BLOCK
     debug() << "Track finished completely, updating statistics";
+
     if( m_currentTrack ) // not sure why this should not be the case, but sometimes happens. don't crash.
         m_currentTrack->finishedPlaying( 1.0 ); // If we reach aboutToFinish, the track is done as far as we are concerned.
     if( m_multiPlayback )
@@ -627,7 +907,23 @@ EngineController::slotAboutToFinish()
             The::playlistActions()->requestNextTrack();
             debug() << "no more sources, skip to next track";
         }
-         
+    }
+    else if ( m_boundedPlayback )
+    {
+        debug() << "finished a track that consists of part of another track, go to next track even if this url is technically not done yet";
+
+        //stop this track, now, as the source track might go on and on, and
+        //there might not be any more tracks in the playlist...
+        stop( true );
+        The::playlistActions()->requestNextTrack();
+        slotQueueEnded();
+    }
+    else if ( m_currentTrack && m_currentTrack->playableUrl().url().startsWith( "audiocd:/" ) )
+    {
+        debug() << "finished a CD track, don't care if queue is not empty, just get new track...";
+
+        The::playlistActions()->requestNextTrack();
+        slotQueueEnded();
     }
     else if( m_media->queue().isEmpty() )
         The::playlistActions()->requestNextTrack();
@@ -640,10 +936,10 @@ EngineController::slotQueueEnded()
 
     if( m_currentTrack && !m_multiPlayback && !m_multiSource )
     {
-        m_currentTrack->finishedPlaying( 1.0 );
-        emit trackFinished();
-        playbackEnded( trackPosition(), m_currentTrack->length(), EngineObserver::EndedStopped );
+        m_media->setCurrentSource( Phonon::MediaSource() );
+        playbackEnded( trackPositionMs(), m_currentTrack->length(), EngineObserver::EndedStopped );
         m_currentTrack = 0;
+        trackChangedNotify( m_currentTrack );
     }
 
     m_mutex.lock(); // in case setNextTrack is being handled right now.
@@ -672,7 +968,12 @@ void
 EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
 {
     DEBUG_BLOCK
-    Q_UNUSED( source );
+
+    if( source.type() == Phonon::MediaSource::Empty )
+    {
+        debug() << "Empty MediaSource (engine stop)";
+        return;
+    }
 
     // the new track was taken from the queue, so clear these fields
     if( m_nextTrack )
@@ -691,7 +992,7 @@ EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
             m_preamp = new Phonon::VolumeFaderEffect( this );
             m_path.insertEffect( m_preamp );
         }
-        
+
         Meta::Track::ReplayGainMode mode = ( AmarokConfig::replayGainMode() == AmarokConfig::EnumReplayGainMode::Track)
                                          ? Meta::Track::TrackReplayGain
                                          : Meta::Track::AlbumReplayGain;
@@ -708,14 +1009,15 @@ EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
         // we calculate the volume change ourselves, because m_preamp->setVolumeDecibel is
         // a little confused about minus signs
         m_preamp->setVolume( exp( gain * log10over20 ) );
+        m_preamp->fadeTo( exp( gain * log10over20 ), 0 ); // HACK: we use fadeTo because setVolume is b0rked in Phonon Xine before r1028879
     }
     else if( m_preamp )
-        m_preamp->setVolumeDecibel( 0.0 );
+    {
+        m_preamp->setVolume( 1.0 );
+        m_preamp->fadeTo( 1.0, 0 ); // HACK: we use fadeTo because setVolume is b0rked in Phonon Xine before r1028879
+    }
 
-    // state never changes if tracks are queued, but we need this to update the caption
-    stateChangedNotify( m_media->state(), m_media->state() );
-
-    emit trackChanged( m_currentTrack );
+    trackChangedNotify( m_currentTrack );
     newTrackPlaying();
 }
 
@@ -724,16 +1026,13 @@ EngineController::slotStateChanged( Phonon::State newState, Phonon::State oldSta
 {
     DEBUG_BLOCK
 
-    // Sanity checks:
-    if( newState == oldState )
-        return;
-
+    // HACK:
     // The following check is an attempt to fix http://bugs.kde.org/show_bug.cgi?id=180339
     // ("amarok stops playing tracks") and other issues with Phonon.
     // The theory:
     // It has been observed that Phonon will sometimes emit a stateChanged() with
     // _both_ oldState and newState == 0, which makes little sense. After that it goes
-    // berserk, until you restart Amarok. 
+    // berserk, until you restart Amarok.
     // Now we try to detect this weird state, and then try to destroy and recreate all
     // Phonon objects, in the hope of fixing the situation. Fingers crossed.
     if( newState == Phonon::LoadingState && oldState == Phonon::LoadingState )
@@ -743,6 +1042,10 @@ EngineController::slotStateChanged( Phonon::State newState, Phonon::State oldSta
         newState = Phonon::ErrorState;  // Indicate error
     }
 
+    // Sanity checks:
+    if( newState == oldState )
+        return;
+
     if( newState == Phonon::ErrorState )  // If media is borked, skip to next track
     {
         warning() << "Phonon failed to play this URL. Error: " << m_media->errorString();
@@ -750,7 +1053,7 @@ EngineController::slotStateChanged( Phonon::State newState, Phonon::State oldSta
         {
             DEBUG_LINE_INFO
             m_mutex.lock();
-            m_playWhenFetched = false;
+            m_playWhenFetched = true;
             m_mutex.unlock();
             m_multiPlayback->fetchNext();
             debug() << "The queue has: " << m_media->queue().size() << " tracks in it";
@@ -776,12 +1079,17 @@ EngineController::slotStateChanged( Phonon::State newState, Phonon::State oldSta
             The::playlistActions()->requestNextTrack();
     }
 
-    stateChangedNotify( newState, oldState );
+    if ( m_fadeoutTimer->isActive() )
+    {
+        // We've stopped already as far as the rest of Amarok is concerned
+        if ( oldState == Phonon::PlayingState )
+            oldState = Phonon::StoppedState;
 
-    if( newState == Phonon::PlayingState )
-        emit trackPlayPause( Playing );
-    else if( newState == Phonon::PausedState )
-        emit trackPlayPause( Paused );
+        if ( oldState == newState )
+            return;
+    }
+
+    stateChangedNotify( newState, oldState );
 }
 
 void
@@ -824,11 +1132,7 @@ EngineController::slotTrackLengthChanged( qint64 milliseconds )
 {
     DEBUG_BLOCK
 
-    //Phonon reports bad info on streams, so call trackLength() to verify
-    milliseconds = trackLength() * 1000;
-
-    if( milliseconds != 0 ) //don't notify for 0 seconds, it's probably just a stream
-        trackLengthChangedNotify( static_cast<long>( milliseconds ) / 1000 );
+    trackLengthChangedNotify( ( m_multiPlayback || m_boundedPlayback ) ? trackLength() : milliseconds );
 }
 
 void
@@ -885,14 +1189,33 @@ EngineController::slotStopFadeout() //SLOT
 {
     DEBUG_BLOCK
 
-    // Make sure the timer won't call this method again
-    m_fadeoutTimer->stop();
+    m_media->stop();
+    m_media->setCurrentSource( Phonon::MediaSource() );
+    resetFadeout();
+}
 
-    if ( m_fader ) {
-        m_fader->deleteLater();
-        m_media->stop();
+void
+EngineController::resetFadeout()
+{
+    m_fadeoutTimer->stop();
+    if ( m_fader )
+    {
+        m_fader->setVolume( 1.0 );
+        m_fader->fadeTo( 1.0, 0 ); // HACK: we use fadeTo because setVolume is b0rked in Phonon Xine before r1028879
     }
 }
 
-#include "EngineController.moc"
+void EngineController::slotTitleChanged( int titleNumber )
+{
+    DEBUG_BLOCK
+    Q_UNUSED( titleNumber );
 
+    slotAboutToFinish();
+}
+
+bool EngineController::isPlayingAudioCd()
+{
+    return m_currentIsAudioCd;
+}
+
+#include "EngineController.moc"
