@@ -1,21 +1,20 @@
-/******************************************************************************
- * Copyright (c) 2007 Alexandre Pereira de Oliveira <aleprj@gmail.com>        *
- *           (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>  *
- *           (c) 2007 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>          *
- *                                                                            *
- * This program is free software; you can redistribute it and/or              *
- * modify it under the terms of the GNU General Public License as             *
- * published by the Free Software Foundation; either version 2 of             *
- * the License, or (at your option) any later version.                        *
- *                                                                            *
- * This program is distributed in the hope that it will be useful,            *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
- * GNU General Public License for more details.                               *
- *                                                                            *
- * You should have received a copy of the GNU General Public License          *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.      *
- ******************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2007 Alexandre Pereira de Oliveira <aleprj@gmail.com>                  *
+ * Copyright (c) 2007-2009 Maximilian Kossick <maximilian.kossick@googlemail.com>       *
+ * Copyright (c) 2007 Nikolaj Hald Nielsen <nhn@kde.org>                                *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #define DEBUG_PREFIX "CollectionTreeItemModelBase"
 
@@ -24,9 +23,7 @@
 #include "Amarok.h"
 #include "AmarokMimeData.h"
 #include "Collection.h"
-#include "CollectionManager.h"
 #include "CollectionTreeItem.h"
-#include "CollectionTreeView.h"
 #include "Debug.h"
 #include "Expression.h"
 #include "MetaConstants.h"
@@ -40,6 +37,7 @@
 #include <KStandardDirs>
 #include <QPixmap>
 #include <QTimeLine>
+#include <QTimer>
 
 using namespace Meta;
 
@@ -72,11 +70,11 @@ CollectionTreeItemModelBase::~CollectionTreeItemModelBase()
 
 Qt::ItemFlags CollectionTreeItemModelBase::flags(const QModelIndex & index) const
 {
-    if ( !index.isValid() )
-        return Qt::ItemIsEnabled;
-
-    Qt::ItemFlags flags;
-    flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEditable;
+    Qt::ItemFlags flags = 0;
+    if( index.isValid() )
+    {
+        flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEditable;
+    }
     return flags;
 
 }
@@ -105,19 +103,6 @@ CollectionTreeItemModelBase::setData( const QModelIndex &index, const QVariant &
             return true;
         }
     }
-    /*else
-    {
-        QModelIndex in = index;
-        while( in.model()->hasChildren( in ) )
-        {
-            int rows = in.model()->rowCount( in );
-            for( int i = 0; i < rows; i++ )
-            {
-                setData( in.child( i, 1 ), value, role );
-                in = in.sibling( in.row() + 1, in.column() );
-            }
-        }
-    }*/
     else if( Meta::AlbumPtr album = Meta::AlbumPtr::dynamicCast( data ) )
     {
         Meta::TrackList tracks = album->tracks();
@@ -215,6 +200,11 @@ CollectionTreeItemModelBase::headerData(int section, Qt::Orientation orientation
 QModelIndex
 CollectionTreeItemModelBase::index(int row, int column, const QModelIndex & parent) const
 {
+    //ensure sanity of parameters
+    //we are a tree model, there are no columns
+    if( row < 0 || column != 0 )
+        return QModelIndex();
+
     CollectionTreeItem *parentItem;
 
     if (!parent.isValid())
@@ -222,16 +212,11 @@ CollectionTreeItemModelBase::index(int row, int column, const QModelIndex & pare
     else
         parentItem = static_cast<CollectionTreeItem*>(parent.internalPointer());
 
-   //if ( parentItem->childrenLoaded() )
-   //{
-        CollectionTreeItem *childItem = parentItem->child(row);
-        if( childItem )
-            return createIndex(row, column, childItem);
-        else
-            return QModelIndex();
-    //}
-    //else
-    //   return QModelIndex();
+    CollectionTreeItem *childItem = parentItem->child(row);
+    if( childItem )
+        return createIndex(row, column, childItem);
+    else
+        return QModelIndex();
 }
 
 QModelIndex
@@ -259,9 +244,7 @@ CollectionTreeItemModelBase::rowCount(const QModelIndex & parent) const
     else
         parentItem = static_cast<CollectionTreeItem*>(parent.internalPointer());
 
-    if( parentItem->childrenLoaded() )
-        return parentItem->childCount();
-    return 0;
+    return parentItem->childCount();
 }
 
 int CollectionTreeItemModelBase::columnCount(const QModelIndex & parent) const
@@ -335,6 +318,28 @@ CollectionTreeItemModelBase::mimeData(const QList<CollectionTreeItem*> & items)
     return mimeData;
 }
 
+bool
+CollectionTreeItemModelBase::hasChildren ( const QModelIndex & parent ) const
+{
+     if( !parent.isValid() )
+         return true; // must be root item!
+
+    CollectionTreeItem *item = static_cast<CollectionTreeItem*>(parent.internalPointer());
+    //we added the collection level so we have to be careful with the item level
+    return !item->isDataItem() || item->level() + levelModifier() <= m_levelType.count();
+
+}
+
+void
+CollectionTreeItemModelBase::ensureChildrenLoaded( CollectionTreeItem *item )
+{
+    //only start a query if necessary and we are not querying for the item's children already
+    if ( item->requiresUpdate() && !d->m_runningQueries.contains( item ) )
+    {
+        listForLevel( item->level() + levelModifier(), item->queryMaker(), item );
+    }
+}
+
 QPixmap
 CollectionTreeItemModelBase::iconForLevel(int level) const
 {
@@ -361,20 +366,20 @@ CollectionTreeItemModelBase::iconForLevel(int level) const
     return KIconLoader::global()->loadIcon( icon, KIconLoader::Toolbar, KIconLoader::SizeSmall );
 }
 
-void CollectionTreeItemModelBase::listForLevel(int level, QueryMaker * qm, CollectionTreeItem * parent) const
+void CollectionTreeItemModelBase::listForLevel(int level, QueryMaker * qm, CollectionTreeItem * parent)
 {
-    //DEBUG_BLOCK
     if ( qm && parent )
     {
         //this check should not hurt anyone... needs to check if single... needs it
-        for( QMapIterator<QueryMaker*, CollectionTreeItem*> iter( d->m_childQueries ); iter.hasNext(); )
-        {
-            if( iter.next().value() == parent )
-                return;             //we are already querying for children of parent
-        }
+        if( d->m_runningQueries.contains( parent ) )
+            return;
+
         if ( level > m_levelType.count() )
             return;
 
+        if( parent->isVariousArtistItem() )
+            return;
+
         if ( level == m_levelType.count() )
             qm->setQueryType( QueryMaker::Track );
         else
@@ -385,7 +390,7 @@ void CollectionTreeItemModelBase::listForLevel(int level, QueryMaker * qm, Colle
                     qm->setQueryType( QueryMaker::Album );
                     //restrict query to normal albums if the previous level
                     //was the artist category. in that case we handle compilations below
-                    if( level > 0 && m_levelType[level-1] == CategoryId::Artist )
+                    if( level > 0 && m_levelType[level-1] == CategoryId::Artist && !parent->isVariousArtistItem() )
                     {
                         qm->setAlbumQueryMode( QueryMaker::OnlyNormalAlbums );
                     }
@@ -415,8 +420,12 @@ void CollectionTreeItemModelBase::listForLevel(int level, QueryMaker * qm, Colle
         CollectionTreeItem *tmpItem = parent;
         while( tmpItem->isDataItem() )
         {
-            //ignore Various artists node (whichh will not have a data pointer
-            if( tmpItem->data() )
+            //ignore Various artists node (which will not have a data pointer)
+            if( tmpItem->isVariousArtistItem() )
+            {
+                qm->setAlbumQueryMode( QueryMaker::OnlyCompilations );
+            }
+            else
                 qm->addMatch( tmpItem->data() );
 
             tmpItem = tmpItem->parent();
@@ -426,11 +435,12 @@ void CollectionTreeItemModelBase::listForLevel(int level, QueryMaker * qm, Colle
         connect( qm, SIGNAL( newResultReady( QString, Meta::DataList ) ), SLOT( newResultReady( QString, Meta::DataList ) ), Qt::QueuedConnection );
         connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
         d->m_childQueries.insert( qm, parent );
+        d->m_runningQueries.insert( parent, qm );
         qm->run();
 
-        //start animation
-        if( ( m_timeLine->state() != QTimeLine::Running ) && ( parent != m_rootItem ) )
-            m_timeLine->start();
+        //some very quick queries may be dode so fast that the loading
+        //animation creates an unnecessary flicker, therfore delay it for a bit
+        QTimer::singleShot( 150, this, SLOT( startAnimationTick() ) );
     }
 }
 
@@ -460,7 +470,7 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                 qm->beginAnd();
             else
                 qm->beginOr();
-            debug() << "field:" << elem.field << "negate?:" << elem.negate << "text:" << elem.text;
+
             if ( elem.field.isEmpty() )
             {
                 foreach ( int level, m_levelType )
@@ -503,6 +513,12 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                     ADD_OR_EXCLUDE_FILTER( Meta::valTitle, elem.text, false, false ); //always filter for track title too
                     qm->endAndOr();
                 }
+                if ( ( validFilters & QueryMaker::UrlFilter ) != 0 ) {
+                    qm->beginOr();
+                    ADD_OR_EXCLUDE_FILTER( Meta::valUrl, elem.text, false, false ); //always filter for track URL
+                    qm->endAndOr();
+                }
+
             }
             else
             {
@@ -537,7 +553,12 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                     if ( ( validFilters & QueryMaker::GenreFilter ) == 0 ) continue;
                     ADD_OR_EXCLUDE_FILTER( Meta::valGenre, elem.text, false, false );
                 }
-                else if ( lcField.compare( "composer", Qt::CaseInsensitive ) == 0|| lcField.compare( i18n( "composer" ), Qt::CaseInsensitive ) == 0 )
+                else if ( lcField.compare( "title", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "title" ), Qt::CaseInsensitive ) == 0 )
+                {
+                    if ( ( validFilters & QueryMaker::TitleFilter ) == 0 ) continue;
+                    ADD_OR_EXCLUDE_FILTER( Meta::valTitle, elem.text, false, false );
+                }
+                else if ( lcField.compare( "composer", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "composer" ), Qt::CaseInsensitive ) == 0 )
                 {
                     if ( ( validFilters & QueryMaker::ComposerFilter ) == 0 ) continue;
                     ADD_OR_EXCLUDE_FILTER( Meta::valComposer, elem.text, false, false );
@@ -545,12 +566,20 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                 else if ( lcField.compare( "year", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "year" ), Qt::CaseInsensitive ) == 0)
                 {
                     if ( ( validFilters & QueryMaker::YearFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valYear, elem.text, false, false );
+                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valYear, elem.text.toInt(), compare );
+                }
+                else if ( lcField.compare( "bpm", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "bpm" ), Qt::CaseInsensitive ) == 0)
+                {
+                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valBpm, elem.text.toInt(), compare );
                 }
                 else if( lcField.compare( "comment", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "comment" ), Qt::CaseInsensitive ) == 0 )
                 {
                     ADD_OR_EXCLUDE_FILTER( Meta::valYear, elem.text, false, false );
                 }
+                else if( lcField.compare( "bitrate", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "bitrate" ), Qt::CaseInsensitive ) == 0 )
+                {
+                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valBitrate, elem.text.toInt(), compare );
+                }
                 else if( lcField.compare( "rating", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "rating" ), Qt::CaseInsensitive ) == 0 )
                 {
                     ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valRating, elem.text.toInt(), compare );
@@ -563,6 +592,10 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                 {
                     ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valPlaycount, elem.text.toInt(), compare );
                 }
+                else if( lcField.compare( "samplerate", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "samplerate" ), Qt::CaseInsensitive ) == 0 )
+                {
+                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valSamplerate, elem.text.toInt(), compare );
+                }
                 else if( lcField.compare( "length", Qt::CaseInsensitive ) == 0 || lcField.compare( i18n( "length" ), Qt::CaseInsensitive ) == 0 )
                 {
                     ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valLength, elem.text.toInt(), compare );
@@ -608,15 +641,15 @@ CollectionTreeItemModelBase::addFilters( QueryMaker * qm ) const
                                 tmp += c;
                             else if( c == 'm' )
                             {
-                                months = 0 - QString( tmp ).toInt();
-                                tmp = "";
+                                months = 0 - tmp.toInt();
+                                tmp.clear();
                             } else if( c == 'w' )
                             {
-                                weeks = 0 - 7 * QString( tmp ).toInt();
-                                tmp = "";
+                                weeks = 0 - 7 * tmp.toInt();
+                                tmp.clear();
                             } else if( c == 'd' )
                             {
-                                days = 0 - QString( tmp ).toInt();
+                                days = 0 - tmp.toInt();
                                 break;
                             }
                         }
@@ -641,14 +674,18 @@ CollectionTreeItemModelBase::queryDone()
 
     CollectionTreeItem* item = d->m_childQueries.contains( qm ) ? d->m_childQueries.take( qm ) : d->m_compilationQueries.take( qm );
 
+    d->m_runningQueries.remove( item );
+
     //reset icon for this item
     if( item && item != m_rootItem )
-        emit ( dataChanged ( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) ) );
+    {
+        emit dataChanged( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) );
+        emit queryFinished();
+    }
 
     //stop timer if there are no more animations active
-    if( d->m_childQueries.count() == 0 /*&& d->m_compilationQueries.count() == 0 */ )
+    if( d->m_runningQueries.count() == 0 )
         m_timeLine->stop();
-    qm->deleteLater();
 }
 
 void
@@ -656,9 +693,6 @@ CollectionTreeItemModelBase::newResultReady(const QString & collectionId, Meta::
 {
     Q_UNUSED( collectionId )
 
-    if ( data.count() == 0 )
-        return;
-
     //if we are expanding an item, we'll find the sender in m_childQueries
     //otherwise we are filtering all collections
     QueryMaker *qm = qobject_cast<QueryMaker*>( sender() );
@@ -667,82 +701,215 @@ CollectionTreeItemModelBase::newResultReady(const QString & collectionId, Meta::
 
     if( d->m_childQueries.contains( qm ) )
     {
-        CollectionTreeItem *parent = d->m_childQueries.value( qm );
-        QModelIndex parentIndex;
-        if( parent ) {
-            if( parent == m_rootItem ) // will never happen in CollectionTreeItemModel
-                parentIndex = QModelIndex();
-            else
-                parentIndex = createIndex( parent->row(), 0, parent );
-
-            //add new rows after existing ones here (which means all artists nodes
-            //will be inserted after the "Various Artists" node
-            beginInsertRows( parentIndex, parent->childCount(), parent->childCount() + data.count()-1 );
-            populateChildren( data, parent );
-            endInsertRows();
-
-            for( int count = parent->childCount(), i = 0; i < count; ++i )
-            {
-                CollectionTreeItem *item = parent->child( i );
-                if ( m_expandedItems.contains( item->data() ) ) //item will always be a data item
-                {
-                    listForLevel( item->level(), item->queryMaker(), item );
-                }
-            }
-
-            if ( parent->isDataItem() )
-            {
-                if ( m_expandedItems.contains( parent->data() ) )
-                    emit expandIndex( parentIndex );
-                else
-                    //simply insert the item, nothing will change if it is already in the set
-                    m_expandedItems.insert( parent->data() );
-            }
-            else
-            {
-                m_expandedCollections.insert( parent->parentCollection() );
-            }
-        }
+        handleNormalQueryResult( qm, data );
     }
     else if( d->m_compilationQueries.contains( qm ) )
     {
-        CollectionTreeItem *parent = d->m_compilationQueries.value( qm );
-        QModelIndex parentIndex;
-        if( parent ) {
-            if (parent == m_rootItem ) // will never happen in CollectionTreeItemModel
+        handleCompilationQueryResult( qm, data );
+    }
+}
+
+void
+CollectionTreeItemModelBase::handleCompilationQueryResult( QueryMaker *qm, const Meta::DataList &dataList )
+{
+    CollectionTreeItem *parent = d->m_compilationQueries.value( qm );
+    QModelIndex parentIndex;
+    if( parent )
+    {
+        if (parent == m_rootItem ) // will never happen in CollectionTreeItemModel
+            parentIndex = QModelIndex();
+        else
+            parentIndex = createIndex( parent->row(), 0, parent );
+
+        //if the compilation query did not return a result we have to remove the
+        //the various artists node itself
+        if( dataList.isEmpty() )
+        {
+            for( int i = 0; i < parent->childCount(); i++ )
             {
-                parentIndex = QModelIndex();
+                CollectionTreeItem *cti = parent->child( i );
+                if( cti->isVariousArtistItem() )
+                {
+                    //found the various artists node
+                    beginRemoveRows( parentIndex, cti->row(), cti->row() );
+                    cti = 0; //will be deleted;
+                    parent->removeChild( i );
+                    endRemoveRows();
+                    break;
+                }
+            }
+            //we have removed the VA node if it existed
+            return;
+        }
+
+        CollectionTreeItem *vaNode = 0;
+        if( parent->childCount() == 0 )
+        {
+            //we only insert the "Various Artists" node
+            beginInsertRows( parentIndex, 0, 0 );
+            vaNode = new CollectionTreeItem( dataList, parent, this );
+            //set requiresUpdate, otherwise we will query for the children of vaNode again!
+            vaNode->setRequiresUpdate( false );
+            endInsertRows();
+        }
+        else
+        {
+            for( int i = 0; i < parent->childCount(); i++ )
+            {
+                CollectionTreeItem *cti = parent->child( i );
+                if( cti->isVariousArtistItem() )
+                {
+                    //found the various artists node
+                    vaNode = cti;
+                    break;
+                }
+            }
+            if( !vaNode )
+            {
+                //we only insert the "Various Artists" node
+                beginInsertRows( parentIndex, 0, 0 );
+                vaNode = new CollectionTreeItem( dataList, parent, this );
+                //set requiresUpdate, otherwise we will query for the children of vaNode again!
+                vaNode->setRequiresUpdate( false );
+                endInsertRows();
             }
             else
             {
-                parentIndex = createIndex( parent->row(), 0, parent );
+                //only call populateChildren for the VA node if we have not
+                //created it in this method call. The VA ndoe ctor takes care
+                //of that itself
+                populateChildren( dataList, vaNode, createIndex( vaNode->row(), 0, vaNode ) );
             }
-            //we only insert the "Various Artists" node
-            beginInsertRows( parentIndex, 0, 0 );
-            CollectionTreeItem *vaItem = new CollectionTreeItem( data, parent );
-            endInsertRows();
+            //populate children will call setRequiresUpdate on vaNode
+            //but as the VA query is based on vaNode's parent,
+            //we have to call setRequiresUpdate on the parent too
+            //yes, this will mean we will call setRequiresUpdate twice
+            parent->setRequiresUpdate( false );
 
+            for( int count = vaNode->childCount(), i = 0; i < count; ++i )
+            {
+                CollectionTreeItem *item = vaNode->child( i );
+                if ( m_expandedItems.contains( item->data() ) ) //item will always be a data item
+                {
+                    listForLevel( item->level() + levelModifier(), item->queryMaker(), item );
+                }
+            }
+        }
+
+        //if the va node exists, check if it has to be expanded
+        if(vaNode)
+        {
             CollectionTreeItem *tmp = parent;
             while( tmp->isDataItem() )
                 tmp = tmp->parent();
 
             if( m_expandedVariousArtistsNodes.contains( tmp->parentCollection() ) )
             {
-                debug() << "Expand vanode for collection " << collectionId;
-                QModelIndex vanode = createIndex( 0, 0, vaItem ); //we've just inserted the vaItem at row 0
-                emit expandIndex( vanode );
+                emit expandIndex( createIndex( 0, 0, vaNode ) ); //we have just inserted the vaItem at row 0
             }
-
         }
     }
 }
 
 void
-CollectionTreeItemModelBase::populateChildren(const DataList & dataList, CollectionTreeItem * parent) const
+CollectionTreeItemModelBase::handleNormalQueryResult( QueryMaker *qm, const Meta::DataList &dataList )
 {
-    foreach( Meta::DataPtr data, dataList )
-        new CollectionTreeItem( data, parent );
-    parent->setChildrenLoaded( true );
+    CollectionTreeItem *parent = d->m_childQueries.value( qm );
+    QModelIndex parentIndex;
+    if( parent ) {
+        if( parent == m_rootItem ) // will never happen in CollectionTreeItemModel, but will happen in Single!
+            parentIndex = QModelIndex();
+        else
+            parentIndex = createIndex( parent->row(), 0, parent );
+
+        populateChildren( dataList, parent, parentIndex );
+
+        if ( parent->isDataItem() )
+        {
+            if ( m_expandedItems.contains( parent->data() ) )
+                emit expandIndex( parentIndex );
+            else
+                //simply insert the item, nothing will change if it is already in the set
+                m_expandedItems.insert( parent->data() );
+        }
+        else
+        {
+            m_expandedCollections.insert( parent->parentCollection() );
+        }
+    }
+}
+
+void
+CollectionTreeItemModelBase::populateChildren(const DataList & dataList, CollectionTreeItem * parent, const QModelIndex &parentIndex )
+{
+    //add new rows after existing ones here (which means all artists nodes
+    //will be inserted after the "Various Artists" node)
+    {
+        //figure out which children of parent have to be removed,
+        //which new children have to be added, and preemptively emit dataChanged for the rest
+        //have to check how that influences performance...
+        QHash<Meta::DataPtr, int> dataToIndex;
+        QSet<Meta::DataPtr> childrenSet;
+        foreach( CollectionTreeItem *child, parent->children() )
+        {
+            if( child->isVariousArtistItem() )
+                continue;
+            childrenSet.insert( child->data() );
+            dataToIndex.insert( child->data(), child->row() );
+        }
+        QSet<Meta::DataPtr> dataSet = dataList.toSet();
+        QSet<Meta::DataPtr> dataToBeAdded = dataSet - childrenSet;
+        QSet<Meta::DataPtr> dataToBeRemoved = childrenSet - dataSet;
+
+        QList<int> currentIndices;
+        //first remove all rows that have to be removed
+        //walking through the cildren in reverse order does not screw up the order
+        for( int i = parent->childCount() - 1; i >= 0; i-- )
+        {
+            CollectionTreeItem *child = parent->child( i );
+            bool toBeRemoved = child->isDataItem() && !child->isVariousArtistItem() && dataToBeRemoved.contains( child->data() ) ;
+            if( toBeRemoved )
+            {
+                currentIndices.append( i );
+            }
+            //make sure we remove the rows if the first row (we are using reverse order) has to be removed too!
+            if( ( !toBeRemoved || i == 0 ) && !currentIndices.isEmpty() )
+            {
+                //be careful in which order you insert the rows above!!
+                beginRemoveRows( parentIndex, currentIndices.last(), currentIndices.first() );
+                foreach( int i, currentIndices )
+                {
+                    parent->removeChild( i );
+                }
+                endRemoveRows();
+                currentIndices.clear();
+            }
+        }
+        //hopefully the view has figured out that we've removed rows yet!
+        int lastRow = parent->childCount() - 1;
+        if( lastRow >= 0 )
+        {
+            emit dataChanged( createIndex( 0, 0, parent->child( 0 ) ), createIndex( lastRow, 0, parent->child( lastRow ) ) );
+        }
+        //the remainging child items may be dirty, so refresh them
+        foreach( CollectionTreeItem *item, parent->children() )
+        {
+            if( item->isDataItem() && item->data() && m_expandedItems.contains( item->data() ) )
+                ensureChildrenLoaded( item );
+        }
+        //add the new rows
+        if( !dataToBeAdded.isEmpty() )
+        {
+            //the above check ensures that Qt does not crash on beginInsertRows ( because lastRow+1 > lastRow+0)
+            beginInsertRows( parentIndex, lastRow + 1, lastRow + dataToBeAdded.count() );
+            foreach( Meta::DataPtr data, dataToBeAdded )
+            {
+                new CollectionTreeItem( data, parent, this );
+            }
+            endInsertRows();
+        }
+        parent->setRequiresUpdate( false );
+    }
 }
 
 void
@@ -783,7 +950,7 @@ CollectionTreeItemModelBase::handleCompilations( CollectionTreeItem *parent ) co
     while( tmpItem->isDataItem()  )
     {
         //ignore Various artists node (which will not have a data pointer)
-        if( tmpItem->data() )
+        if( !tmpItem->isVariousArtistItem() )
             qm->addMatch( tmpItem->data() );
         tmpItem = tmpItem->parent();
     }
@@ -792,9 +959,17 @@ CollectionTreeItemModelBase::handleCompilations( CollectionTreeItem *parent ) co
     connect( qm, SIGNAL( newResultReady( QString, Meta::DataList ) ), SLOT( newResultReady( QString, Meta::DataList ) ), Qt::QueuedConnection );
     connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
     d->m_compilationQueries.insert( qm, parent );
+    d->m_runningQueries.insert( parent, qm );
     qm->run();
 }
 
+void CollectionTreeItemModelBase::startAnimationTick()
+{
+    //start animation
+    if( ( m_timeLine->state() != QTimeLine::Running ) && !d->m_runningQueries.isEmpty() )
+        m_timeLine->start();
+}
+
 void CollectionTreeItemModelBase::loadingAnimationTick()
 {
     if ( m_animFrame == 0 )
@@ -805,10 +980,13 @@ void CollectionTreeItemModelBase::loadingAnimationTick()
     m_animFrame = 1 - m_animFrame;
 
     //trigger an update of all items being populated at the moment;
-    QList<CollectionTreeItem* > items = d->m_childQueries.values();
 
-    foreach ( CollectionTreeItem* item, items )
-        emit ( dataChanged ( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) ) );
+    foreach ( CollectionTreeItem* item, d->m_runningQueries.keys() )
+    {
+        if( item == m_rootItem )
+            continue;
+        emit dataChanged ( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) );
+    }
 }
 
 void
@@ -820,11 +998,7 @@ CollectionTreeItemModelBase::setCurrentFilter( const QString &filter )
 void
 CollectionTreeItemModelBase::slotFilter()
 {
-    if ( isQuerying() )
-        return; // we are already busy, do not try to change filters in the middle of everything as that will cause crashes
-
     filterChildren();
-    reset();
     if ( !m_expandedCollections.isEmpty() )
     {
         foreach( Amarok::Collection *expanded, m_expandedCollections )
@@ -842,11 +1016,11 @@ CollectionTreeItemModelBase::slotCollapsed( const QModelIndex &index )
     if ( index.isValid() )      //probably unnecessary, but let's be safe
     {
         CollectionTreeItem *item = static_cast<CollectionTreeItem*>( index.internalPointer() );
-        if ( item->isDataItem() && item->data() )
+        if ( item->isDataItem() && !item->isVariousArtistItem() )
         {
             m_expandedItems.remove( item->data() );
         }
-        else if( item->isDataItem() && !item->data() ) //Various artists nodes
+        else if( item->isVariousArtistItem() ) //Various artists nodes
         {
             CollectionTreeItem *tmp = item->parent();
             while( tmp->isDataItem() )
@@ -877,8 +1051,6 @@ CollectionTreeItemModelBase::slotExpanded( const QModelIndex &index )
             while( tmp->isDataItem() )
                 tmp = tmp->parent();
 
-            if( tmp->parentCollection() )
-                debug() << "VA node for collection " << tmp->parentCollection()->collectionId() << " expanded";
             m_expandedVariousArtistsNodes.insert( tmp->parentCollection() );
         }
     }
@@ -889,11 +1061,38 @@ void CollectionTreeItemModelBase::update()
     reset();
 }
 
-bool CollectionTreeItemModelBase::isQuerying() const
+void CollectionTreeItemModelBase::markSubTreeAsDirty( CollectionTreeItem *item )
 {
-    return !( d->m_childQueries.isEmpty() && d->m_compilationQueries.isEmpty() );
+    //tracks are the leaves so they are never dirty
+    if( !item->isTrackItem() )
+        item->setRequiresUpdate( true );
+    for( int i = 0; i < item->childCount(); i++ )
+    {
+        markSubTreeAsDirty( item->child( i ) );
+    }
 }
 
+void CollectionTreeItemModelBase::itemAboutToBeDeleted( CollectionTreeItem *item )
+{
+    if( !d->m_runningQueries.contains( item ) )
+        return;
+    //replace this hack with QWeakPointer as soon as we depend on Qt 4.6
+    QueryMaker *qm = d->m_runningQueries.take( item );
+    if( qm )
+    {
+
+        d->m_childQueries.remove( qm );
+        d->m_compilationQueries.remove( qm );
+        //we still need to disconnect the qm below
+    }
+    if( qm )
+    {
+        //Disconnect all signals from the QueryMaker so we do not get notified about the results
+        qm->disconnect();
+        //Nuke it
+        qm->deleteLater();
+    }
+}
 
 #include "CollectionTreeItemModelBase.moc"
 
