@@ -1,20 +1,19 @@
-/*
- *  Copyright (c) 2007-2008 Maximilian Kossick <maximilian.kossick@googlemail.com>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- */
+/****************************************************************************************
+ * Copyright (c) 2007-2008 Maximilian Kossick <maximilian.kossick@googlemail.com>       *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
+
 #define DEBUG_PREFIX "CollectionManager"
 
 #include "CollectionManager.h"
@@ -25,10 +24,11 @@
 #include "EngineController.h"
 #include "MetaQueryMaker.h"
 #include "meta/file/File.h"
-#include "meta/cue/Cue.h"
 #include "meta/stream/Stream.h"
 #include "PluginManager.h"
+#include "SmartPointerList.h"
 #include "SqlStorage.h"
+#include "timecode/TimecodeTrackProvider.h"
 
 #include <QList>
 #include <QMetaEnum>
@@ -39,20 +39,49 @@
 #include <KBuildSycocaProgressDialog>
 #include <KGlobal>
 #include <KMessageBox>
-#include <KRun>
 #include <KService>
+#include <KPluginLoader>
+#include <KPluginFactory>
 
 #include <cstdlib>
 
 typedef QPair<Amarok::Collection*, CollectionManager::CollectionStatus> CollectionPair;
 
+class SqlStorageWrapper : public SqlStorage
+{
+public:
+    SqlStorageWrapper()
+        : SqlStorage()
+        , m_sqlStorage( 0 )
+    {}
+
+    virtual int sqlDatabasePriority() const { return ( m_sqlStorage ? m_sqlStorage->sqlDatabasePriority() : 0 ); }
+    virtual QString type() const  { return ( m_sqlStorage ? m_sqlStorage->type() : "SqlStorageWrapper" ); }
+    virtual QString escape( QString text ) const  { return ( m_sqlStorage ? m_sqlStorage->escape( text ) : text ); }
+    virtual QStringList query( const QString &query )  { return ( m_sqlStorage ? m_sqlStorage->query( query ) : QStringList() ); }
+    virtual int insert( const QString &statement, const QString &table )  { return ( m_sqlStorage ? m_sqlStorage->insert( statement, table ) : 0 ); }
+    virtual QString boolTrue() const  { return ( m_sqlStorage ? m_sqlStorage->boolTrue() : "1" ); }
+    virtual QString boolFalse() const  { return ( m_sqlStorage ? m_sqlStorage->boolFalse() : "0" ); }
+    virtual QString idType() const  { return ( m_sqlStorage ? m_sqlStorage->idType() : "WRAPPER_NOT_IMPLEMENTED" ); }
+    virtual QString textColumnType( int length ) const { return ( m_sqlStorage ? m_sqlStorage->textColumnType( length ) : "WRAPPER_NOT_IMPLEMENTED" ); }
+    virtual QString exactTextColumnType( int length ) const { return ( m_sqlStorage ? m_sqlStorage->exactTextColumnType( length ) : "WRAPPER_NOT_IMPLEMENTED" ); }
+    virtual QString exactIndexableTextColumnType( int length ) const { return ( m_sqlStorage ? m_sqlStorage->exactIndexableTextColumnType( length ) : "WRAPPER_NOT_IMPLEMENTED" ); };
+    virtual QString longTextColumnType() const { return ( m_sqlStorage ? m_sqlStorage->longTextColumnType() : "WRAPPER_NOT_IMPLEMENTED" ); }
+    virtual QString randomFunc() const { return ( m_sqlStorage ? m_sqlStorage->randomFunc() : "WRAPPER_NOT_IMPLEMENTED" ); }
+
+    void setSqlStorage( SqlStorage *sqlStorage ) { m_sqlStorage = sqlStorage; }
+private:
+    SqlStorage *m_sqlStorage;
+};
+
 struct CollectionManager::Private
 {
     QList<CollectionPair> collections;
-    QList<Amarok::CollectionFactory*> factories;
+    SmartPointerList<Amarok::CollectionFactory> factories;
     SqlStorage *sqlDatabase;
+    SqlStorageWrapper *sqlStorageWrapper;
     QList<Amarok::Collection*> unmanagedCollections;
-    QList<Amarok::Collection*> managedCollections;
+    SmartPointerList<Amarok::Collection> managedCollections;
     QList<Amarok::TrackProvider*> trackProviders;
     Amarok::Collection *primaryCollection;
 };
@@ -78,10 +107,13 @@ CollectionManager::CollectionManager()
     : QObject()
     , d( new Private )
 {
+    qRegisterMetaType<SqlStorage *>( "SqlStorage*" );
+    d->sqlDatabase = 0;
+    d->primaryCollection = 0;
+    d->sqlStorageWrapper = new SqlStorageWrapper();
     s_instance = this;
+    m_haveEmbeddedMysql = false;
 
-
-    qRegisterMetaType<TrackUrls>( "TrackUrls" );
     init();
 }
 
@@ -89,12 +121,17 @@ CollectionManager::~CollectionManager()
 {
     DEBUG_BLOCK
 
+    //not deleting SqlStorageWrapper here as somebody might be caching it
+    //Amarok really needs a proper state management...
+    d->sqlStorageWrapper->setSqlStorage( 0 );
+    delete m_timecodeTrackProvider;
     d->collections.clear();
     d->unmanagedCollections.clear();
     d->trackProviders.clear();
     qDeleteAll( d->managedCollections );
     qDeleteAll( d->factories );
 
+
     delete d;
 }
 
@@ -103,8 +140,10 @@ CollectionManager::init()
 {
     DEBUG_BLOCK
 
-    d->sqlDatabase = 0;
-    d->primaryCollection = 0;
+    //register the timecode track provider now, as it needs to get added before loading
+    //the stored playlist... Since it can have playable urls that might also match other providers, it needs to get added first.
+    m_timecodeTrackProvider = new TimecodeTrackProvider();
+    addTrackProvider( m_timecodeTrackProvider );
 
     KService::List plugins = PluginManager::query( "[X-KDE-Amarok-plugintype] == 'collection'" );
     debug() << "Received [" << QString::number( plugins.count() ) << "] collection plugin offers";
@@ -135,10 +174,55 @@ CollectionManager::init()
 
     foreach( KService::Ptr service, plugins )
     {
-        Amarok::Plugin *plugin = PluginManager::createFromService( service );
-        if ( plugin )
+        const QString name = service->property( "X-KDE-Amarok-name" ).toString();
+        if( name == "mysqlserver-collection" &&
+           !Amarok::config( "MySQL" ).readEntry( "UseServer", false ) )
+                continue;
+        if( name == "mysqle-collection" &&
+            Amarok::config( "MySQL" ).readEntry( "UseServer", false ) )
+                continue;
+        if( name == "mysqle-collection" || name == "mysqlserver-collection" )
         {
-            Amarok::CollectionFactory* factory = dynamic_cast<Amarok::CollectionFactory*>( plugin );
+            KPluginLoader loader( *( service.constData() ) );
+            KPluginFactory *pluginFactory = loader.factory();
+            if ( pluginFactory )
+            {
+                Amarok::CollectionFactory* factory = pluginFactory->create<Amarok::CollectionFactory>( this );
+                if ( factory )
+                {
+                    debug() << "Initialising sqlcollection";
+                    connect( factory, SIGNAL( newCollection( Amarok::Collection* ) ), this, SLOT( slotNewCollection( Amarok::Collection* ) ) );
+                    d->factories.append( factory );
+                    factory->init();
+                    if( name == "mysqle-collection" )
+                        m_haveEmbeddedMysql = true;
+                }
+                else
+                {
+                    debug() << "SqlCollection Plugin has wrong factory class: " << loader.errorString();
+                }
+            }
+            else
+            {
+                warning() << "Failed to get factory from KPluginLoader: " << loader.errorString();
+            }
+            break;
+        }
+    }
+
+    foreach( KService::Ptr service, plugins )
+    {
+        //ignore sqlcollection plugins, we have already loaded it above
+        if( service->property( "X-KDE-Amarok-name" ).toString() == "mysqlserver-collection" )
+                continue;
+        if( service->property( "X-KDE-Amarok-name" ).toString() == "mysqle-collection" )
+                continue;
+
+        KPluginLoader loader( *( service.constData() ) );
+        KPluginFactory *pluginFactory = loader.factory();
+        if ( pluginFactory )
+        {
+            Amarok::CollectionFactory* factory = pluginFactory->create<Amarok::CollectionFactory>( this );
             if ( factory )
             {
                 connect( factory, SIGNAL( newCollection( Amarok::Collection* ) ), this, SLOT( slotNewCollection( Amarok::Collection* ) ) );
@@ -147,11 +231,17 @@ CollectionManager::init()
             }
             else
             {
-                debug() << "Plugin has wrong factory class";
+                debug() << "Plugin has wrong factory class: " << loader.errorString();
                 continue;
             }
         }
+        else
+        {
+            warning() << "Failed to get factory from KPluginLoader: " << loader.errorString();
+        }
     }
+
+
 }
 
 void
@@ -163,6 +253,15 @@ CollectionManager::startFullScan()
     }
 }
 
+void
+CollectionManager::startIncrementalScan( const QString &directory )
+{
+    foreach( const CollectionPair &pair, d->collections )
+    {
+        pair.first->startIncrementalScan( directory );
+    }
+}
+
 void
 CollectionManager::stopScan()
 {
@@ -200,11 +299,21 @@ CollectionManager::queryMaker() const
 void
 CollectionManager::slotNewCollection( Amarok::Collection* newCollection )
 {
+    DEBUG_BLOCK
     if( !newCollection )
     {
         debug() << "Warning, newCollection in slotNewCollection is 0";
         return;
     }
+    foreach( CollectionPair p, d->collections )
+    {
+        if( p.first == newCollection )
+        {
+            debug() << "Warning, newCollection is already being managed";
+            return;
+        }
+    }
+
     const QMetaObject *mo = metaObject();
     const QMetaEnum me = mo->enumerator( mo->indexOfEnumerator( "CollectionStatus" ) );
     const QString &value = KGlobal::config()->group( "CollectionManager" ).readEntry( newCollection->collectionId() );
@@ -217,28 +326,40 @@ CollectionManager::slotNewCollection( Amarok::Collection* newCollection )
     d->trackProviders.append( newCollection );
     connect( newCollection, SIGNAL( remove() ), SLOT( slotRemoveCollection() ), Qt::QueuedConnection );
     connect( newCollection, SIGNAL( updated() ), SLOT( slotCollectionChanged() ), Qt::QueuedConnection );
-    SqlStorage *sqlCollection = dynamic_cast<SqlStorage*>( newCollection );
-    if( sqlCollection )
+    //by convention, collections that provide a SQL database have a Qt property called "sqlStorage"
+    int propertyIndex = newCollection->metaObject()->indexOfProperty( "sqlStorage" );
+    if( propertyIndex != -1 )
     {
-        //let's cheat a bit and assume that sqlStorage and the primaryCollection are always the same
-        //it is true for now anyway
-        if( d->sqlDatabase )
+        SqlStorage *sqlStorage = newCollection->property( "sqlStorage" ).value<SqlStorage*>();
+        if( sqlStorage )
         {
-            if( d->sqlDatabase->sqlDatabasePriority() < sqlCollection->sqlDatabasePriority() )
+            //let's cheat a bit and assume that sqlStorage and the primaryCollection are always the same
+            //it is true for now anyway
+            if( d->sqlDatabase )
             {
-                d->sqlDatabase = sqlCollection;
+                if( d->sqlDatabase->sqlDatabasePriority() < sqlStorage->sqlDatabasePriority() )
+                {
+                    d->sqlDatabase = sqlStorage;
+                    d->primaryCollection = newCollection;
+                    d->sqlStorageWrapper->setSqlStorage( sqlStorage );
+                }
+            }
+            else
+            {
+                d->sqlDatabase = sqlStorage;
                 d->primaryCollection = newCollection;
+                d->sqlStorageWrapper->setSqlStorage( sqlStorage );
             }
         }
         else
         {
-            d->sqlDatabase = sqlCollection;
-            d->primaryCollection = newCollection;
+            warning() << "Collection " << newCollection->collectionId() << " has sqlStorage property but did not provide a SqlStorage pointer";
         }
     }
     if( status & CollectionViewable )
     {
         emit collectionAdded( newCollection );
+        emit collectionAdded( newCollection, status );
     }
 }
 
@@ -253,25 +374,33 @@ CollectionManager::slotRemoveCollection()
         d->collections.removeAll( pair );
         d->managedCollections.removeAll( collection );
         d->trackProviders.removeAll( collection );
-        SqlStorage *sqlDb = dynamic_cast<SqlStorage*>( collection );
-        if( sqlDb && sqlDb == d->sqlDatabase )
+        QVariant v = collection->property( "sqlStorage" );
+        if( v.isValid() )
         {
-            SqlStorage *newSqlDatabase = 0;
-            foreach( const CollectionPair &pair, d->collections )
+            SqlStorage *sqlDb = v.value<SqlStorage*>();
+            if( sqlDb && sqlDb == d->sqlDatabase )
             {
-                SqlStorage *sqlDb = dynamic_cast<SqlStorage*>( pair.first );
-                if( sqlDb )
+                SqlStorage *newSqlDatabase = 0;
+                foreach( const CollectionPair &pair, d->collections )
                 {
-                    if( newSqlDatabase )
+                    QVariant variant = pair.first->property( "sqlStorage" );
+                    if( !variant.isValid() )
+                        continue;
+                    SqlStorage *sqlDb = variant.value<SqlStorage*>();
+                    if( sqlDb )
                     {
-                        if( newSqlDatabase->sqlDatabasePriority() < sqlDb->sqlDatabasePriority() )
+                        if( newSqlDatabase )
+                        {
+                            if( newSqlDatabase->sqlDatabasePriority() < sqlDb->sqlDatabasePriority() )
+                                newSqlDatabase = sqlDb;
+                        }
+                        else
                             newSqlDatabase = sqlDb;
                     }
-                    else
-                        newSqlDatabase = sqlDb;
                 }
+                d->sqlDatabase = newSqlDatabase;
+                d->sqlStorageWrapper->setSqlStorage( newSqlDatabase );
             }
-            d->sqlDatabase = newSqlDatabase;
         }
         emit collectionRemoved( collection->collectionId() );
         QTimer::singleShot( 0, collection, SLOT( deleteLater() ) );
@@ -326,7 +455,7 @@ CollectionManager::primaryCollection() const
 SqlStorage*
 CollectionManager::sqlStorage() const
 {
-    return d->sqlDatabase;
+    return d->sqlStorageWrapper;
 }
 
 Meta::TrackList
@@ -355,9 +484,7 @@ CollectionManager::trackForUrl( const KUrl &url )
     //might be a lastfm track, another stream
     //or a file which is not in any collection
     if( !url.isValid() )
-    {
         return Meta::TrackPtr( 0 );
-    }
 
     foreach( Amarok::TrackProvider *provider, d->trackProviders )
     {
@@ -369,19 +496,11 @@ CollectionManager::trackForUrl( const KUrl &url )
         }
     }
 
-    if( url.protocol() == "http" || url.protocol() == "mms" )
+    if( url.protocol() == "http" || url.protocol() == "mms" || url.protocol() == "smb" )
         return Meta::TrackPtr( new MetaStream::Track( url ) );
 
-    if( url.protocol() == "file" && EngineController::canDecode( url ) )
-    {
-        KUrl cuesheet = MetaCue::Track::locateCueSheet( url );
-        if( !cuesheet.isEmpty() ) {
-            debug() << "Creating MetaCue::Track";
-            return Meta::TrackPtr( new MetaCue::Track( url, cuesheet ) );
-        }
-        debug() << "Creating MetaFile::Track";
+    if( url.protocol() == "file" && EngineController::canDecode( url ) )       
         return Meta::TrackPtr( new MetaFile::Track( url ) );
-    }
 
     return Meta::TrackPtr( 0 );
 }
@@ -484,6 +603,7 @@ CollectionManager::addUnmanagedCollection( Amarok::Collection *newCollection, Co
         if( status & CollectionViewable )
         {
             emit collectionAdded( newCollection );
+            emit collectionAdded( newCollection, status );
         }
         emit trackProviderAdded( newCollection );
     }
@@ -518,6 +638,7 @@ CollectionManager::setCollectionStatus( const QString &collectionId, CollectionS
                     !( status & CollectionViewable ) )
             {
                 emit collectionAdded( pair.first );
+                emit collectionAdded( pair.first, pair.second );
             }
             CollectionPair &pair2 = const_cast<CollectionPair&>( pair );
             pair2.second = status;
