@@ -1,30 +1,27 @@
-/*
- *  Copyright (c) 2003-2008 Mark Kretschmann <kretschmann@kde.org>
- *  Copyright (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>
- *  Copyright (c) 2007 Casey Link <unnamedrambler@gmail.com>
- *  Copyright (c) 2008 Leo Franchi <lfranchi@kde.org>
- *  Copyright (c) 2008-2009 Jeff Mitchell <mitchell@kde.org>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- */
+/****************************************************************************************
+ * Copyright (c) 2003-2008 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>            *
+ * Copyright (c) 2007 Casey Link <unnamedrambler@gmail.com>                             *
+ * Copyright (c) 2008 Leo Franchi <lfranchi@kde.org>                                    *
+ * Copyright (c) 2008-2009 Jeff Mitchell <mitchell@kde.org>                             *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "ScanManager.h"
 
 #include "App.h"
 #include "Debug.h"
-#include "MountPointManager.h"
 #include "ScanResultProcessor.h"
 #include "SqlCollection.h"
 #include "SqlCollectionDBusHandler.h"
@@ -54,10 +51,11 @@ static const int MAX_FAILURE_PERCENTAGE = 5;
 static const int WATCH_INTERVAL = 60 * 1000; // = 60 seconds
 
 
-ScanManager::ScanManager( SqlCollection *parent )
+ScanManager::ScanManager( QObject *parent )
     : QObject( parent )
-    , m_collection( parent )
+    , m_collection( 0 )
     , m_dbusHandler( 0 )
+    , m_storage( 0 )
     , m_scanner( 0 )
     , m_parser( 0 )
     , m_restartCount( 0 )
@@ -66,8 +64,7 @@ ScanManager::ScanManager( SqlCollection *parent )
 {
     DEBUG_BLOCK
 
-    // If Amarok is not installed in standard directory
-    m_amarokCollectionScanDir = App::instance()->applicationDirPath() + QDir::separator();
+    m_dbusHandler = new SqlCollectionDBusHandler( this );
 
     QTimer *watchFoldersTimer = new QTimer( this );
     connect( watchFoldersTimer, SIGNAL( timeout() ), SLOT( slotWatchFolders() ) );
@@ -79,8 +76,13 @@ ScanManager::~ScanManager()
     DEBUG_BLOCK
 
     stopParser();
-}
 
+    if( m_dbusHandler )
+    {
+        m_dbusHandler->deleteLater();
+        m_dbusHandler = 0;
+    }
+}
 
 void
 ScanManager::startFullScan()
@@ -96,6 +98,8 @@ ScanManager::startFullScan()
         debug() << "scanning currently blocked";
         return;
     }
+
+    checkTables( true );
     cleanTables();
 
     if( m_parser )
@@ -108,20 +112,21 @@ ScanManager::startFullScan()
     connect( m_parser, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( slotJobDone() ) );
     ThreadWeaver::Weaver::instance()->enqueue( m_parser );
 
-    QString batchfileLocation( KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchfullscan.xml" );
+    const QString batchfileLocation( KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchfullscan.xml" );
     debug() << "Checking for batch file in " << batchfileLocation;
-    
+
     if( !QFile::exists( batchfileLocation ) || !readBatchFile( batchfileLocation )  )
     {
         m_scanner = new AmarokProcess( this );
-        *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "-p";
+        *m_scanner <<  App::collectionScannerLocation() << "-p";
         if( AmarokConfig::scanRecursively() )
             *m_scanner << "-r";
+        if( AmarokConfig::useCharsetDetector() )
+            *m_scanner << "-c";
         *m_scanner << "--savelocation" << KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true );
-        debug() << "GOING TO SCAN:";
-        foreach( const QString &dir, MountPointManager::instance()->collectionFolders() )
-            debug() << "    " << dir;
-        *m_scanner << MountPointManager::instance()->collectionFolders();
+
+        QStringList collectionFolders = m_collection->mountPointManager()->collectionFolders();
+        *m_scanner << collectionFolders;
         m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
         connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
         connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished(  ) ) );
@@ -130,7 +135,7 @@ ScanManager::startFullScan()
     }
 }
 
-void ScanManager::startIncrementalScan()
+void ScanManager::startIncrementalScan( const QString &directory )
 {
     DEBUG_BLOCK
     if( m_parser )
@@ -144,28 +149,50 @@ void ScanManager::startIncrementalScan()
         return;
     }
 
-    QString batchfileLocation( KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchincrementalscan.xml" );
-    bool batchfileExists = QFile::exists( batchfileLocation );
-    if( batchfileExists )
-        debug() << "Found batchfile in " << batchfileLocation;
+    checkTables( false );
 
     QStringList dirs;
-    
-    if( !batchfileExists )
+    QString batchfileLocation;
+    bool batchfileExists = false;
+
+    if( !directory.isEmpty() )
     {
-        dirs = getDirsToScan();
-
-        debug() << "GOING TO SCAN:";
-        foreach( const QString &dir, dirs )
-            debug() << "    " << dir;
-
-        if( dirs.isEmpty() )
+        debug() << "Scanning directory " << directory;
+        if( isDirInCollection( directory ) )
+            dirs << directory;
+        else
         {
-            debug() << "Scanning nothing, return.";
-            writeBatchIncrementalInfoFile();
+            debug() << "Directory not in collection!";
             return;
         }
+    }
+    else
+    {
+        batchfileLocation = KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchincrementalscan.xml";
+        batchfileExists = QFile::exists( batchfileLocation );
+        if( batchfileExists )
+            debug() << "Found batchfile in " << batchfileLocation;
 
+        if( !batchfileExists )
+        {
+            dirs = getDirsToScan();
+
+            debug() << "GOING TO SCAN:";
+            if( dirs.size() > 30 )
+                debug() << QString( "(a *lot*: %1 directories)" ).arg( dirs.size() );
+            else
+            {
+                foreach( const QString &dir, dirs )
+                    debug() << "    " << dir;
+            }
+
+            writeBatchIncrementalInfoFile();
+            if( dirs.isEmpty() )
+            {
+                debug() << "Scanning nothing, return.";
+                return;
+            }
+        }
     }
 
     if( m_parser )
@@ -180,24 +207,23 @@ void ScanManager::startIncrementalScan()
 
     if( !batchfileExists || !readBatchFile( batchfileLocation )  )
     {
-        if( !m_dbusHandler )
-        {
-            m_dbusHandler = new SqlCollectionDBusHandler( m_collection );
-        }
         m_scanner = new AmarokProcess( this );
-        *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "-i"
-                << "--collectionid" << m_collection->collectionId() << "-p";
-        if( AmarokConfig::scanRecursively() )
+        *m_scanner << App::collectionScannerLocation() << "-i" << "--collectionid" << m_collection->collectionId() << "-p";
+
+        if( AmarokConfig::scanRecursively() && directory.isEmpty() )
             *m_scanner << "-r";
+        if( AmarokConfig::useCharsetDetector() )
+            *m_scanner << "-c";
         *m_scanner << "--savelocation" << KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true );
         if( pApp->isNonUniqueInstance() )
             *m_scanner << "--pid" << QString::number( QApplication::applicationPid() );
-        *m_scanner << dirs;
+        *m_scanner << "--mtime-file" << KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true )
+                + "amarokcollectionscanner_batchincrementalinput.data";
         m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
         connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
         connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished() ) );
         connect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), SLOT( slotError( QProcess::ProcessError ) ) );
-        m_scanner->start();        
+        m_scanner->start();
     }
 }
 
@@ -208,12 +234,12 @@ ScanManager::isDirInCollection( QString path )
     if( !path.endsWith( '/' ) )
         path += '/';
 
-    const int deviceid = MountPointManager::instance()->getIdForUrl( path );
-    const QString rpath = MountPointManager::instance()->getRelativePath( deviceid, path );
+    const int deviceid = m_collection->mountPointManager()->getIdForUrl( path );
+    const QString rpath = m_collection->mountPointManager()->getRelativePath( deviceid, path );
 
     const QStringList values =
-            m_collection->query( QString( "SELECT changedate FROM directories WHERE dir = '%2' AND deviceid = %1;" )
-            .arg( QString::number( deviceid ), m_collection->escape( rpath ) ) );
+            m_storage->query( QString( "SELECT changedate FROM directories WHERE dir = '%2' AND deviceid = %1;" )
+            .arg( QString::number( deviceid ), m_storage->escape( rpath ) ) );
 
     //debug() << "dir " << rpath << " is in collection? " << !values.isEmpty();
     return !values.isEmpty();
@@ -222,11 +248,11 @@ ScanManager::isDirInCollection( QString path )
 bool
 ScanManager::isFileInCollection( const QString &url  )
 {
-    const int deviceid = MountPointManager::instance()->getIdForUrl( url );
-    const QString rpath = MountPointManager::instance()->getRelativePath( deviceid, url );
+    const int deviceid = m_collection->mountPointManager()->getIdForUrl( url );
+    const QString rpath = m_collection->mountPointManager()->getRelativePath( deviceid, url );
 
     QString sql = QString( "SELECT id FROM urls WHERE rpath = '%2' AND deviceid = %1" )
-            .arg( QString::number( deviceid ),  m_collection->escape( rpath ) );
+            .arg( QString::number( deviceid ),  m_storage->escape( rpath ) );
 
     if ( deviceid == -1 )
     {
@@ -236,10 +262,10 @@ ScanManager::isFileInCollection( const QString &url  )
     {
         QString rpath2 = '.' + url;
         sql += QString( " OR rpath = '%1' AND deviceid = -1;" )
-                .arg( m_collection->escape( rpath2 ) );
+                .arg( m_storage->escape( rpath2 ) );
     }
 
-    const QStringList values = m_collection->query( sql );
+    const QStringList values = m_storage->query( sql );
 
     //debug() << "File " << rpath << " is in collection? " << !values.isEmpty();
     return !values.isEmpty();
@@ -256,6 +282,13 @@ ScanManager::setBlockScan( bool blockScan )
     //TODO what happens if the collection scanner is currently running?
 }
 
+void
+ScanManager::setCollection( SqlCollection *collection )
+{
+    m_collection = collection;
+    m_dbusHandler->setCollection( collection );
+}
+
 void
 ScanManager::slotWatchFolders()
 {
@@ -294,14 +327,6 @@ ScanManager::slotFinished( )
     m_scanner->deleteLater();
     m_scanner = 0;
     m_restartCount = 0;
-    if( m_dbusHandler )
-    {
-        m_dbusHandler->deleteLater();
-        m_dbusHandler = 0;
-    }
-
-    if( m_isIncremental )
-        writeBatchIncrementalInfoFile();
 }
 
 void
@@ -326,7 +351,7 @@ ScanManager::abort( const QString &reason )
         debug() << "Scan error: " << reason;
     else
         debug() << "Unknown error: reseting scan manager state";
-    
+
     slotReadReady(); //make sure that we read the complete buffer
 
     disconnect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
@@ -334,7 +359,7 @@ ScanManager::abort( const QString &reason )
     disconnect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( slotError( QProcess::ProcessError ) ) );
     m_scanner->deleteLater();
     m_scanner = 0;
-    
+
     stopParser();
 }
 
@@ -348,7 +373,7 @@ ScanManager::readBatchFile( QString fileLocation )
         debug() << "Couldn't open batchscan file, which does exist";
         return false;
     }
-    
+
     QByteArray data;
     data = file.readAll();
 
@@ -388,7 +413,8 @@ ScanManager::getDirsToScan()
 {
     DEBUG_BLOCK
 
-    const IdList list = MountPointManager::instance()->getMountedDeviceIds();
+    m_incrementalDirs.clear();
+    const IdList list = m_collection->mountPointManager()->getMountedDeviceIds();
     QString deviceIds;
     foreach( int id, list )
     {
@@ -396,7 +422,7 @@ ScanManager::getDirsToScan()
         deviceIds += QString::number( id );
     }
 
-    const QStringList values = m_collection->query(
+    const QStringList values = m_storage->query(
             QString( "SELECT id, deviceid, dir, changedate FROM directories WHERE deviceid IN (%1);" )
             .arg( deviceIds ) );
 
@@ -408,18 +434,15 @@ ScanManager::getDirsToScan()
     {
         int id = iter.next().toInt();
         int deviceid = iter.next().toInt();
-        const QString folder = MountPointManager::instance()->getAbsolutePath( deviceid, iter.next() );
+        const QString folder = m_collection->mountPointManager()->getAbsolutePath( deviceid, iter.next() );
         const uint mtime = iter.next().toUInt();
 
         QFileInfo info( folder );
         if( info.exists() )
         {
-            m_incrementalDirs << folder;
+            m_incrementalDirs << QString( folder + "_AMAROKMTIME_" + QString::number( mtime ) );
             if( info.lastModified().toTime_t() != mtime )
-            {
-                result << folder;
                 changedFolderIds << id;
-            }
         }
         else
         {
@@ -439,7 +462,7 @@ ScanManager::getDirsToScan()
         if( !ids.isEmpty() )
         {
             QString query = QString( "SELECT id FROM urls WHERE directory IN ( %1 );" ).arg( ids );
-            QStringList urlIds = m_collection->query( query );
+            QStringList urlIds = m_storage->query( query );
             ids.clear();
             foreach( const QString &id, urlIds )
             {
@@ -450,7 +473,7 @@ ScanManager::getDirsToScan()
             if( !ids.isEmpty() )
             {
                 QString sql = QString( "DELETE FROM tracks WHERE url IN ( %1 );" ).arg( ids );
-                m_collection->query( sql );
+                m_storage->query( sql );
             }
         }
     }
@@ -465,11 +488,11 @@ ScanManager::getDirsToScan()
         if( !ids.isEmpty() )
         {
             QString sql = QString( "DELETE FROM directories WHERE id IN ( %1 );" ).arg( ids );
-            m_collection->query( sql );
+            m_storage->query( sql );
         }
     }
     //debug() << "Scanning the following dirs: " << result;
-    return result;
+    return m_incrementalDirs;
 }
 
 void
@@ -512,7 +535,8 @@ ScanManager::restartScanner()
     DEBUG_BLOCK
 
     m_scanner = new AmarokProcess( this );
-    *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner";
+    *m_scanner << App::collectionScannerLocation();
+
     if( m_isIncremental )
     {
         *m_scanner << "-i" << "--collectionid" << m_collection->collectionId();
@@ -532,12 +556,43 @@ void
 ScanManager::cleanTables()
 {
     DEBUG_BLOCK
-    m_collection->query( "DELETE FROM tracks;" );
-    m_collection->query( "DELETE FROM genres;" );
-    m_collection->query( "DELETE FROM years;" );
-    m_collection->query( "DELETE FROM composers;" );
-    m_collection->query( "DELETE FROM albums;" );
-    m_collection->query( "DELETE FROM artists;" );
+    m_storage->query( "DELETE FROM tracks;" );
+    m_storage->query( "DELETE FROM genres;" );
+    m_storage->query( "DELETE FROM years;" );
+    m_storage->query( "DELETE FROM composers;" );
+    m_storage->query( "DELETE FROM albums;" );
+    m_storage->query( "DELETE FROM artists;" );
+    //images table is deleted in DatabaseUpdater::copyToPermanentTables
+}
+
+void
+ScanManager::checkTables( bool full )
+{
+    DEBUG_BLOCK
+
+    int checkCount = 0; //0 is the default in amarokrc
+    if( !full )
+    {
+        checkCount = Amarok::config( "MySQL" ).readEntry( "CheckCount" ).toInt();
+        if( checkCount != 0 //always run a full one if you don't know the status
+            && checkCount < 30 ) //every 30 mins seems like a good amount
+        {
+            checkCount++;
+            Amarok::config( "MySQL" ).writeEntry( "CheckCount", QString::number( checkCount ) );
+            return;
+        }
+    }
+
+    DatabaseUpdater *dbUpdater = m_collection->dbUpdater();
+    if( !dbUpdater )
+    {
+        debug() << "WOAH, why is there no DB updater?";
+        return;
+    }
+
+    dbUpdater->checkTables( checkCount == 0 );
+
+    Amarok::config( "MySQL" ).writeEntry( "CheckCount", 1 );
 }
 
 void
@@ -572,7 +627,6 @@ ScanManager::writeBatchIncrementalInfoFile()
         stream << m_incrementalDirs.join( "\n" );
         incrementalFile.close();
     }
-    m_incrementalDirs.clear();
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -611,19 +665,17 @@ XmlParseJob::run()
 {
     DEBUG_BLOCK
 
-    QList<QVariantMap > directoryData;
+    QList<QVariantMap> directoryData;
     bool firstTrack = true;
     QString currentDir;
 
     ScanResultProcessor processor( m_collection );
+    processor.setSqlStorage( m_collection->sqlStorage() );
     if( m_isIncremental )
-    {
         processor.setScanType( ScanResultProcessor::IncrementalScan );
-    }
     else
-    {
         processor.setScanType( ScanResultProcessor::FullScan );
-    }
+
     do
     {
         m_abortMutex.lock();
@@ -637,9 +689,7 @@ XmlParseJob::run()
 
         m_mutex.lock();
         if( m_nextData.isEmpty() )
-        {
             m_wait.wait( &m_mutex );
-        }
 
         if( m_nextData.isEmpty() )
             break;
@@ -659,14 +709,19 @@ XmlParseJob::run()
                     emit incrementProgress();
                 }
 
+                if( localname == "dud" || localname == "compilation" )
+                    processor.doneWithImages();
+
                 if( localname == "itemcount" )
                 {
+                    processor.doneWithImages();
 //                     debug() << "Got an itemcount with value: " << localname.toString();
                     if( The::statusBar() )
                         The::statusBar()->incrementProgressTotalSteps( this, m_reader.attributes().value( "count" ).toString().toInt() );
                 }
                 else if( localname == "tags" )
                 {
+                    processor.doneWithImages();
 //                     debug() << "Parsing FILE:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> list = attrs.toList();
@@ -677,15 +732,15 @@ XmlParseJob::run()
 
                     //NOTE: compilation may not be used according to the comment below,
                     //but this is functionality getting moved from the collection scanner
-                    //until (if ever) pure-Qt transations are supported
+                    //until (if ever) pure-Qt translations are supported
                     QString compilationValue = attrs.value( "compilation" ).toString();
                     if( compilationValue == "checkforvarious" )
                     {
                         if( attrs.value( "artist" ).toString() == i18n( "Various Artists" ) )
                             compilationValue = QString::number( 1 );
                         else
-                            compilationValue = QString();
-                    }                            
+                            compilationValue.clear();
+                    }
 
                     QVariantMap data;
                     data.insert( Meta::Field::URL, attrs.value( "path" ).toString() );
@@ -747,6 +802,7 @@ XmlParseJob::run()
                 }
                 else if( localname == "folder" )
                 {
+                    processor.doneWithImages();
 //                     debug() << "Parsing FOLDER:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> list = attrs.toList();
@@ -762,13 +818,14 @@ XmlParseJob::run()
                 }
                 else if( localname == "playlist" )
                 {
+                    processor.doneWithImages();
                     //TODO check for duplicates
                     //debug() << "Saving playlist with path: " << m_reader.attributes().value( "path" ).toString();
                     The::playlistManager()->import( m_reader.attributes().value( "path" ).toString() );
                 }
                 else if( localname == "image" )
                 {
-//                     debug() << "Parsing IMAGE:\n";
+                    //debug() << "Parsing IMAGE:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> thisList = attrs.toList();
 
@@ -778,7 +835,7 @@ XmlParseJob::run()
 
                     // Deserialize CoverBundle list
                     QStringList list = attrs.value( "list" ).toString().split( "AMAROK_MAGIC" );
-                    QList< QPair<QString, QString> > covers;
+                    QList<QPair<QString, QString> > covers;
 
                     // Don't iterate if the list only has one element
                     if( list.size() > 1 )
@@ -786,6 +843,7 @@ XmlParseJob::run()
                         for( int i = 0; i < list.count(); i += 2 )
                             covers += qMakePair( list[i], list[i + 1] );
 
+                        //debug() << "Adding image with path " << attrs.value( "path" );
                         processor.addImage( attrs.value( "path" ).toString(), covers );
                     }
                 }
@@ -793,7 +851,7 @@ XmlParseJob::run()
         }
         if( m_reader.error() != QXmlStreamReader::PrematureEndOfDocumentError && m_reader.error() != QXmlStreamReader::NoError )
         {
-            debug() << "do-while done with error: " << m_reader.error();
+            error() << "Collection scanner abort error: " << m_reader.error();
             //the error cannot be PrematureEndOfDocumentError, so handle an unrecoverable error here
 
             // At this point, most likely the scanner has crashed and is about to get restarted.
