@@ -1,21 +1,18 @@
-/***************************************************************************
- *   Copyright (C) 2004-2008 by Mark Kretschmann <kretschmann@kde.org>     *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2004-2008 Mark Kretschmann <kretschmann@kde.org>                       *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "ConfigDialog.h"
 
@@ -24,9 +21,11 @@
 #include "Debug.h"
 
 #include "CollectionConfig.h"
+#include "DatabaseConfig.h"
 #include "GeneralConfig.h"
+#include "MainWindow.h"
 //#include "MediadeviceConfig.h"
-#include "OsdConfig.h"
+#include "NotificationsConfig.h"
 #include "PlaybackConfig.h"
 #include "ServiceConfig.h"
 
@@ -48,15 +47,18 @@ Amarok2ConfigDialog::Amarok2ConfigDialog( QWidget *parent, const char* name, KCo
     ConfigDialogBase* collection  = new CollectionConfig( this );
     ConfigDialogBase* services    = new ServiceConfig( this );
     ConfigDialogBase* playback    = new PlaybackConfig( this );
-    ConfigDialogBase* osd         = new OsdConfig( this );
-//    ConfigDialogBase* mediadevice = new MediadeviceConfig( this );
+    ConfigDialogBase* notify      = new NotificationsConfig( this );
+    ConfigDialogBase* database    = new DatabaseConfig( this );
+
+    //ConfigDialogBase* mediadevice = new MediadeviceConfig( this );
 
     addPage( general,     i18nc( "Miscellaneous settings", "General" ), "preferences-other-amarok", i18n( "Configure General Options" ) );
     addPage( collection,  i18n( "Collection" ), "collection-amarok", i18n( "Configure Collection" ) );
     addPage( services,    i18n( "Internet Services" ), "services-amarok", i18n( "Configure Services" ) );
     addPage( playback,    i18n( "Playback" ), "preferences-media-playback-amarok", i18n( "Configure Playback" ) );
-    addPage( osd,         i18n( "On Screen Display" ), "preferences-indicator-amarok", i18n( "Configure On-Screen-Display" ) );
-//    addPage( mediadevice, i18n( "Media Devices" ), "preferences-multimedia-player-amarok", i18n( "Configure Portable Player Support" ) );
+    addPage( notify,         i18n( "Notifications" ), "preferences-indicator-amarok", i18n( "Configure Notifications" ) );
+    addPage( database,    i18n( "Database" ), "server-database", i18n( "Configure Database" ) );
+    //addPage( mediadevice, i18n( "Media Devices" ), "preferences-multimedia-player-amarok", i18n( "Configure Portable Player Support" ) );
 
     setButtons( Help | Ok | Apply | Cancel );
 }
@@ -86,7 +88,6 @@ void Amarok2ConfigDialog::updateButtons() //SLOT
     enableButtonApply( hasChanged() );
 }
 
-
 /** Reimplemented from KConfigDialog */
 void Amarok2ConfigDialog::addPage( ConfigDialogBase *page, const QString &itemName, const QString &pixmapName, const QString &header, bool manage )
 {
@@ -98,7 +99,6 @@ void Amarok2ConfigDialog::addPage( ConfigDialogBase *page, const QString &itemNa
     m_pageMap.insert( page, pageWidget );
 }
 
-
 void Amarok2ConfigDialog::show( QString page )
 {
     if( page.isNull() )
@@ -136,7 +136,6 @@ void Amarok2ConfigDialog::updateSettings()
         page->updateSettings();
 }
 
-
 /**
  * Update the configuration-widgets in the dialog based on Amarok's current settings.
  * Example use: Initialisation of dialog.
@@ -149,7 +148,6 @@ void Amarok2ConfigDialog::updateWidgets()
         page->updateWidgets();
 }
 
-
 /**
  * Update the configuration-widgets in the dialog based on the default values for Amarok's settings.
  * Example use: User clicks Defaults button in a configure dialog.
@@ -186,7 +184,6 @@ bool Amarok2ConfigDialog::hasChanged()
     return changed;
 }
 
-
 /** REIMPLEMENTED */
 bool Amarok2ConfigDialog::isDefault()
 {
