@@ -1,16 +1,20 @@
-/*****************************************************************************
- * copyright            : (C) 2007-2009 Leo Franchi <lfranchi@gmail.com>     *
- *                      : (C) 2008 William Viana Soares <vianasw@gmail.com>  *
- *****************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2007-2009 Leo Franchi <lfranchi@gmail.com>                             *
+ * Copyright (c) 2008 William Viana Soares <vianasw@gmail.com>                          *
+ * Copyright (c) 2009 simon.esneault <simon.esneault@gmail.com>                         *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "CurrentTrack.h"
 
@@ -18,17 +22,21 @@
 #include "App.h"
 #include "Debug.h"
 #include "EngineController.h"
-#include "context/popupdropper/libpud/PopupDropperAction.h"
+#include "GlobalCurrentTrackActions.h"
 #include "meta/capabilities/CurrentTrackActionsCapability.h"
 #include "meta/MetaUtility.h"
 #include "PaletteHandler.h"
 #include "SvgHandler.h"
-#include <context/widgets/RatingWidget.h>
+#include "context/widgets/RatingWidget.h"
+#include "context/widgets/TextScrollingWidget.h"
+#include "context/widgets/DropPixmapItem.h"
+#include "UpdateCapability.h"
 
 #include <plasma/theme.h>
 #include <plasma/widgets/tabbar.h>
 
 #include <KColorScheme>
+#include <KConfigDialog>
 
 #include <QFont>
 #include <QGraphicsLinearLayout>
@@ -43,11 +51,9 @@ CurrentTrack::CurrentTrack( QObject* parent, const QVariantList& args )
     , m_tracksToShow( 0 )
     , m_tabBar( 0 )
 {
-    setHasConfigurationInterface( false );
-    setConfigurationRequired( false );
+    setHasConfigurationInterface( true );
     setBackgroundHints( Plasma::Applet::NoBackground );
     setImmutability( Plasma::Mutable );
-
     // Fix for BUG 190923:
     // If the widget receives focus, somehow this makes it impossible to open the Amarok menu. Reason unclear.
     setFocusPolicy( Qt::NoFocus );
@@ -68,38 +74,59 @@ CurrentTrack::init()
 
     connect( m_ratingWidget, SIGNAL( ratingChanged( int ) ), SLOT( changeTrackRating( int ) ) );
 
-    m_title        = new QGraphicsSimpleTextItem( this );
-    m_artist       = new QGraphicsSimpleTextItem( this );
-    m_album        = new QGraphicsSimpleTextItem( this );
+    // TextScrollingWidget is a QGraphicsSimpleTextItem which will scroll is content on mouse hover
+    m_title        = new TextScrollingWidget( this );
+    m_artist       = new TextScrollingWidget( this );
+    m_album        = new TextScrollingWidget( this );
     m_noTrack      = new QGraphicsSimpleTextItem( this );
-    m_albumCover   = new QGraphicsPixmapItem    ( this );
+    m_albumCover   = new DropPixmapItem( this );
     m_byText       = new QGraphicsSimpleTextItem( i18nc( "What artist is this track by", "By" ), this );
     m_onText       = new QGraphicsSimpleTextItem( i18nc( "What album is this track on", "On" ), this );
 
-    QBrush brush = KColorScheme( QPalette::Active ).foreground( KColorScheme::NormalText );
+    connect( m_albumCover, SIGNAL( imageDropped( QPixmap) ), this, SLOT( coverDropped( QPixmap ) ) );
+
+    const QBrush brush = KColorScheme( QPalette::Active ).foreground( KColorScheme::NormalText );
 
     m_title->setBrush( brush );
     m_artist->setBrush( brush );
-    m_album->setBrush( brush );;
+    m_album->setBrush( brush );
     m_noTrack->setBrush( brush );
+    m_byText->setBrush( brush );
+    m_onText->setBrush( brush );
+
+    // Read config
+    KConfigGroup config = Amarok::config("Current Track Applet");
+    QFont configFont( config.readEntry( "Font", QString() ),
+                config.readEntry( "Size", -1 ) );
+
+    if( configFont.family().isEmpty() )
+    {
+        QFont bigFont;
+        bigFont.setPointSize( bigFont.pointSize() +  3 );
+
+        m_noTrack->setFont( bigFont );
+        m_title->setFont( bigFont );
+        m_artist->setFont( bigFont );
+        m_album->setFont( bigFont );
+    }
+    else
+    {
+        m_noTrack->setFont( configFont );
+        m_title->setFont( configFont );
+        m_artist->setFont( configFont );
+        m_album->setFont( configFont );
+    }
 
-    QFont bigFont;
-    bigFont.setPointSize( bigFont.pointSize() +  3 );
-    
     QFont tinyFont;
     //tinyFont.setPointSize( tinyFont.pointSize() - 2 );
 
-    m_noTrack->setFont( bigFont );
-    m_title->setFont( bigFont );
-    m_artist->setFont( bigFont );
-    m_album->setFont( bigFont );
     m_byText->setFont( tinyFont );
     m_onText->setFont( tinyFont );
-    
+
     m_noTrackText = i18n( "No track playing" );
     m_noTrack->hide();
     m_noTrack->setText( m_noTrackText );
-    
+
     m_tabBar = new Plasma::TabBar( this );
 
     m_playCountLabel = i18n( "Play count" );
@@ -108,7 +135,7 @@ CurrentTrack::init()
 
     for( int i = 0; i < MAX_PLAYED_TRACKS; i++ )
         m_tracks[i] = new TrackWidget( this );
- 
+
     m_tabBar->addTab( i18n( "Last played" ) );
     m_tabBar->addTab( i18n( "Favorite tracks" ) );
 
@@ -120,8 +147,13 @@ CurrentTrack::init()
     connect( dataEngine( "amarok-current" ), SIGNAL( sourceAdded( const QString& ) ), this, SLOT( connectSource( const QString& ) ) );
     connect( The::paletteHandler(), SIGNAL( newPalette( const QPalette& ) ), SLOT(  paletteChanged( const QPalette &  ) ) );
 
-    // completely arbitrary yet necessary to kick start the layout before the user acts and resizes manually
-    resize( 500, .75 );
+    // figure out the size we want to be, in order to be able to squeeze in all that we want
+    // depends on the current font size,  basically
+    // height should be increased for larger point sizes. here, the layout works correctly with size 8, which has the fontMetrics height of 13
+    // a size too big, like font size 12, has a fontMetrics height of 19. So we add some height if it's too big
+    int additional = ( QApplication::fontMetrics().height()-13 ) * 2;
+    resize( 500, 180 + additional );
+    
 
     // hide the items while we startup. as soon as the query is done, they'll be shown.
     foreach ( QGraphicsItem * childItem, QGraphicsItem::children() )
@@ -134,7 +166,7 @@ CurrentTrack::connectSource( const QString &source )
     if( source == "current" )
     {
         dataEngine( "amarok-current" )->connectSource( source, this );
-        dataUpdated( source, dataEngine("amarok-current" )->query( "current" ) ); // get data initally
+        dataUpdated( source, dataEngine("amarok-current" )->query( "current" ) ); // get data initially
     }
 }
 
@@ -143,8 +175,19 @@ CurrentTrack::changeTrackRating( int rating )
 {
     DEBUG_BLOCK
     Meta::TrackPtr track = The::engineController()->currentTrack();
+
+    if( !track )
+        return;
+
+    // Inform collections of end of a metadata update
+    Meta::UpdateCapability *uc = track->create<Meta::UpdateCapability>();
+    if( !uc )
+        return;
+
     track->setRating( rating );
     debug() << "change rating to: " << rating;
+
+    uc->collectionUpdated();
 }
 
 QList<QAction*>
@@ -154,10 +197,10 @@ CurrentTrack::contextualActions()
     QList<QAction*> actions;
 
     Meta::TrackPtr track = The::engineController()->currentTrack();
-    
+
     if( !track )
         return actions;
-    
+
     Meta::AlbumPtr album = track->album();
 
     if( album )
@@ -165,9 +208,9 @@ CurrentTrack::contextualActions()
         Meta::CustomActionsCapability *cac = album->create<Meta::CustomActionsCapability>();
         if( cac )
         {
-            QList<PopupDropperAction *> pudActions = cac->customActions();
-             
-            foreach( PopupDropperAction *action, pudActions )
+            QList<QAction *> customActions = cac->customActions();
+
+            foreach( QAction *action, customActions )
                 actions.append( action );
         }
     }
@@ -178,11 +221,10 @@ CurrentTrack::contextualActions()
 void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
 {
     Q_UNUSED( constraints )
-    DEBUG_BLOCK
 
     prepareGeometryChange();
 
-    // these all used to be based on fancy calculatons based on the height
+    // these all used to be based on fancy calculations based on the height
     // guess what: the height is fixed. so that's a waste of hard-working gerbils
     const qreal textHeight = 30;
     const qreal albumWidth = 135;
@@ -197,7 +239,7 @@ void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
     const qreal x = standardPadding();
     const qreal y = boundingRect().height() - m_ratingWidget->boundingRect().height() - standardPadding();
     m_ratingWidget->setPos( x, y );
-    
+
     /* this is the pos of the albumcover */
     const qreal textX =  albumCoverPos.x() +  albumWidth + standardPadding();
     const qreal textWidth = size().toSize().width() - ( textX + standardPadding() * 2 + 23 );
@@ -208,19 +250,21 @@ void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
     qreal lineSpacing = fm.height() + 4;
     m_maxTextWidth = textWidth;
 
-    // align to either the album or artist line, depending of "On" or "By" is longer in this current translation
-    // i10n makes things complicated :P
+    // align to either the album or artist line, depending if "On" or "By" is longer in this current translation
+    // i18n makes things complicated :P
     if( m_byText->boundingRect().width() > m_onText->boundingRect().width() )
-    { // align to location of on text
+    {
+        // align to location of on text
         m_byText->setPos( textX, textY + lineSpacing + 3  );
         m_artist->setPos( m_byText->pos().x() + m_byText->boundingRect().width() + 5, 0 );
         alignBottomToFirst( m_byText, m_artist );
         m_album->setPos( m_artist->pos().x(), 0 );
         m_onText->setPos( m_album->pos().x() - m_onText->boundingRect().width() - 5, textY + lineSpacing * 2 + 3 );
         alignBottomToFirst( m_onText, m_album );
-
-    } else // align to location/width of by text
+    }
+    else
     {
+        // align to location/width of by text
         m_onText->setPos( textX, textY + lineSpacing * 2 + 3 );
         m_album->setPos( m_onText->pos().x() + m_onText->boundingRect().width() + 5, 0 );
         alignBottomToFirst( m_onText, m_album );
@@ -228,21 +272,35 @@ void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
         m_byText->setPos( m_artist->pos().x() - m_byText->boundingRect().width() - 5, textY + lineSpacing + 3 );
         alignBottomToFirst( m_byText, m_artist );
     }
-    
+
     m_title->setPos( m_artist->pos().x(), textY );
 
     const QString title = m_currentInfo[ Meta::Field::TITLE ].toString();
     const QString artist = m_currentInfo.contains( Meta::Field::ARTIST ) ? m_currentInfo[ Meta::Field::ARTIST ].toString() : QString();
     const QString album = m_currentInfo.contains( Meta::Field::ALBUM ) ? m_currentInfo[ Meta::Field::ALBUM ].toString() : QString();
-    const QString lastPlayed = m_currentInfo.contains( Meta::Field::LAST_PLAYED ) ? Amarok::conciseTimeSince( m_currentInfo[ Meta::Field::LAST_PLAYED ].toUInt() ) : QString();
 
-    m_artist->setText( truncateTextToFit( artist, m_artist->font(), QRectF( 0, 0, textWidth, 30 ) ) );
-    m_title->setText( truncateTextToFit( title, m_title->font(), QRectF( 0, 0, textWidth, 30 ) ) );
-    m_album->setText( truncateTextToFit( album, m_album->font(), QRectF( 0, 0, textWidth, 30 ) ) );
+    m_title->setScrollingText( title, QRectF( m_title->pos().x(), textY, textWidth, 30 ) );
+    m_artist->setScrollingText( artist, QRectF( m_artist->pos().x(), textY, textWidth, 30 ) );
+    m_album->setScrollingText( album, QRectF( m_album->pos().x(), textY, textWidth, 30 ) );
 
+    if( !m_trackActions.isEmpty() )
+    {
+        QPointF iconPos = m_album->pos();
+        iconPos.setY( iconPos.y() + m_album->boundingRect().height() );
+        foreach( Plasma::IconWidget *icon, m_trackActions )
+        {
+            debug() << "painting action: " << icon->text();
+            const int iconSize = icon->size().width();
+            icon->setPos( iconPos );
+            iconPos.rx() += iconSize + 4;
+#if QT_VERSION >= 0x040500
+	        icon->setOpacity( .72 );
+#endif
+        }
+    }
 
     if( !m_lastTracks.isEmpty() )
-    {                
+    {
         m_tracksToShow = qMin( m_lastTracks.count(), ( int )( ( contentsRect().height() ) / ( textHeight ) ) );
 
         // Note: TabBar disabled for 2.1-beta1 release, due to issues with visual appearance and usability
@@ -254,7 +312,7 @@ void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
 
         m_tabBar->show();
 #endif
-        
+
         for( int i = 0; i < m_tracksToShow; i++ )
         {
             m_tracks[i]->resize( contentsRect().width() - standardPadding() * 2, textHeight * .8 );
@@ -263,14 +321,14 @@ void CurrentTrack::constraintsEvent( Plasma::Constraints constraints )
             //m_tracks[i]->setPos( ( rect().width() - m_tracks[i]->boundingRect().width() ) / 2, textHeight * 1.2 * i + 43 );
             m_tracks[i]->setPos( ( rect().width() - m_tracks[i]->boundingRect().width() ) / 2, ( textHeight * .8 + standardPadding() / 2 ) * i + 25 );
         }
-    }        
+    }
     else if( !m_noTrackText.isEmpty() )
     {
         m_noTrack->setText( truncateTextToFit( m_noTrackText, m_noTrack->font(), QRectF( 0, 0, textWidth, 30 ) ) );
         m_noTrack->setPos( size().toSize().width() / 2 - m_noTrack->boundingRect().width() / 2,
                        size().toSize().height() / 2  - 30 );
     }
-    
+
     dataEngine( "amarok-current" )->setProperty( "coverWidth", albumWidth );
 }
 
@@ -280,7 +338,7 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
     DEBUG_BLOCK
     Q_UNUSED( name );
 
-    if( data.size() == 0 ) 
+    if( data.size() == 0 )
         return;
 
     QRect textRect( 0, 0, m_maxTextWidth, 30 );
@@ -288,7 +346,7 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
     m_noTrackText = data[ "notrack" ].toString();
     m_lastTracks = data[ "lastTracks" ].value<Meta::TrackList>();
     m_favoriteTracks = data[ "favoriteTracks" ].value<Meta::TrackList>();
-    
+
     if( !m_lastTracks.isEmpty() )
     {
         Meta::TrackList tracks;
@@ -302,7 +360,7 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
             m_tracks[i]->setTrack( track );
             i++;
         }
-        
+
         updateConstraints();
         update();
         return;
@@ -322,13 +380,11 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
     m_favoriteTracks.clear();
 
     m_currentInfo = data[ "current" ].toMap();
-    m_title->setText( truncateTextToFit( m_currentInfo[ Meta::Field::TITLE ].toString(), m_title->font(), textRect ) );
-
+    m_title->setScrollingText( m_currentInfo[ Meta::Field::TITLE ].toString(), textRect );
     const QString artist = m_currentInfo.contains( Meta::Field::ARTIST ) ? m_currentInfo[ Meta::Field::ARTIST ].toString() : QString();
-    m_artist->setText( truncateTextToFit( artist, m_artist->font(), textRect ) );
-
+    m_artist->setScrollingText( artist, textRect );
     const QString album = m_currentInfo.contains( Meta::Field::ALBUM ) ? m_currentInfo[ Meta::Field::ALBUM ].toString() : QString();
-    m_album->setText( truncateTextToFit( album, m_album->font(), textRect ) );
+    m_album->setScrollingText( album, textRect );
 
     m_rating = m_currentInfo[ Meta::Field::RATING ].toInt();
 
@@ -345,20 +401,43 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
     m_bigCover = data[ "albumart" ].value<QPixmap>();
     m_sourceEmblemPath = data[ "source_emblem" ].toString();
 
+    qDeleteAll( m_trackActions );
+    m_trackActions.clear();
+
+    Meta::TrackPtr track = The::engineController()->currentTrack();
+    if( track )
+    {
+
+        //first, add any global CurrentTrackActions (iow, actions that are shown for all tracks)
+
+        foreach( QAction* action, The::globalCurrentTrackActions()->actions() )
+        {
+            Plasma::IconWidget *icon = addAction( action, 24 );
+            icon->setText( QString() );
+            m_trackActions << icon;
+        }
+        
+        if( track->hasCapabilityInterface( Meta::Capability::CurrentTrackActions ) )
+        {
+            Meta::CurrentTrackActionsCapability *cac = track->create<Meta::CurrentTrackActionsCapability>();
+            if( cac )
+            {
+                QList<QAction*> actions = cac->customActions();
+                foreach( QAction *action, actions )
+                {
+                    Plasma::IconWidget *icon = addAction( action, 24 );
+                    icon->setText( QString() );
+                    m_trackActions << icon;
+                }
+            }
+        }
+    }
+
     // without that the rating doesn't get update for a playing track
     update();
     updateConstraints();
 }
 
-QSizeF 
-CurrentTrack::sizeHint( Qt::SizeHint which, const QSizeF & constraint) const
-{
-    Q_UNUSED( which )
-
-    return QSizeF( constraint.width(), 180 );
-}
-
-
 void
 CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect )
 {
@@ -367,7 +446,7 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
 
     // tint the whole applet
     addGradientToAppletBackground( p );
-    
+
     //bail out if there is no room to paint. Prevents crashes and really there is no sense in painting if the
     //context view has been minimized completely
     if( ( contentsRect.width() < 20 ) || ( contentsRect.height() < 20 ) )
@@ -379,8 +458,8 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
     else
     {
         foreach ( QGraphicsItem * childItem, QGraphicsItem::children () )
-            childItem->show();        
-    }    
+            childItem->show();
+    }
 
     if( !m_lastTracks.isEmpty() )
     {
@@ -391,7 +470,7 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
         // Note: TabBar disabled for 2.1-beta1 release, due to issues with visual appearance and usability
 #if 0
         m_tabBar->show();
-#endif 
+#endif
         for( int i = 0; i < m_tracksToShow; i++)
             m_tracks[i]->show();
         return;
@@ -411,26 +490,27 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
         for( int i = 0; i < MAX_PLAYED_TRACKS; i++)
             m_tracks[i]->hide();
     }
-    
+
     p->save();
-    
-    
+
+
     Meta::TrackPtr track = The::engineController()->currentTrack();
 
-    // Only show the ratings widget if the current track is in the collection
-    if( track && track->inCollection() )
+    bool hasActions = !m_trackActions.isEmpty();
+    bool updatable = false;
+
+    if( track )
     {
-        m_ratingWidget->show();
-        m_showStatistics = true;
+        // Don't show collection actions for local tracks
+        Meta::UpdateCapability *uc = track->create<Meta::UpdateCapability>();
+        if( uc )
+            updatable = true;
+
+        m_showStatistics = updatable || hasActions;
     }
-    else
-    {
-        m_ratingWidget->hide();
-        m_showStatistics = false;
-    }
-    
+
     p->restore();
-    
+
     // draw border around ratingwidget
     p->save();
     p->setRenderHint( QPainter::Antialiasing );
@@ -488,16 +568,16 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
         headerPath.lineTo( leftEdge, m_ratingWidget->pos().y() - m_ratingWidget->boundingRect().height() + 8 + 6 ); // curve back through start
         headerPath.quadTo( leftEdge, m_ratingWidget->pos().y() - m_ratingWidget->boundingRect().height() + 8,
                         leftEdge + 6, m_ratingWidget->pos().y() - m_ratingWidget->boundingRect().height() + 8 );
-        
+
         p->fillPath( headerPath, topColor );
 
         // draw label text
         p->save();
         // draw "Play count"
-        QString playCountText = i18n( "Play count" );
-        QString scoreText = i18n( "Score" );
-        QString lastPlayedText = i18n( "Last Played" );
-        
+        const QString playCountText = i18n( "Play count" );
+        const QString scoreText = i18n( "Score" );
+        const QString lastPlayedText = i18n( "Last Played" );
+
         //Align labels taking into account the string widths for each label
         QFontMetrics fm( this->font() );
         qreal totalWidth = fm.width( playCountText ) + fm.width( scoreText ) + fm.width( lastPlayedText );
@@ -510,7 +590,7 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
                             localMaxTextWidth * factor,
                             m_ratingWidget->boundingRect().height() - 4 ); // just the "first" row, so go halfway down
 
-        
+
         m_playCountLabel = truncateTextToFit( playCountText, this->font(), rect );
         p->drawText( rect, Qt::AlignCenter | Qt::TextSingleLine, m_playCountLabel );
 
@@ -525,7 +605,7 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
         factor = fm.width( lastPlayedText ) / totalWidth;
         rect.setWidth( localMaxTextWidth * factor );
         rect.moveLeft( rect.topLeft().x() + localMaxTextWidth * prevFactor );
-        
+
         m_lastPlayedLabel = truncateTextToFit( lastPlayedText, this->font(), rect );
         p->drawText( rect, Qt::AlignCenter | Qt::TextSingleLine, m_lastPlayedLabel );
 
@@ -536,7 +616,7 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
                     localMaxTextWidth * factor,
                     m_ratingWidget->boundingRect().height() - 4 );
         p->drawText( rect,  Qt::AlignCenter | Qt::TextSingleLine, m_numPlayed );
-        
+
         factor = fm.width( scoreText ) / totalWidth;
         rect.setWidth( localMaxTextWidth * factor );
         rect.moveLeft( rect.topLeft().x() + localMaxTextWidth * prevFactor );
@@ -553,21 +633,21 @@ CurrentTrack::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *optio
         p->restore();
     }
     p->restore();
-    
+
     // draw source emblem
     if( !m_sourceEmblemPath.isEmpty() )
     {
         p->save();
-        p->setOpacity( .4 );
+        p->setOpacity( 0.19 );
         KSvgRenderer svg( m_sourceEmblemPath );
 
         // paint the emblem half as tall as the applet, anchored at the top-right
         // assume it is a square emblem
         qreal height = boundingRect().height() / 2;
-        QRectF rect( boundingRect().width() - standardPadding() - height, ( boundingRect().height() / 2 - ( height / 2 ) ),
+        QRectF rect( boundingRect().width() - standardPadding() - height, standardPadding(),
                      height, height );
         svg.render( p, rect );
-        
+
         p->restore();
     }
 }
@@ -576,7 +656,7 @@ bool
 CurrentTrack::resizeCover( QPixmap cover, qreal width, QPointF albumCoverPos )
 {
     const int borderWidth = 5;
-    
+
     if( !cover.isNull() )
     {
         width -= borderWidth * 2;
@@ -595,20 +675,70 @@ CurrentTrack::resizeCover( QPixmap cover, qreal width, QPointF albumCoverPos )
         // center
         moveByX += ( width / 2 ) - cover.rect().width() / 2;
         moveByY += ( width / 2 ) - cover.rect().height() / 2;
-        
-        debug() << "placing album at X:" << moveByX << " and Y:"  << moveByY;
+
         m_albumCover->setPos( moveByX, moveByY );
 
 
         QPixmap coverWithBorders = The::svgHandler()->addBordersToPixmap( cover, borderWidth, m_album->text(), true );
 
-        
+        // HACK to make it "on top always !"
+        m_albumCover->setCacheMode(NoCache);
         m_albumCover->setPixmap( coverWithBorders );
+        m_albumCover->update();
+        m_albumCover->setCacheMode(ItemCoordinateCache);
         return true;
     }
     return false;
 }
 
+void
+CurrentTrack::changeTitleFont()
+{
+    QFont font = ui_Settings.fontChooser->font();
+
+    m_noTrack->setFont( font );
+    m_title->setFont( font );
+    m_artist->setFont( font );
+    m_album->setFont( font );
+
+    KConfigGroup config = Amarok::config("Current Track Applet");
+    config.writeEntry( "Font", font.family() );
+    config.writeEntry( "Size", font.pointSize() );
+
+    constraintsEvent();
+
+    debug() << "Setting Lyrics Applet font: " << font.family() << " " << font.pointSize();
+}
+
+void
+CurrentTrack::createConfigurationInterface( KConfigDialog *parent )
+{
+    KConfigGroup configuration = config();
+    QWidget *settings = new QWidget;
+    ui_Settings.setupUi( settings );
+    ui_Settings.fontChooser->setFont( m_title->font() );
+
+    parent->addPage( settings, i18n( "Current Track Settings" ), "preferences-system");
+
+    connect( parent, SIGNAL( accepted() ), this, SLOT( changeTitleFont() ) );
+}
+
+void
+CurrentTrack::coverDropped( QPixmap cover )
+{
+    DEBUG_BLOCK
+    Meta::TrackPtr track = The::engineController()->currentTrack();
+    if( !track )
+        return;
+
+    Meta::AlbumPtr album = track->album();
+    if( !album )
+        return;
+
+    if ( !cover.isNull() )
+        album->setImage( cover );
+}
+
 void
 CurrentTrack::paletteChanged( const QPalette & palette )
 {
@@ -618,6 +748,8 @@ CurrentTrack::paletteChanged( const QPalette & palette )
     m_artist->setBrush( palette.text() );
     m_album->setBrush( palette.text() );
     m_noTrack->setBrush( palette.text() );
+    m_byText->setBrush( palette.text() );
+    m_onText->setBrush( palette.text() );
 }
 
 void
@@ -628,8 +760,8 @@ CurrentTrack::tabChanged( int index )
         tracks = m_lastTracks;
     else
         tracks = m_favoriteTracks;
-    
-    int i = 0;    
+
+    int i = 0;
     foreach( Meta::TrackPtr track, tracks )
     {
         m_tracks[i]->hide();
