@@ -1,26 +1,24 @@
-/***************************************************************************
- *   Copyright (C) 2004-2007 by Mark Kretschmann <kretschmann@kde.org>     *
- *                 2005-2007 by Seb Ruiz <ruiz@kde.org>                    *
- *                      2006 by Alexandre Oliveira <aleprj@gmail.com>      *
- *                      2006 by Martin Ellis <martin.ellis@kdemail.net>    *
- *                      2007 by Leonardo Franchi <lfranchi@gmail.com>      *
- *                      2008 by Peter ZHOU <peterzhoulei@gmail.com>        *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2004-2010 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2005-2007 Seb Ruiz <ruiz@kde.org>                                      *
+ * Copyright (c) 2006 Alexandre Pereira de Oliveira <aleprj@gmail.com>                  *
+ * Copyright (c) 2006 Martin Ellis <martin.ellis@kdemail.net>                           *
+ * Copyright (c) 2007 Leo Franchi <lfranchi@gmail.com>                                  *
+ * Copyright (c) 2008 Peter ZHOU <peterzhoulei@gmail.com>                               *
+ * Copyright (c) 2009 Jakob Kummerow <jakob.kummerow@gmail.com>                         *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #define DEBUG_PREFIX "ScriptManager"
 
@@ -50,6 +48,7 @@
 #include "scriptengine/AmarokWindowScript.h"
 #include "scriptengine/MetaTypeExporter.h"
 #include "scriptengine/ScriptImporter.h"
+#include "ScriptUpdater.h"
 #include "ui_ScriptManagerBase.h"
 
 #include <KApplication>
@@ -57,13 +56,21 @@
 #include <KIO/NetAccess>
 #include <KLocale>
 #include <KMessageBox>
-#include <knewstuff2/engine.h>
-#include <knewstuff2/core/entry.h>
 #include <KProtocolManager>
 #include <KStandardDirs>
 #include <KTar>
 #include <KWindowSystem>
 
+#include <kdeversion.h>
+
+#if KDE_IS_VERSION(4, 3, 80)
+#define HAVE_KNEWSTUFF3
+#include <KNS3/DownloadDialog>
+#endif
+
+#include <knewstuff2/engine.h>
+#include <knewstuff2/core/entry.h>
+
 #include <QFileInfo>
 #include <QTimer>
 #include <QScriptEngine>
@@ -72,30 +79,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-namespace Amarok
-{
-    QString
-    proxyForUrl(const QString& url)
-    {
-        KUrl kurl( url );
-
-        QString proxy;
-
-        if ( KProtocolManager::proxyForUrl( kurl ) !=
-                QString::fromLatin1( "DIRECT" ) ) {
-            KProtocolManager::slaveProtocol ( kurl, proxy );
-        }
-
-        return proxy;
-    }
-
-    QString
-    proxyForProtocol(const QString& protocol)
-    {
-        return KProtocolManager::proxyFor( protocol );
-    }
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // class ScriptManager
 ////////////////////////////////////////////////////////////////////////////////
@@ -125,6 +108,9 @@ ScriptManager::ScriptManager( QWidget* parent )
     gui.setupUi( main );
     setMainWidget( main );
 
+    // Load config
+    gui.kcfg_AutoUpdateScripts->setChecked( AmarokConfig::autoUpdateScripts() );
+
     m_scriptSelector = gui.pluginWidget;
     gui.pluginWidget->setSizePolicy(QSizePolicy::Preferred ,QSizePolicy::Expanding);
 
@@ -134,19 +120,24 @@ ScriptManager::ScriptManager( QWidget* parent )
     connect( gui.okButton,        SIGNAL( clicked() ), SLOT( reject() ) );
     connect( m_scriptSelector, SIGNAL( changed( bool ) ), SLOT( slotConfigChanged( bool ) ) );
     connect( m_scriptSelector, SIGNAL( configCommitted ( const QByteArray & ) ), SLOT( slotConfigComitted( const QByteArray & ) ) );
+    connect( gui.kcfg_AutoUpdateScripts, SIGNAL( toggled( bool ) ), SLOT( slotUpdateSettingChanged( bool ) ) );
 
     gui.installButton  ->setIcon( KIcon( "folder-amarok" ) );
     gui.retrieveButton ->setIcon( KIcon( "get-hot-new-stuff-amarok" ) );
     gui.uninstallButton->setIcon( KIcon( "edit-delete-amarok" ) );
     gui.okButton       ->setIcon( KIcon( "dialog-ok" ) );
+    
     // Center the dialog in the middle of the mainwindow
     const int x = parentWidget()->width() / 2 - sizeHint().width() / 2;
     const int y = parentWidget()->height() / 2 - sizeHint().height() / 2;
     move( x, y );
 
     // Delay this call via eventloop, because it's a bit slow and would block
-    QTimer::singleShot( 0, this, SLOT( findScripts() ) );
+    QTimer::singleShot( 0, this, SLOT( updateAllScripts() ) );
 
+
+    //FIXME This is only here for testing purposes. Remove later.
+    //QTimer::singleShot( 0, this, SLOT( showScriptStalledDialog() ) );
 }
 
 ScriptManager::~ScriptManager()
@@ -245,41 +236,83 @@ ScriptManager::notifyFetchLyricsByUrl( const QString& artist, const QString& tit
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// private slots
+// private slots (script updater stuff)
 ////////////////////////////////////////////////////////////////////////////////
 
 void
-ScriptManager::findScripts() //SLOT
+ScriptManager::updateAllScripts() // SLOT
+{
+
+    DEBUG_BLOCK
+    // find all scripts (both in $KDEHOME and /usr)
+    QStringList foundScripts = KGlobal::dirs()->findAllResources( "data", "amarok/scripts/*/main.js", KStandardDirs::Recursive | KStandardDirs::NoDuplicates);
+    m_nScripts = foundScripts.count();
+
+    // get timestamp of the last update check
+    KConfigGroup config = Amarok::config( "ScriptManager" );
+    uint lastCheck = config.readEntry( "LastUpdateCheck", QVariant( 0 ) ).toUInt();
+    uint now = QDateTime::currentDateTime().toTime_t();
+
+    // last update was at least 7 days ago -> check now
+    if ( now - lastCheck > 7*24*60*60 )
+    {
+        debug() << "ScriptUpdater: Performing script update check now!";
+        // create a ScriptUpdater for each script
+        m_updaters = new ScriptUpdater[m_nScripts];
+        for ( int i = 0; i < m_nScripts; i++ )
+        {
+            // all the ScriptUpdaters are now started in parallel.
+            // tell them which script to work on
+            m_updaters[i].setScriptPath( foundScripts.at( i ) );
+            // tell them whom to signal when they're finished
+            connect ( &(m_updaters[i]), SIGNAL( finished( QString ) ), SLOT( updaterFinished( QString ) ) );
+            // and finally tell them to get to work
+            QTimer::singleShot( 0, &(m_updaters[i]), SLOT( updateScript() ) );
+        }
+        // store current timestamp
+        config.writeEntry( "LastUpdateCheck", QVariant( now ) );
+        config.sync();
+    }
+    // last update was pretty recent, don't check again
+    else
+    {
+        debug() << "ScriptUpdater: Skipping update check";
+        for ( int i = 0; i < m_nScripts; i++ )
+        {
+            loadScript( foundScripts.at( i ) );
+        }
+        findScripts();
+    }
+
+}
+
+void
+ScriptManager::updaterFinished( QString scriptPath ) // SLOT
 {
     DEBUG_BLOCK
-
-    const QStringList allFiles = KGlobal::dirs()->findAllResources( "data", "amarok/scripts/*/main.js", KStandardDirs::Recursive );
-
-    // Add found scripts to treeWidget:
-    QList<KPluginInfo> LyricsInfoList;
-    QList<KPluginInfo> GenericInfoList;
-    QList<KPluginInfo> ServiceInfoList;
-    foreach( const QString &str, allFiles )
+    // count this event
+    m_updateSemaphore.release();
+    // load the script now (so that findScripts() doesn't have to do this)
+    loadScript( scriptPath );
+    // if all ScriptUpdaters have returned now, call findScripts()
+    if ( m_updateSemaphore.tryAcquire(m_nScripts) )
     {
-        loadScript( str );
+        findScripts();
     }
-    foreach( const QString &key, m_scripts.keys() )
-    {
-        if ( m_scripts[key].info.category() == "Generic" )
-            GenericInfoList.append( m_scripts[key].info );
-        else if ( m_scripts[key].info.category() == "Lyrics" )
-            LyricsInfoList.append( m_scripts[key].info );
-        else if ( m_scripts[key].info.category() == "Scriptable Service" )
-            ServiceInfoList.append( m_scripts[key].info );
-    }
-    m_scriptSelector->addScripts( GenericInfoList, KPluginSelector::ReadConfigFile, "Generic" );
-    m_scriptSelector->addScripts( LyricsInfoList, KPluginSelector::ReadConfigFile, "Lyrics" );
-    m_scriptSelector->addScripts( ServiceInfoList, KPluginSelector::ReadConfigFile, "Scriptable Service" );
-
-    // Handle auto-run:
-    slotConfigChanged( true );
 }
 
+void
+ScriptManager::slotUpdateSettingChanged( bool enabled )
+{
+    DEBUG_BLOCK
+    // Save config
+    AmarokConfig::setAutoUpdateScripts( enabled );
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// private slots
+////////////////////////////////////////////////////////////////////////////////
+
 bool
 ScriptManager::slotInstallScript( const QString& path )
 {
@@ -362,6 +395,19 @@ ScriptManager::recurseInstall( const KArchiveDirectory* archiveDir, const QStrin
 void
 ScriptManager::slotRetrieveScript()
 {
+#ifdef HAVE_KNEWSTUFF3
+    KNS3::DownloadDialog dialog("amarok.knsrc", this);
+    dialog.exec();
+
+    if (!dialog.installedEntries().isEmpty()) {
+        KMessageBox::information( 0, i18n( "<p>Script successfully installed.</p>"
+                                            "<p>Please restart Amarok to start the script.</p>" ) );
+    } else if (!dialog.changedEntries().isEmpty()) {
+        KMessageBox::information( 0, i18n( "<p>Script successfully uninstalled.</p>"
+                                            "<p>Please restart Amarok to totally remove the script.</p>" ) );
+    }
+
+#else
     bool installed = false;
     bool deleted = false;
     KNS::Engine engine( this );
@@ -380,6 +426,7 @@ ScriptManager::slotRetrieveScript()
     else if (  deleted )
         KMessageBox::information( 0, i18n( "<p>Script successfully uninstalled.</p>"
                                             "<p>Please restart Amarok to totally remove the script.</p>" ) );
+#endif
 }
 
 void
@@ -430,7 +477,7 @@ ScriptManager::slotRunScript( QString name, bool silent )
     m_scripts[name].evaluating = true;
     if( m_scripts[name].info.category() == "Lyrics" )
         m_lyricsScript = name;
-   
+
     m_scripts[name].log += time.currentTime().toString() + " Script Started!" + '\n';
     m_scripts[name].engine->setProcessEventsInterval( 100 );
     m_scripts[name].engine->evaluate( scriptFile.readAll() );
@@ -460,9 +507,6 @@ ScriptManager::slotRunScript( QString name, bool silent )
     else
         slotStopScript( name );
 
-
-
-
     return true;
 }
 
@@ -557,6 +601,23 @@ ScriptManager::scriptFinished( QString name ) //SLOT
     delete m_scripts[name].engine;
 }
 
+
+//MOCKUP method, see API docs
+void
+ScriptManager::showScriptStalledDialog()  // SLOT
+{
+    const QString script = "FIXME";
+
+    const int reply = KMessageBox::questionYesNo( 0, i18n( "The script '%1' appears to have stalled.\n\n"
+                                                           "Would you like to stop it?", script ),
+                                                           i18n( "Script Manager - Amarok" ),
+                                                           KStandardGuiItem::yes(),
+                                                           KStandardGuiItem::no(),
+                                                           QString(),
+                                                           KMessageBox::Dangerous );
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // private
 ////////////////////////////////////////////////////////////////////////////////
@@ -611,6 +672,32 @@ ScriptManager::loadScript( const QString& path )
     return true;
 }
 
+void
+ScriptManager::findScripts()
+{
+    DEBUG_BLOCK
+
+    // Add found scripts to treeWidget:
+    QList<KPluginInfo> LyricsInfoList;
+    QList<KPluginInfo> GenericInfoList;
+    QList<KPluginInfo> ServiceInfoList;
+    foreach( const QString &key, m_scripts.keys() )
+    {
+        if ( m_scripts[key].info.category() == "Generic" )
+            GenericInfoList.append( m_scripts[key].info );
+        else if ( m_scripts[key].info.category() == "Lyrics" )
+            LyricsInfoList.append( m_scripts[key].info );
+        else if ( m_scripts[key].info.category() == "Scriptable Service" )
+            ServiceInfoList.append( m_scripts[key].info );
+    }
+    m_scriptSelector->addScripts( GenericInfoList, KPluginSelector::ReadConfigFile, "Generic" );
+    m_scriptSelector->addScripts( LyricsInfoList, KPluginSelector::ReadConfigFile, "Lyrics" );
+    m_scriptSelector->addScripts( ServiceInfoList, KPluginSelector::ReadConfigFile, "Scriptable Service" );
+
+    // Handle auto-run:
+    slotConfigChanged( true );
+}
+
 void
 ScriptManager::startScriptEngine( QString name )
 {
@@ -634,7 +721,7 @@ ScriptManager::startScriptEngine( QString name )
     scriptObject = scriptEngine->newQObject( objectPtr );
     m_global.setProperty( "Script", scriptObject );
     m_scripts[name].wrapperList.append( objectPtr );
-    
+
     objectPtr = new InfoScript( m_scripts[name].url );
     QScriptValue infoContext = scriptEngine->newQObject( objectPtr );
     m_global.setProperty( "Info", infoContext );
@@ -657,7 +744,7 @@ ScriptManager::startScriptEngine( QString name )
     scriptObject = scriptEngine->newQObject( objectPtr );
     m_global.setProperty( "Lyrics", scriptObject );
     m_scripts[name].wrapperList.append( objectPtr );
-    
+
     objectPtr = new AmarokScript::AmarokServicePluginManagerScript( scriptEngine );
     scriptObject = scriptEngine->newQObject( objectPtr );
     m_global.setProperty( "ServicePluginManager", scriptObject );
@@ -713,9 +800,5 @@ ScriptManager::startScriptEngine( QString name )
     m_scripts[name].wrapperList.append( trackProto );
 }
 
-
-
-
-
 #include "ScriptManager.moc"
 
