@@ -1,22 +1,24 @@
-/***************************************************************************
- * copyright            : (C) 2008 Soren Harward <stharward@gmail.com>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- **************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2008 Soren Harward <stharward@gmail.com>                               *
+ * Copyright (c) 2009 Téo Mrnjavac <teo.mrnjavac@gmail.com>                             *
+ * Copyright (c) 2009 Nikolaj Hald Nielsen <nhn@kde.org>                                *
+ * Copyright (c) 2009 John Atkinson <john@fauxnetic.co.uk>                              *
+ * Copyright (c) 2009 Oleksandr Khayrullin <saniokh@gmail.com>                          *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) version 3 or        *
+ * any later version accepted by the membership of KDE e.V. (or its successor approved  *
+ * by the membership of KDE e.V.), which shall act as a proxy defined in Section 14 of  *
+ * version 3 of the license.                                                            *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #define DEBUG_PREFIX "Playlist::PrettyListView"
 
@@ -24,30 +26,32 @@
 
 #include "amarokconfig.h"
 #include "context/ContextView.h"
-#include "context/popupdropper/libpud/PopupDropperAction.h"
 #include "context/popupdropper/libpud/PopupDropperItem.h"
 #include "context/popupdropper/libpud/PopupDropper.h"
 #include "Debug.h"
 #include "EngineController.h"
-#include "PrettyItemDelegate.h"
 #include "dialogs/TagDialog.h"
 #include "GlobalCurrentTrackActions.h"
 #include "meta/capabilities/CurrentTrackActionsCapability.h"
+#include "meta/capabilities/FindInSourceCapability.h"
 #include "meta/capabilities/MultiSourceCapability.h"
 #include "meta/Meta.h"
 #include "PaletteHandler.h"
-#include "playlist/GroupingProxy.h"
+#include "playlist/layouts/LayoutManager.h"
+#include "playlist/proxymodels/GroupingProxy.h"
 #include "playlist/PlaylistActions.h"
-#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModelStack.h"
 #include "playlist/view/PlaylistViewCommon.h"
-#include "playlist/navigators/NavigatorFilterProxyModel.h"
 #include "PopupDropperFactory.h"
 #include "SvgHandler.h"
 #include "SourceSelectionPopup.h"
+#include "tooltips/ToolTipManager.h"
 
 #include <KApplication>
 #include <KMenu>
+#include <KUrl>
 
+#include <QClipboard>
 #include <QContextMenuEvent>
 #include <QDropEvent>
 #include <QItemSelection>
@@ -58,24 +62,29 @@
 #include <QPainter>
 #include <QPalette>
 #include <QPersistentModelIndex>
+#include <QScrollBar>
 #include <QTimer>
 
 Playlist::PrettyListView::PrettyListView( QWidget* parent )
         : QListView( parent )
-        , EngineObserver( The::engineController() )
+        , ViewCommon()
         , m_headerPressIndex( QModelIndex() )
         , m_mousePressInHeader( false )
         , m_skipAutoScroll( false )
         , m_pd( 0 )
+        , m_topmostProxy( Playlist::ModelStack::instance()->top() )
+        , m_toolTipManager(0)
 {
-    setModel( GroupingProxy::instance() );
-    setItemDelegate( new PrettyItemDelegate( this ) );
-    setSelectionMode( QAbstractItemView::ExtendedSelection );
-    setDragDropMode( QAbstractItemView::DragDrop );
+    setModel( Playlist::ModelStack::instance()->top() );
+    m_prettyDelegate = new PrettyItemDelegate( this );
+    setItemDelegate( m_prettyDelegate );
+    setSelectionMode( ExtendedSelection );
+    setDragDropMode( DragDrop );
     setDropIndicatorShown( false ); // we draw our own drop indicator
+    setEditTriggers ( SelectedClicked | EditKeyPressed );
     setAutoScroll( true );
 
-    setVerticalScrollMode( QAbstractItemView::ScrollPerPixel );
+    setVerticalScrollMode( ScrollPerPixel );
 
     // rendering adjustments
     setFrameShape( QFrame::NoFrame );
@@ -93,16 +102,35 @@ Playlist::PrettyListView::PrettyListView( QWidget* parent )
 
     connect( m_proxyUpdateTimer, SIGNAL( timeout() ), this, SLOT( updateProxyTimeout() ) );
 
-    connect( The::playlistModel(), SIGNAL( itemsAdded( int ) ), this, SLOT( itemsAdded( int ) ) );
+    connect( model(), SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), this, SLOT( itemsAdded( const QModelIndex&, int, int ) ) );
+
+    connect( model(), SIGNAL( layoutChanged() ), this, SLOT( reset() ) );
+
+    m_animationTimer = new QTimer(this);
+    connect( m_animationTimer, SIGNAL( timeout() ), this, SLOT( redrawActive() ) );
+    m_animationTimer->setInterval( 250 );
+
+    connect( LayoutManager::instance(), SIGNAL( activeLayoutChanged() ), this, SLOT( playlistLayoutChanged() ) );
+
+    if ( LayoutManager::instance()->activeLayout().inlineControls() )
+        m_animationTimer->start();
+
+    connect( model(), SIGNAL( beginRemoveIds() ), this, SLOT( saveTrackSelection() ) );
+    connect( model(), SIGNAL( removedIds( const QList<quint64>& ) ), this, SLOT( restoreTrackSelection() ) );
+
+    m_toolTipManager = new ToolTipManager(this);
 }
 
-Playlist::PrettyListView::~PrettyListView() {}
+Playlist::PrettyListView::~PrettyListView()
+{}
 
-void
-Playlist::PrettyListView::engineNewTrackPlaying()
+int
+Playlist::PrettyListView::verticalOffset() const
 {
-    if( AmarokConfig::autoScrollPlaylist() )
-        scrollToActiveTrack();
+    int ret = QListView::verticalOffset();
+    if ( verticalScrollBar() && verticalScrollBar()->maximum() )
+        ret += verticalScrollBar()->value() * 10 / verticalScrollBar()->maximum();
+    return ret;
 }
 
 void
@@ -135,13 +163,24 @@ Playlist::PrettyListView::removeSelection()
     QList<int> sr = selectedRows();
     if( !sr.isEmpty() )
     {
-        qSort( sr );
-        int firstRow = sr.first();
-        Controller::instance()->removeRows( sr );
+        // Now that we have the list of selected rows in the topmost proxy, we can perform the
+        // removal.
+        The::playlistController()->removeRows( sr );
 
-        //select the track immediately above the cleared are as this is the one that ow has internal focus.
-        firstRow = qBound( 0, firstRow, model()->rowCount() -1 );
-        selectionModel()->select( model()->index( firstRow, 0, QModelIndex() ), QItemSelectionModel::Select );
+        // Next, we look for the first row.
+        int firstRow = sr.first();
+        foreach( int i, sr )
+        {
+            if( i < firstRow )
+                firstRow = i;
+        }
+
+        //Select the track occupied by the first deleted track. Also move the current item to here as
+        //button presses up or down wil otherwise not behave as expected.
+        firstRow = qBound( 0, firstRow, m_topmostProxy->rowCount() -1 );
+        QModelIndex newSelectionIndex = model()->index(  firstRow, 0 );
+        setCurrentIndex( newSelectionIndex );
+        selectionModel()->select( newSelectionIndex, QItemSelectionModel::Select );
     }
 }
 
@@ -157,9 +196,18 @@ Playlist::PrettyListView::dequeueSelection()
     Actions::instance()->dequeue( selectedRows() );
 }
 
+void
+Playlist::PrettyListView::switchQueueState() // slot
+{
+    DEBUG_BLOCK
+
+    const bool isQueued = currentIndex().data( Playlist::StateRole ).toInt() & Item::Queued;
+    isQueued ? dequeueSelection() : queueSelection();
+    update(); // repaint (necessary when using shortcut)
+}
+
 void Playlist::PrettyListView::selectSource()
 {
-
     DEBUG_BLOCK
 
     QList<int> rows = selectedRows();
@@ -169,7 +217,7 @@ void Playlist::PrettyListView::selectSource()
         return;
 
     //get the track...
-    QModelIndex index = GroupingProxy::instance()->index( rows.at( 0 ) );
+    QModelIndex index = model()->index( rows.at( 0 ), 0 );
     Meta::TrackPtr track = index.data( Playlist::TrackRole ).value< Meta::TrackPtr >();
 
     //get multiSource capability:
@@ -182,10 +230,8 @@ void Playlist::PrettyListView::selectSource()
         sourceSelector->show();
         //dialog deletes msc when done with it.
     }
-
 }
 
-
 void
 Playlist::PrettyListView::scrollToActiveTrack()
 {
@@ -196,7 +242,7 @@ Playlist::PrettyListView::scrollToActiveTrack()
         m_skipAutoScroll = false;
         return;
     }
-    QModelIndex activeIndex = model()->index( GroupingProxy::instance()->activeRow(), 0, QModelIndex() );
+    QModelIndex activeIndex = model()->index( m_topmostProxy->activeRow(), 0, QModelIndex() );
     if ( activeIndex.isValid() )
         scrollTo( activeIndex, QAbstractItemView::PositionAtCenter );
 }
@@ -207,14 +253,45 @@ Playlist::PrettyListView::trackActivated( const QModelIndex& idx )
     DEBUG_BLOCK
     m_skipAutoScroll = true; // we don't want to do crazy view changes when selecting an item in the view
     Actions::instance()->play( idx );
+
+    //make sure that the track we just activated is also set as the current index or
+    //the selected index will get moved to the first row, making keyboard navigation difficult (BUG 225791)
+    selectionModel_setCurrentIndex( idx, QItemSelectionModel::ClearAndSelect );
 }
 
+
+// The following 2 functions are a workaround for crash BUG 222961 and BUG 229240:
+//   There appears to be a bad interaction between Qt 'setCurrentIndex()' and
+//   Qt 'selectedIndexes()' / 'selectionModel()->select()' / 'scrollTo()'.
+//
+//   'setCurrentIndex()' appears to do something bad with its QModelIndex parameter,
+//   leading to a crash deep within Qt.
+//
+//   It might be our fault, but we suspect a bug in Qt.  (Qt 4.6 at least)
+//
+//   The problem goes away if we use a fresh QModelIndex, which we also don't re-use
+//   afterwards.
+void
+Playlist::PrettyListView::setCurrentIndex( const QModelIndex &index )
+{
+    QModelIndex indexCopy = model()->index( index.row(), index.column() );
+    QListView::setCurrentIndex( indexCopy );
+}
+
+void
+Playlist::PrettyListView::selectionModel_setCurrentIndex( const QModelIndex &index, QItemSelectionModel::SelectionFlags command )
+{
+    QModelIndex indexCopy = model()->index( index.row(), index.column() );
+    selectionModel()->setCurrentIndex( indexCopy, command );
+}
+
+
 void
 Playlist::PrettyListView::showEvent( QShowEvent* event )
 {
     QTimer::singleShot( 0, this, SLOT( fixInvisible() ) );
 
-    QListView::showEvent( event ); 
+    QListView::showEvent( event );
 }
 
 // This method is a workaround for BUG 184714.
@@ -223,7 +300,7 @@ Playlist::PrettyListView::showEvent( QShowEvent* event )
 // Without this workaround the playlist stays invisible when the application is restored from the tray.
 // This is especially a problem with the Dynamic Playlist mode, which modifies the model without user interaction.
 //
-// The bug only seems to happen with Qt 4.5.x, so it might actually be a bug in Qt. 
+// The bug only seems to happen with Qt 4.5.x, so it might actually be a bug in Qt.
 void
 Playlist::PrettyListView::fixInvisible() //SLOT
 {
@@ -244,12 +321,7 @@ void
 Playlist::PrettyListView::contextMenuEvent( QContextMenuEvent* event )
 {
     DEBUG_BLOCK
-    QModelIndex filteredIndex = indexAt( event->pos() );
-    
-    //translate to real model as we might be looking at a filered list:
-    int sourceRow = NavigatorFilterProxyModel::instance()->rowToSource( filteredIndex.row() );
-
-    QModelIndex index = The::playlistModel()->index( sourceRow );
+    QModelIndex index = indexAt( event->pos() );
 
     if ( !index.isValid() )
         return;
@@ -258,7 +330,7 @@ Playlist::PrettyListView::contextMenuEvent( QContextMenuEvent* event )
     if( event->modifiers() & Qt::ControlModifier )
         return;
 
-    ViewCommon::trackMenu( this, &index, event->globalPos(), true );
+    trackMenu( this, &index, event->globalPos(), true );
     event->accept();
 }
 
@@ -287,15 +359,39 @@ Playlist::PrettyListView::stopAfterTrack()
     }
 }
 
+void
+Playlist::PrettyListView::findInSource()
+{
+    DEBUG_BLOCK
+    const qint64 id = currentIndex().data( UniqueIdRole ).value<quint64>();
+    if( id != -1 )
+    {
+        Meta::TrackPtr track = m_topmostProxy->trackForId( id );
+
+        if( track->hasCapabilityInterface( Meta::Capability::FindInSource ) )
+        {
+            Meta::FindInSourceCapability *fis = track->create<Meta::FindInSourceCapability>();
+            if ( fis )
+            {
+                fis->findInSource();
+            }
+            delete fis;
+        }
+    }
+}
+
 void
 Playlist::PrettyListView::dragMoveEvent( QDragMoveEvent* event )
 {
     QModelIndex index = indexAt( event->pos() );
-    if ( index.isValid() ) {
+    if ( index.isValid() )
+    {
         m_dropIndicator = visualRect( index );
-    } else {
+    }
+    else
+    {
         // draw it on the bottom of the last item
-        index = model()->index( GroupingProxy::instance()->rowCount() - 1, 0, QModelIndex() );
+        index = model()->index( m_topmostProxy->rowCount() - 1, 0, QModelIndex() );
         m_dropIndicator = visualRect( index );
         m_dropIndicator = m_dropIndicator.translated( 0, m_dropIndicator.height() );
     }
@@ -314,7 +410,7 @@ Playlist::PrettyListView::dropEvent( QDropEvent* event )
         int targetRow = indexAt( event->pos() ).row();
         targetRow = ( targetRow < 0 ) ? plModel->rowCount() : targetRow; // target of < 0 means we dropped on the end of the playlist
         QList<int> sr = selectedRows();
-        int realtarget = Controller::instance()->moveRows( sr, targetRow );
+        int realtarget = The::playlistController()->moveRows( sr, targetRow );
         QItemSelection selItems;
         foreach( int row, sr )
         {
@@ -364,10 +460,23 @@ Playlist::PrettyListView::keyPressEvent( QKeyEvent* event )
 void
 Playlist::PrettyListView::mousePressEvent( QMouseEvent* event )
 {
+    //get the item that was clicked
+    QModelIndex index = indexAt( event->pos() );
+
+    //first of all, if a left click, check if the delegate wants to do something about this click
+    if( event->button() == Qt::LeftButton )
+    {
+        //we need to translate the position of the click into something relative to the item that was clicked.
+        QRect itemRect = visualRect( index );
+        QPoint relPos =  event->pos() - itemRect.topLeft();
+
+        if ( m_prettyDelegate->clicked( relPos, itemRect, index ) )
+            return;  //click already handled...
+    }
+
     if ( mouseEventInHeader( event ) && ( event->button() == Qt::LeftButton ) )
     {
         m_mousePressInHeader = true;
-        QModelIndex index = indexAt( event->pos() );
         m_headerPressIndex = QPersistentModelIndex( index );
         int rows = index.data( GroupedTracksRole ).toInt();
         QModelIndex bottomIndex = model()->index( index.row() + rows - 1, 0 );
@@ -377,13 +486,24 @@ Playlist::PrettyListView::mousePressEvent( QMouseEvent* event )
         QItemSelectionModel::SelectionFlags command = headerPressSelectionCommand( index, event );
         selectionModel()->select( selItems, command );
         // TODO: if you're doing shift-select on rows above the header, then the rows following the header will be lost from the selection
-        selectionModel()->setCurrentIndex( index, QItemSelectionModel::NoUpdate );
+        selectionModel_setCurrentIndex( index, QItemSelectionModel::NoUpdate );
     }
     else
     {
         m_mousePressInHeader = false;
     }
 
+    if ( event->button() == Qt::MidButton )
+    {
+        KUrl url( QApplication::clipboard()->text() );
+        if ( url.isValid() )
+        {
+            QList<KUrl> list;
+            list.append( url );
+            The::playlistController()->insertOptioned( list, Playlist::AppendAndPlay );
+        }
+    }
+
     // This should always be forwarded, as it is used to determine the offset
     // relative to the mouse of the selection we are dragging!
     QListView::mousePressEvent( event );
@@ -391,7 +511,18 @@ Playlist::PrettyListView::mousePressEvent( QMouseEvent* event )
     // This must go after the call to the super class as the current index is not yet selected otherwise
     // Queueing support for Ctrl Right click
     if( event->button() == Qt::RightButton && event->modifiers() & Qt::ControlModifier )
-        queueSelection();
+    {
+        // HACK: Implement a nicer way in Actions class to queue just one row
+        // TODO: Make it possible to enqueue multiple rows. Tricky.
+        QList<int> list;
+        list.append( index.row() );
+
+        if( index.data( Playlist::StateRole ).toInt() & Item::Queued )
+            Actions::instance()->dequeue( list );
+        else
+            Actions::instance()->queue( list );
+        update();
+    }
 }
 
 void
@@ -452,6 +583,10 @@ Playlist::PrettyListView::startDrag( Qt::DropActions supportedActions )
 {
     DEBUG_BLOCK
 
+    QModelIndexList indices = selectedIndexes();
+    if( indices.isEmpty() )
+        return; // no items selected in the view, abort. See bug 226167
+
     //Waah? when a parent item is dragged, startDrag is called a bunch of times
     static bool ongoingDrags = false;
     if( ongoingDrags )
@@ -463,16 +598,13 @@ Playlist::PrettyListView::startDrag( Qt::DropActions supportedActions )
 
     if( m_pd && m_pd->isHidden() )
     {
-
         m_pd->setSvgRenderer( The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ) );
         qDebug() << "svgHandler SVG renderer is " << (QObject*)(The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ));
         qDebug() << "m_pd SVG renderer is " << (QObject*)(m_pd->svgRenderer());
         qDebug() << "does play exist in renderer? " << ( The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" )->elementExists( "load" ) );
-        QModelIndexList indices = selectedIndexes();
 
-        QList<PopupDropperAction*> actions =  ViewCommon::actionsFor( this, &indices.first(), true );
-
-        foreach( PopupDropperAction * action, actions )
+        QList<QAction*> actions =  actionsFor( this, &indices.first(), true );
+        foreach( QAction * action, actions )
             m_pd->addItem( The::popupDropperFactory()->createItem( action ), true );
 
         m_pd->show();
@@ -529,10 +661,7 @@ Playlist::PrettyListView::selectedRows() const
 {
     QList<int> rows;
     foreach( const QModelIndex &idx, selectedIndexes() )
-    {
-        int sourceRow = NavigatorFilterProxyModel::instance()->rowToSource( idx.row() );
-        rows.append( sourceRow );
-    }
+        rows.append( idx.row() );
     return rows;
 }
 
@@ -546,20 +675,19 @@ void Playlist::PrettyListView::newPalette( const QPalette & palette )
 void Playlist::PrettyListView::find( const QString &searchTerm, int fields, bool filter )
 {
     bool updateProxy = false;
-    if ( ( GroupingProxy::instance()->currentSearchFields() != fields ) || ( GroupingProxy::instance()->currentSearchTerm() != searchTerm ) )
+    if ( ( m_topmostProxy->currentSearchFields() != fields ) || ( m_topmostProxy->currentSearchTerm() != searchTerm ) )
         updateProxy = true;
 
-    int row = GroupingProxy::instance()->find( searchTerm, fields );
+    int row = m_topmostProxy->find( searchTerm, fields );
     if( row != -1 )
     {
         //select this track
+        QModelIndex index = model()->index( row, 0 );
+        QItemSelection selItems( index, index );
+        selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
 
         if ( !filter )
         {
-            QModelIndex index = model()->index( row, 0 );
-            QItemSelection selItems( index, index );
-            selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
-
             QModelIndex foundIndex = model()->index( row, 0, QModelIndex() );
             setCurrentIndex( foundIndex );
             if ( foundIndex.isValid() )
@@ -571,7 +699,6 @@ void Playlist::PrettyListView::find( const QString &searchTerm, int fields, bool
     else
         emit( notFound() );
 
-
     //instead of kicking the proxy right away, start a 500msec timeout.
     //this stops us from updating it for each letter of a long search term,
     //and since it does not affect any views, this is fine. Worst case is that
@@ -587,14 +714,14 @@ void Playlist::PrettyListView::findNext( const QString & searchTerm, int fields
     QList<int> selected = selectedRows();
 
     bool updateProxy = false;
-    if ( ( GroupingProxy::instance()->currentSearchFields() != fields ) || ( GroupingProxy::instance()->currentSearchTerm() != searchTerm ) )
+    if ( ( m_topmostProxy->currentSearchFields() != fields ) || ( m_topmostProxy->currentSearchTerm() != searchTerm ) )
         updateProxy = true;
 
     int currentRow = -1;
     if( selected.size() > 0 )
         currentRow = selected.last();
 
-    int row = GroupingProxy::instance()->findNext( searchTerm, currentRow, fields );
+    int row = m_topmostProxy->findNext( searchTerm, currentRow, fields );
     if( row != -1 )
     {
         //select this track
@@ -614,7 +741,7 @@ void Playlist::PrettyListView::findNext( const QString & searchTerm, int fields
         emit( notFound() );
 
     if ( updateProxy )
-        NavigatorFilterProxyModel::instance()->filterUpdated();
+        m_topmostProxy->filterUpdated();
 }
 
 void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fields )
@@ -623,14 +750,14 @@ void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fie
     QList<int> selected = selectedRows();
 
     bool updateProxy = false;
-    if ( ( GroupingProxy::instance()->currentSearchFields() != fields ) || ( GroupingProxy::instance()->currentSearchTerm() != searchTerm ) )
+    if ( ( m_topmostProxy->currentSearchFields() != fields ) || ( m_topmostProxy->currentSearchTerm() != searchTerm ) )
         updateProxy = true;
 
-    int currentRow = GroupingProxy::instance()->totalLength();
+    int currentRow = m_topmostProxy->totalLength();
     if( selected.size() > 0 )
         currentRow = selected.first();
 
-    int row = GroupingProxy::instance()->findPrevious( searchTerm, currentRow, fields );
+    int row = m_topmostProxy->findPrevious( searchTerm, currentRow, fields );
     if( row != -1 )
     {
         //select this track
@@ -650,31 +777,32 @@ void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fie
         emit( notFound() );
 
     if ( updateProxy )
-        NavigatorFilterProxyModel::instance()->filterUpdated();
+        m_topmostProxy->filterUpdated();
 }
 
 void Playlist::PrettyListView::clearSearchTerm()
 {
     DEBUG_BLOCK
-            
+
     //We really do not want to reset the view to the top when the search/filter is cleared, so
     //we store the first shown row and scroll to that once the term is removed.
     QModelIndex index = indexAt( QPoint( 0, 0 ) );
 
-     //ah... but we want the source row and not the one reported by the filter model(s)
-    int row = NavigatorFilterProxyModel::instance()->rowToSource( index.row() );
+    //We don't want to mess around with source rows because this would break our wonderful
+    //lasagna code, but we do want to grab something unique that represents the row, like
+    //its unique 64-bit id.
+    quint64 id = m_topmostProxy->idAt( index.row() );
 
     debug() << "first row in filtered list: " << index.row();
-    debug() << "source row: " << row;
 
-    NavigatorFilterProxyModel::instance()->filterUpdated();
-    GroupingProxy::instance()->clearSearchTerm();
+    m_topmostProxy->filterUpdated();
+    m_topmostProxy->clearSearchTerm();
 
-    //now scroll to the selected row again
-
-    QModelIndex sourceIndex = model()->index( row, 0, QModelIndex() );
-    if ( sourceIndex.isValid() )
-        scrollTo( sourceIndex, QAbstractItemView::PositionAtTop );
+    //Now we scroll to the previously stored row again. Note that it's not the same row in
+    //the topmost model any more, so we need to grab it again using its id.
+    QModelIndex newIndex = model()->index( m_topmostProxy->rowForId( id ), 0, QModelIndex() );
+    if ( newIndex.isValid() )
+        scrollTo( newIndex, QAbstractItemView::PositionAtTop );
 }
 
 void Playlist::PrettyListView::startProxyUpdateTimeout()
@@ -690,30 +818,63 @@ void Playlist::PrettyListView::startProxyUpdateTimeout()
 void Playlist::PrettyListView::updateProxyTimeout()
 {
     DEBUG_BLOCK
-    NavigatorFilterProxyModel::instance()->filterUpdated();
+    m_topmostProxy->filterUpdated();
 }
 
 void Playlist::PrettyListView::showOnlyMatches( bool onlyMatches )
 {
-    NavigatorFilterProxyModel::instance()->setPassThrough( !onlyMatches );
+    m_topmostProxy->showOnlyMatches( onlyMatches );
 }
 
-void Playlist::PrettyListView::itemsAdded( int firstRow )
+void Playlist::PrettyListView::itemsAdded( const QModelIndex& parent, int firstRow, int lastRow )
 {
     DEBUG_BLOCK
+    Q_UNUSED( parent )
+    Q_UNUSED( lastRow )
 
-    QModelIndex index = model()->index( NavigatorFilterProxyModel::instance()->rowFromSource( firstRow ) , 0 );
+    QModelIndex index = model()->index( firstRow, 0);
     if( !index.isValid() )
         return;
 
     debug() << "index has row: " << index.row();
-    //scrollTo( firstItem/*, QAbstractItemView::PositionAtCenter*/ );
     scrollTo( index, QAbstractItemView::PositionAtCenter );
-    
+
 }
 
+void Playlist::PrettyListView::redrawActive()
+{
+    int activeRow = m_topmostProxy->activeRow();
+    QModelIndex index = model()->index( activeRow, 0, QModelIndex() );
+    update( index );
+}
 
+void Playlist::PrettyListView::playlistLayoutChanged()
+{
+    if ( LayoutManager::instance()->activeLayout().inlineControls() )
+        m_animationTimer->start();
+    else
+        m_animationTimer->stop();
+}
+
+void Playlist::PrettyListView::saveTrackSelection()
+{
+    m_savedTrackSelection.clear();
+
+    foreach( int rowId, selectedRows() )
+        m_savedTrackSelection.append( m_topmostProxy->idAt( rowId ) );
+}
+
+void Playlist::PrettyListView::restoreTrackSelection()
+{
+    selectionModel()->clearSelection();
+
+    foreach( qint64 savedTrackId, m_savedTrackSelection )
+    {
+        QModelIndex restoredIndex = model()->index( m_topmostProxy->rowForId( savedTrackId ), 0, QModelIndex() );
+
+        if( restoredIndex.isValid() )
+            selectionModel()->select( restoredIndex, QItemSelectionModel::Select );
+    }
+}
 
 #include "PrettyListView.moc"
-
-
