@@ -1,22 +1,20 @@
-/*
-   Copyright (C) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>
-   Copyright (C) 2007 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>
-   Copyright (C) 2008 Casey Link <unnamedrambler@gmail.com>
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
-*/
+/****************************************************************************************
+ * Copyright (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>            *
+ * Copyright (c) 2007 Nikolaj Hald Nielsen <nhn@kde.org>                                *
+ * Copyright (c) 2008 Casey Link <unnamedrambler@gmail.com>                             *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #ifndef SERVICECOLLECTION_H
 #define SERVICECOLLECTION_H
@@ -29,6 +27,7 @@
 #include <KIcon>
 
 #include <QtGlobal>
+#include <QSharedPointer>
 
 typedef QMap<int, Meta::TrackPtr> TrackIdMap;
 typedef QMap<int, Meta::ArtistPtr> ArtistIdMap;
@@ -40,7 +39,7 @@ typedef QMap<int, Meta::GenrePtr> GenreIdMap;
  *  fetch their data from somewhere ( a web service, an external program, etc....)
  */
 
-class AMAROK_EXPORT ServiceCollection : public Amarok::Collection, public MemoryCollection
+class AMAROK_EXPORT ServiceCollection : public Amarok::Collection
 {
     Q_OBJECT
     public:
@@ -85,8 +84,22 @@ class AMAROK_EXPORT ServiceCollection : public Amarok::Collection, public Memory
 
         ServiceBase * service();
 
+        //convenience functions for subclasses
+        void acquireWriteLock() { m_mc->acquireWriteLock(); }
+        void acquireReadLock() { m_mc->acquireReadLock(); }
+        void releaseLock() { m_mc->releaseLock(); }
+        GenreMap genreMap() const { return m_mc->genreMap(); }
+        void setGenreMap( const GenreMap &map ) { m_mc->setGenreMap( map ); }
+        ArtistMap artistMap() const { return m_mc->artistMap(); }
+        void setArtistMap( const ArtistMap &map ) { m_mc->setArtistMap( map ); }
+        TrackMap trackMap() const { return m_mc->trackMap(); }
+        void setTrackMap( const TrackMap &map ) { m_mc->setTrackMap( map ); }
+        AlbumMap albumMap() const { return m_mc->albumMap(); }
+        void setAlbumMap( const AlbumMap &map ) { m_mc->setAlbumMap( map ); }
+
     private:
         ServiceBase * m_service;
+        QSharedPointer<MemoryCollection> m_mc;
 
         ServiceMetaFactory * m_metaFactory;
 
