@@ -1,24 +1,20 @@
-/***************************************************************************
- *   Copyright (c) 2007  Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>    *
- *             (c) 2007  Adam Pigg <adam@piggz.co.uk>                      *
- *             (c) 2007  Casey Link <unnamedrambler@gmail.com>             *
- *                                                                         *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2007 Nikolaj Hald Nielsen <nhn@kde.org>                                *
+ * Copyright (c) 2007 Adam Pigg <adam@piggz.co.uk>                                      *
+ * Copyright (c) 2007 Casey Link <unnamedrambler@gmail.com>                             *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "AmpacheServiceQueryMaker.h"
 
@@ -49,7 +45,7 @@ AmpacheServiceQueryMaker::AmpacheServiceQueryMaker( AmpacheServiceCollection * c
     , d( new Private )
     , m_server( server )
     , m_sessionId( sessionId )
-    , m_dateFilter ( -1 )
+    , m_dateFilter( -1 )
 {
     DEBUG_BLOCK
     m_collection = collection;
@@ -97,9 +93,9 @@ AmpacheServiceQueryMaker::run()
     //TODO error handling
     if ( d->type == Private::NONE )
         return;
-    
+
     m_collection->acquireReadLock();
-    
+
     if (  d->type == Private::ARTIST )
         fetchArtists();
     else if( d->type == Private::ALBUM )
@@ -149,24 +145,53 @@ QueryMaker *
 AmpacheServiceQueryMaker::addMatch( const ArtistPtr & artist )
 {
     DEBUG_BLOCK
-
-    if ( m_parentAlbumId.isEmpty() )
+    if( m_parentAlbumId.isEmpty() )
     {
-        const ServiceArtist * serviceArtist = static_cast< const ServiceArtist * >( artist.data() );
-        m_parentArtistId = QString::number( serviceArtist->id() );
+        const ServiceArtist * serviceArtist = dynamic_cast< const ServiceArtist * >( artist.data() );
+        if( serviceArtist )
+        {
+            m_parentArtistId = QString::number( serviceArtist->id() );
+        }
+        else
+        {
+            if( m_collection->artistMap().contains( artist->name() ) )
+            {
+                serviceArtist = static_cast< const ServiceArtist* >( m_collection->artistMap().value( artist->name() ).data() );
+                m_parentArtistId = QString::number( serviceArtist->id() );
+            }
+            else
+            {
+                //hmm, not sure what to do now
+            }
+        }
         //debug() << "parent id set to: " << m_parentArtistId;
     }
     return this;
 }
 
 QueryMaker *
-AmpacheServiceQueryMaker::addMatch(const Meta::AlbumPtr & album)
+AmpacheServiceQueryMaker::addMatch( const Meta::AlbumPtr & album )
 {
     DEBUG_BLOCK
-    const ServiceAlbum * serviceAlbum = static_cast< const ServiceAlbum * >( album.data() );
-    m_parentAlbumId = QString::number( serviceAlbum->id() );
-    //debug() << "parent id set to: " << m_parentAlbumId;
-    m_parentArtistId.clear();
+    const ServiceAlbum * serviceAlbum = dynamic_cast< const ServiceAlbum * >( album.data() );
+    if( serviceAlbum )
+    {
+        m_parentAlbumId = QString::number( serviceAlbum->id() );
+        //debug() << "parent id set to: " << m_parentAlbumId;
+        m_parentArtistId.clear();
+    }
+    else
+    {
+        if( m_collection->albumMap().contains( album->name() ) )
+        {
+            serviceAlbum = static_cast< const ServiceAlbum* >( m_collection->albumMap().value( album->name() ).data() );
+            m_parentAlbumId = QString::number( serviceAlbum->id() );
+        }
+        else
+        {
+            //hmm, not sure what to do now
+        }
+    }
 
     return this;
 }
@@ -206,7 +231,7 @@ AmpacheServiceQueryMaker::handleResult( const AlbumList &albums )
 }
 
 void
-AmpacheServiceQueryMaker::handleResult(const TrackList & tracks)
+AmpacheServiceQueryMaker::handleResult( const TrackList & tracks )
 {
     DEBUG_BLOCK
 
@@ -228,29 +253,28 @@ AmpacheServiceQueryMaker::fetchArtists()
 
     }*/
     //else {
-
-        QString urlString = "<SERVER>/server/xml.server.php?action=artists&auth=<SESSION_ID>";
-
-        urlString.replace( "<SERVER>", m_server);
-        urlString.replace( "<SESSION_ID>", m_sessionId);
+        KUrl request( m_server );
+        request.addPath( "/server/xml.server.php" );
+        request.addQueryItem( "action", "artists" );
+        request.addQueryItem( "auth", m_sessionId );
 
         if ( !m_artistFilter.isEmpty() )
-            urlString += QString( "&filter=" + m_artistFilter );
-        
+            request.addQueryItem( "filter", m_artistFilter );
+
         if( m_dateFilter > 0 )
         {
             QDateTime from;
             from.setTime_t( m_dateFilter );
-            urlString += QString( "&add=" + from.toString( Qt::ISODate ) );
+            request.addQueryItem( "add", from.toString( Qt::ISODate ) );
             debug() << "added date filter with time:" <<  from.toString( Qt::ISODate );
         } else
             debug() << "m_dateFilter is:" << m_dateFilter;
-        
-        urlString += QString( "&limit=" + QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
-        debug() << "Artist url: " << urlString;
+
+        request.addQueryItem( "limit", QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
+        debug() << "Artist url: " << request.url();
 
 
-        m_storedTransferJob =  KIO::storedGet(  KUrl( urlString ), KIO::NoReload, KIO::HideProgressInfo );
+        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
         connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
             , this, SLOT( artistDownloadComplete( KJob *) ) );
     //}
@@ -265,12 +289,9 @@ AmpacheServiceQueryMaker::fetchAlbums()
 
     AlbumList albums;
 
-    //debug() << "parent id: " << m_parentId;
-
     if( !m_parentArtistId.isEmpty() )
     {
-        ArtistMatcher artistMatcher( m_collection->artistById( m_parentArtistId.toInt() ) );
-        albums = artistMatcher.matchAlbums( m_collection );
+        albums = matchAlbums( m_collection, m_collection->artistById( m_parentArtistId.toInt() ) );
     }
 
     if ( albums.count() > 0 )
@@ -280,23 +301,24 @@ AmpacheServiceQueryMaker::fetchAlbums()
     }
     else
     {
-        QString urlString = "<SERVER>/server/xml.server.php?action=artist_albums&auth=<SESSION_ID>";
+        KUrl request( m_server );
+        request.addPath( "/server/xml.server.php" );
+        request.addQueryItem( "action", "artist_albums" );
+        request.addQueryItem( "auth", m_sessionId );
 
-        urlString.replace( "<SERVER>", m_server);
-        urlString.replace( "<SESSION_ID>", m_sessionId);
         if( !m_parentArtistId.isEmpty() )
-            urlString += QString( "&filter=" + m_parentArtistId );
-            
+            request.addQueryItem( "filter", m_parentArtistId );
+
         if( m_dateFilter > 0 )
         {
             QDateTime from;
             from.setTime_t( m_dateFilter );
-            urlString += QString( "&add=" + from.toString( Qt::ISODate ) );
+            request.addQueryItem( "add", from.toString( Qt::ISODate ) );
         }
-        debug() << "request url: " << urlString;
-        urlString += QString( "&limit=" + QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
+        request.addQueryItem( "limit", QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
+        debug() << "request url: " << request.url();
 
-        m_storedTransferJob =  KIO::storedGet(  KUrl( urlString ), KIO::NoReload, KIO::HideProgressInfo );
+        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
         connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
             , this, SLOT( albumDownloadComplete( KJob *) ) );
     }
@@ -314,12 +336,12 @@ AmpacheServiceQueryMaker::fetchTracks()
     if( !m_parentAlbumId.isEmpty() )
     {
         AlbumMatcher albumMatcher( m_collection->albumById( m_parentAlbumId.toInt() ) );
-        tracks = albumMatcher.match( m_collection );
+        tracks = albumMatcher.match( m_collection->trackMap().values() );
     }
     else if ( !m_parentArtistId.isEmpty() )
     {
         ArtistMatcher artistMatcher( m_collection->artistById( m_parentArtistId.toInt() ) );
-        tracks = artistMatcher.match( m_collection );
+        tracks = artistMatcher.match( m_collection->trackMap().values() );
     }
 
     if( tracks.count() > 0 )
@@ -329,32 +351,32 @@ AmpacheServiceQueryMaker::fetchTracks()
     }
     else
     {
-        QString urlString;
+        KUrl request( m_server );
+        request.addPath( "/server/xml.server.php" );
+        request.addQueryItem( "auth", m_sessionId );
 
         if( !m_parentAlbumId.isEmpty() )
-            urlString = "<SERVER>/server/xml.server.php?action=album_songs&auth=<SESSION_ID>";
+            request.addQueryItem( "action", "album_songs" );
         else if( !m_parentArtistId.isEmpty() )
-            urlString = "<SERVER>/server/xml.server.php?action=artist_songs&auth=<SESSION_ID>";
+            request.addQueryItem( "action", "artist_songs" );
         else
-            urlString = "<SERVER>/server/xml.server.php?action=songs&auth=<SESSION_ID>";
+            request.addQueryItem( "action", "songs" );
 
-        urlString.replace( "<SERVER>", m_server);
-        urlString.replace( "<SESSION_ID>", m_sessionId);
         if( !m_parentAlbumId.isEmpty() )
-            urlString += QString( "&filter=" + m_parentAlbumId );
+            request.addQueryItem( "filter", m_parentAlbumId );
         else if( !m_parentArtistId.isEmpty() )
-            urlString += QString( "&filter=" + m_parentArtistId );
+            request.addQueryItem( "filter", m_parentArtistId );
         if( m_dateFilter > 0 )
         {
             QDateTime from;
             from.setTime_t( m_dateFilter );
-            urlString += QString( "&add=" + from.toString( Qt::ISODate ) );
+            request.addQueryItem( "add", from.toString( Qt::ISODate ) );
         }
-        debug() << "request url: " << urlString;
-        
-        urlString += QString( "&limit=" + QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
+        debug() << "request url: " << request.url();
 
-        m_storedTransferJob =  KIO::storedGet(  KUrl( urlString ), KIO::NoReload, KIO::HideProgressInfo );
+        request.addQueryItem( "limit", QString::number( d->maxsize ) );// set to 0 in reset() so fine to use uncondiationally
+
+        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
         connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
             , this, SLOT( trackDownloadComplete( KJob *) ) );
     }
@@ -374,25 +396,23 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
 
     ArtistList artists;
 
-    //debug() << "received artists: " <<  m_storedTransferJob->data();
-
      //so lets figure out what we got here:
     QDomDocument doc( "reply" );
     doc.setContent( m_storedTransferJob->data() );
-    QDomElement root = doc.firstChildElement("root");
+    QDomElement root = doc.firstChildElement( "root" );
 
-    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning 
-    QDomElement error = root.firstChildElement("error"); 
+    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
+    QDomElement error = root.firstChildElement( "error" );
 
-    if ( !error.isNull() ) 
+    if ( !error.isNull() )
     {
         debug () << "Error getting Artist List" << error.text();
-        AmpacheService *m_parentService = dynamic_cast< AmpacheService * >(m_collection->service());
-        if ( m_parentService == 0 ) 
+        AmpacheService *m_parentService = dynamic_cast< AmpacheService * >( m_collection->service() );
+        if ( m_parentService == 0 )
         {
                 return;
         }
-        else 
+        else
         {
             m_parentService->reauthenticate();
         }
@@ -405,7 +425,7 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
         //if ( ! (e.tagName() == "item") )
         //    break;
 
-        QDomElement element = n.firstChildElement("name");
+        QDomElement element = n.firstChildElement( "name" );
         ServiceArtist * artist = new AmpacheArtist( element.text(), m_collection->service() );
 
         int artistId = e.attribute( "id", "0").toInt();
@@ -432,7 +452,7 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
 }
 
 void
-AmpacheServiceQueryMaker::albumDownloadComplete(KJob * job)
+AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
 {
     DEBUG_BLOCK
 
@@ -454,8 +474,8 @@ AmpacheServiceQueryMaker::albumDownloadComplete(KJob * job)
     doc.setContent( m_storedTransferJob->data() );
     QDomElement root = doc.firstChildElement( "root" );
 
-    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning 
-    QDomElement error = root.firstChildElement("error");
+    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
+    QDomElement error = root.firstChildElement( "error" );
 
     if ( !error.isNull() )
     {
@@ -490,7 +510,7 @@ AmpacheServiceQueryMaker::albumDownloadComplete(KJob * job)
         album->setId( albumId );
 
 
-        element = n.firstChildElement("art");
+        element = n.firstChildElement( "art" );
 
         QString coverUrl = element.text();
         album->setCoverUrl( coverUrl );
@@ -527,7 +547,7 @@ AmpacheServiceQueryMaker::albumDownloadComplete(KJob * job)
 }
 
 void
-AmpacheServiceQueryMaker::trackDownloadComplete(KJob * job)
+AmpacheServiceQueryMaker::trackDownloadComplete( KJob * job )
 {
     DEBUG_BLOCK
 
@@ -545,15 +565,15 @@ AmpacheServiceQueryMaker::trackDownloadComplete(KJob * job)
      //so lets figure out what we got here:
     QDomDocument doc( "reply" );
     doc.setContent( m_storedTransferJob->data() );
-    QDomElement root = doc.firstChildElement("root");
+    QDomElement root = doc.firstChildElement( "root" );
 
-    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning 
-    QDomElement error = root.firstChildElement("error");
+    // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
+    QDomElement error = root.firstChildElement( "error" );
 
     if ( !error.isNull() )
     {
         debug () << "Error getting Track Download " << error.text();
-        AmpacheService *m_parentService = dynamic_cast< AmpacheService * >(m_collection->service());
+        AmpacheService *m_parentService = dynamic_cast< AmpacheService * >( m_collection->service() );
         if ( m_parentService == 0 )
         {
                 return;
@@ -571,14 +591,14 @@ AmpacheServiceQueryMaker::trackDownloadComplete(KJob * job)
         //if ( ! (e.tagName() == "item") )
         //    break;
 
-        int trackId = e.attribute( "id", "0").toInt();
-        QDomElement element = n.firstChildElement("title");
+        int trackId = e.attribute( "id", "0" ).toInt();
+        QDomElement element = n.firstChildElement( "title" );
 
         QString title = element.text();
         if ( title.isEmpty() )
             title = "Unknown";
-        element = n.firstChildElement("url");
-        AmpacheTrack * track = new AmpacheTrack( title, m_collection->service()  );
+        element = n.firstChildElement( "url" );
+        AmpacheTrack * track = new AmpacheTrack( title, m_collection->service() );
         TrackPtr trackPtr( track );
 
         //debug() << "Adding track: " <<  title;
@@ -586,20 +606,20 @@ AmpacheServiceQueryMaker::trackDownloadComplete(KJob * job)
         track->setId( trackId );
         track->setUidUrl( element.text() );
 
-        element = n.firstChildElement("time");
-        track->setLength( element.text().toInt() );
+        element = n.firstChildElement( "time" );
+        track->setLength( element.text().toInt() * 1000 );
 
-        element = n.firstChildElement("track");
+        element = n.firstChildElement( "track" );
         track->setTrackNumber( element.text().toInt() );
 
         m_collection->acquireWriteLock();
         m_collection->addTrack( trackPtr );
         m_collection->releaseLock();
 
-        QDomElement albumElement = n.firstChildElement("album");
+        QDomElement albumElement = n.firstChildElement( "album" );
         int albumId = albumElement.attribute( "id", "0").toInt();
 
-        QDomElement artistElement = n.firstChildElement("artist");
+        QDomElement artistElement = n.firstChildElement( "artist" );
         int artistId = artistElement.attribute( "id", "0").toInt();
 
         ArtistPtr artistPtr = m_collection->artistById( artistId );
@@ -631,7 +651,7 @@ AmpacheServiceQueryMaker::trackDownloadComplete(KJob * job)
 }
 
 QueryMaker *
-AmpacheServiceQueryMaker::addFilter(qint64 value, const QString & filter, bool matchBegin, bool matchEnd)
+AmpacheServiceQueryMaker::addFilter( qint64 value, const QString & filter, bool matchBegin, bool matchEnd )
 {
     DEBUG_BLOCK
     Q_UNUSED( matchBegin )
@@ -647,11 +667,11 @@ AmpacheServiceQueryMaker::addFilter(qint64 value, const QString & filter, bool m
     return this;
 }
 
-QueryMaker* 
+QueryMaker*
 AmpacheServiceQueryMaker::addNumberFilter( qint64 value, qint64 filter, QueryMaker::NumberComparison compare )
 {
     DEBUG_BLOCK
-    
+
     if( value == valCreateDate && compare == QueryMaker::GreaterThan )
     {
         debug() << "asking to filter based on added date";
