@@ -1,24 +1,20 @@
-/***************************************************************************
- *   Copyright (c) 2009  Casey Link <unnamedrambler@gmail.com>             *
- *   Copyright (C) 2005 - 2007 by                                          *
- *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
- *      Max Howell, Last.fm Ltd <max@last.fm>                              *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2009 Casey Link <unnamedrambler@gmail.com>                             *
+ * Copyright (c) 2005-2007 Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>          *
+ * Copyright (c) 2005-2007 Max Howell, Last.fm Ltd <max@last.fm>                        *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "LastFmTreeView.h"
 
@@ -29,12 +25,12 @@
 #include "SvgHandler.h"
 #include "context/ContextView.h"
 #include "context/popupdropper/libpud/PopupDropper.h"
-#include "context/popupdropper/libpud/PopupDropperAction.h"
 #include "context/popupdropper/libpud/PopupDropperItem.h"
 
 #include <KIcon>
 #include <KMenu>
 
+#include <QAction>
 #include <QContextMenuEvent>
 #include <QDesktopServices>
 #include <QDropEvent>
@@ -78,23 +74,23 @@ LastFmTreeView::contextMenuEvent ( QContextMenuEvent* event )
     }
     if ( m_currentItems.isEmpty() )
         return;
-    PopupDropperAction separator ( this );
+    QAction separator ( this );
     separator.setSeparator ( true );
 
-    PopupDropperActionList actions = createBasicActions( m_currentItems );
+    QActionList actions = createBasicActions( m_currentItems );
 
     actions += &separator;
     KMenu menu;
-    foreach ( PopupDropperAction * action, actions )
+    foreach ( QAction * action, actions )
         menu.addAction ( action );
 
     menu.exec ( event->globalPos() );
 }
 
-PopupDropperActionList LastFmTreeView::createBasicActions( const QModelIndexList & indices )
+QActionList LastFmTreeView::createBasicActions( const QModelIndexList & indices )
 {
     Q_UNUSED( indices )
-    PopupDropperActionList actions;
+    QActionList actions;
     QModelIndex index = currentIndex();
     QVariant type = model()->data(index, LastFm::TypeRole);
     switch ( type.toInt() )
@@ -113,7 +109,8 @@ PopupDropperActionList LastFmTreeView::createBasicActions( const QModelIndexList
         {
             if ( m_appendAction == 0 )
             {
-                m_appendAction = new PopupDropperAction ( The::svgHandler()->getRenderer ( "amarok/images/pud_items.svg" ), "append", KIcon ( "media-track-add-amarok" ), i18n ( "&Append to Playlist" ), this );
+                m_appendAction = new QAction ( KIcon ( "media-track-add-amarok" ), i18n ( "&Add to Playlist" ), this );
+                m_appendAction->setProperty( "popupdropper_svg_id", "append" );
                 connect ( m_appendAction, SIGNAL ( triggered() ), this, SLOT ( slotAppendChildTracks() ) );
             }
 
@@ -121,7 +118,8 @@ PopupDropperActionList LastFmTreeView::createBasicActions( const QModelIndexList
 
             if ( m_loadAction == 0 )
             {
-                m_loadAction = new PopupDropperAction ( The::svgHandler()->getRenderer ( "amarok/images/pud_items.svg" ), "load", KIcon ( "folder-open" ), i18nc ( "Replace the currently loaded tracks with these", "&Replace Playlist" ), this );
+                m_loadAction = new QAction ( KIcon ( "folder-open" ), i18nc ( "Replace the currently loaded tracks with these", "&Replace Playlist" ), this );
+                m_appendAction->setProperty( "popupdropper_svg_id", "load" );
                 connect ( m_loadAction, SIGNAL ( triggered() ), this, SLOT ( slotPlayChildTracks() ) );
             }
             actions.append ( m_loadAction );
@@ -168,14 +166,14 @@ LastFmTreeView::startDrag(Qt::DropActions supportedActions)
 
         QModelIndexList indices = selectedIndexes();
 
-        PopupDropperActionList actions = createBasicActions( indices );
+        QActionList actions = createBasicActions( indices );
 
         QFont font;
         font.setPointSize( 16 );
         font.setBold( true );
 
-        foreach( PopupDropperAction * action, actions )
-            m_pd->addItem( The::popupDropperFactory()->createItem( action ), false );
+        foreach( QAction * action, actions )
+            m_pd->addItem( The::popupDropperFactory()->createItem( action ) );
 
 
         m_currentItems.clear();
@@ -192,17 +190,17 @@ LastFmTreeView::startDrag(Qt::DropActions supportedActions)
         {
             morePud = The::popupDropperFactory()->createPopupDropper( 0 );
 
-            foreach( PopupDropperAction * action, actions )
-                morePud->addItem( The::popupDropperFactory()->createItem( action ), false );
+            foreach( QAction * action, actions )
+                morePud->addItem( The::popupDropperFactory()->createItem( action ) );
         }
         else
-            m_pd->addItem( The::popupDropperFactory()->createItem( actions[0] ), false );
+            m_pd->addItem( The::popupDropperFactory()->createItem( actions[0] ) );
 
         //TODO: Keep bugging i18n team about problems with 3 dots
         if ( actions.count() > 1 )
         {
-            subItem = m_pd->addSubmenu( &morePud, The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ), "more",  i18n( "More..." )  );
-            The::popupDropperFactory()->adjustSubmenuItem( subItem );
+            subItem = m_pd->addSubmenu( &morePud, i18n( "More..." )  );
+            The::popupDropperFactory()->adjustItem( subItem );
         }
 
         m_pd->show();
