@@ -1,37 +1,38 @@
-/***************************************************************************
- *   Copyright (c) 2006, 2007                                              *
- *        Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                   *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/****************************************************************************************
+ * Copyright (c) 2006,2007 Nikolaj Hald Nielsen <nhn@kde.org>                           *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #include "MagnatuneStore.h"
 
 #include "Amarok.h"
+#include "amarokurls/AmarokUrlHandler.h"
+#include "browsers/CollectionTreeItem.h"
+#include "browsers/SingleCollectionTreeItemModel.h"
 #include "statusbar/StatusBar.h"
 #include "statusbar/ProgressBar.h"
 #include "EngineController.h"
 #include "MagnatuneConfig.h"
 #include "MagnatuneDatabaseWorker.h"
 #include "MagnatuneInfoParser.h"
-#include "../ServiceInfoProxy.h"
+#include "browsers/InfoProxy.h"
+#include "MagnatuneUrlRunner.h"
+
 #include "../ServiceSqlRegistry.h"
 #include "CollectionManager.h"
 #include "Debug.h"
-#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModelStack.h"
 #include "widgets/SearchWidget.h"
 
 #include <KAction>
@@ -97,10 +98,13 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
     DEBUG_BLOCK
     //initTopPanel( );
 
-    setShortDescription( i18n( "The friendly record company with the motto \"We are not evil!\"" ) );
+    setShortDescription( i18n( "\"Fair trade\" online music store." ) );
     setIcon( KIcon( "view-services-magnatune-amarok" ) );
 
-    debug() << "Magnatune browser starting...";
+    // xgettext: no-c-format
+    setLongDescription( i18n( "Magnatune.com is a different kind of record company with the motto \"We are not evil!\" 50% of every purchase goes directly to the artist and if you purchase an album through Amarok, the Amarok project receives a 10% commission. Magnatune.com also offers \"all you can eat\" memberships that lets you download as much of their music as you like." ) );
+    setImagePath( KStandardDirs::locate( "data", "amarok/images/hover_info_magnatune.png" ) );
+
 
     //initBottomPanel();
 //    m_currentlySelectedItem = 0;
@@ -112,11 +116,11 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
     //do this stuff now to make us function properly as a track provider on startup. The expensive stuff will
     //not happen untill the model is added to the view anyway.
     MagnatuneMetaFactory * metaFactory = new MagnatuneMetaFactory( "magnatune", this );
-
+    
     MagnatuneConfig config;
     if ( config.isMember() ) {
         setMembership( config.membershipType(), config.username(), config.password() );
-        metaFactory->setMembershipInfo( m_membershipType.toLower(), m_username, m_password );
+        metaFactory->setMembershipInfo( config.membershipPrefix(), m_username, m_password );
     }
 
     setStreamType( config.streamType() );
@@ -125,9 +129,16 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
     m_registry = new ServiceSqlRegistry( metaFactory );
     m_collection = new MagnatuneSqlCollection( "magnatune", "Magnatune.com", metaFactory, m_registry );
     m_serviceready = true;
+    CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
     emit( ready() );
 }
 
+MagnatuneStore::~MagnatuneStore()
+{
+    CollectionManager::instance()->removeUnmanagedCollection( m_collection );
+    //hm, memory handling?
+}
+
 
 void MagnatuneStore::purchase( )
 {
@@ -174,14 +185,14 @@ void MagnatuneStore::purchase( Meta::MagnatuneAlbum * album )
 
     m_purchaseInProgress = true;
     m_purchaseAlbumButton->setEnabled( false );
-    
+
     if ( !m_purchaseHandler )
     {
         m_purchaseHandler = new MagnatunePurchaseHandler();
         m_purchaseHandler->setParent( this );
         connect( m_purchaseHandler, SIGNAL( purchaseCompleted( bool ) ), this, SLOT( purchaseCompleted( bool ) ) );
     }
-    
+
     m_purchaseHandler->purchaseAlbum( album );
 }
 
@@ -190,7 +201,7 @@ void MagnatuneStore::initTopPanel( )
 {
 
     QMenu *filterMenu = new QMenu( 0 );
-    
+
     QAction *action = filterMenu->addAction( i18n("Artist") );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByArtist() ) );
 
@@ -226,7 +237,7 @@ void MagnatuneStore::initTopPanel( )
 
     KAction *actionsMenuAction = new KAction( KIcon( "list-add" ), i18n( "Tools" ), this );
     actionsMenuAction->setMenu( actionsMenu );
-    
+
     m_searchWidget->toolBar()->addAction( actionsMenuAction );
 
     tbutton = qobject_cast<QToolButton*>( m_searchWidget->toolBar()->widgetForAction( actionsMenuAction ) );
@@ -243,11 +254,11 @@ void MagnatuneStore::initBottomPanel()
     m_purchaseAlbumButton->setParent( m_bottomPanel );
 
     MagnatuneConfig config;
-    if ( config.isMember() && config.membershipType() == "Download" )
+    if ( config.isMember() && config.membershipType() == MagnatuneConfig::DOWNLOAD )
         m_purchaseAlbumButton->setText( i18n( "Download Album" ) );
     else
         m_purchaseAlbumButton->setText( i18n( "Purchase Album" ) );
-    
+
     m_purchaseAlbumButton->setObjectName( "purchaseButton" );
     m_purchaseAlbumButton->setIcon( KIcon( "download-amarok" ) );
     m_purchaseAlbumButton->setEnabled( false );
@@ -285,7 +296,7 @@ bool MagnatuneStore::updateMagnatuneList()
     m_listDownloadJob = KIO::file_copy( KUrl( "http://magnatune.com/info/album_info_xml.bz2" ),  KUrl( m_tempFileName ), 0700 , KIO::HideProgressInfo | KIO::Overwrite );
     The::statusBar()->newProgressOperation( m_listDownloadJob, i18n( "Downloading Magnatune.com Database" ) )
     ->setAbortSlot( this, SLOT( listDownloadCancelled() ) );
-            
+
 
     connect( m_listDownloadJob, SIGNAL( result( KJob * ) ),
             this, SLOT( listDownloadComplete( KJob * ) ) );
@@ -327,10 +338,9 @@ void MagnatuneStore::listDownloadComplete( KJob * downLoadJob )
 void MagnatuneStore::listDownloadCancelled( )
 {
     DEBUG_BLOCK
-    
+
     //The::statusBar()->endProgressOperation( m_listDownloadJob );
     m_listDownloadJob->kill();
-    delete m_listDownloadJob;
     m_listDownloadJob = 0;
     debug() << "Aborted xml download";
 
@@ -351,7 +361,7 @@ void MagnatuneStore::doneParsing()
         config.setLastUpdateTimestamp( QDateTime::currentDateTime().toTime_t() );
     else
         config.setLastUpdateTimestamp( m_magnatuneTimestamp );
-    
+
     config.save();
 }
 
@@ -417,7 +427,7 @@ void MagnatuneStore::addMoodyTracksToPlaylist( const QString &mood, int count )
     MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
     databaseWorker->fetchTrackswithMood( mood, count, m_registry );
     connect( databaseWorker, SIGNAL( gotMoodyTracks( Meta::TrackList ) ), this, SLOT( moodyTracksReady(Meta::TrackList ) ) );
-    
+
     ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
 }
 
@@ -434,11 +444,25 @@ void MagnatuneStore::polish()
 
         QList<int> levels;
         levels << CategoryId::Genre << CategoryId::Artist << CategoryId::Album;
-        
-        setInfoParser( new MagnatuneInfoParser() );
+
+        m_magnatuneInfoParser = new MagnatuneInfoParser();
+
+        setInfoParser( m_magnatuneInfoParser );
         setModel( new SingleCollectionTreeItemModel( m_collection, levels ) );
 
         connect( m_contentView, SIGNAL( itemSelected( CollectionTreeItem * ) ), this, SLOT( itemSelected( CollectionTreeItem * ) ) );
+
+        //add a custom url runner
+
+        MagnatuneUrlRunner * runner = new MagnatuneUrlRunner();
+
+        connect( runner, SIGNAL( showFavorites() ), this, SLOT( showFavoritesPage() ) );
+        connect( runner, SIGNAL( showHome() ), this, SLOT( showHomePage() ) );
+        connect( runner, SIGNAL( showRecommendations() ), this, SLOT( showRecommendationsPage() ) );
+        connect( runner, SIGNAL( buyOrDownload( const QString & ) ), this, SLOT( purchase( const QString & ) ) );
+        connect( runner, SIGNAL( removeFromFavorites( const QString & ) ), this, SLOT( removeFromFavorites( const QString & ) ) );
+
+        The::amarokUrlHandler()->registerRunner( runner, "service_magnatune" );
     }
 
     const KUrl url( KStandardDirs::locate( "data", "amarok/data/" ) );
@@ -460,7 +484,7 @@ void MagnatuneStore::polish()
 
 
 
-void MagnatuneStore::setMembership(const QString & type, const QString & username, const QString & password)
+void MagnatuneStore::setMembership( int type, const QString & username, const QString & password)
 {
     m_isMember = true;
     m_membershipType = type;
@@ -477,7 +501,7 @@ void MagnatuneStore::moodMapReady(QMap< QString, int > map)
     QVariantMap dbusActions;
 
     foreach( const QString &key, map.keys() ) {
-    
+
         strings << key;
         weights << map.value( key );
 
@@ -497,8 +521,8 @@ void MagnatuneStore::moodMapReady(QMap< QString, int > map)
     variantMap["cloud_strings"] = QVariant( strings );
     variantMap["cloud_weights"] = QVariant( weights );
     variantMap["cloud_actions"] = QVariant( dbusActions );
-    
-    The::serviceInfoProxy()->setCloud( variantMap );
+
+    The::infoProxy()->setCloud( variantMap );
 }
 
 
@@ -562,7 +586,7 @@ void MagnatuneStore::checkForUpdates()
 void MagnatuneStore::timestampDownloadComplete( KJob *  job )
 {
     DEBUG_BLOCK
-    
+
     if ( !job->error() == 0 )
     {
         //TODO: error handling here
@@ -635,6 +659,82 @@ QString MagnatuneStore::sendMessage( const QString & message )
     return i18n( "ERROR: Unknown argument." );
 }
 
+void MagnatuneStore::showFavoritesPage()
+{
+    DEBUG_BLOCK
+    m_magnatuneInfoParser->getFavoritesPage();
+}
+
+void MagnatuneStore::showHomePage()
+{
+    DEBUG_BLOCK
+    m_magnatuneInfoParser->getFrontPage();
+}
+
+void MagnatuneStore::showRecommendationsPage()
+{
+    DEBUG_BLOCK
+    m_magnatuneInfoParser->getRecommendationsPage();
+}
+
+void MagnatuneStore::purchase( const QString &sku )
+{
+    DEBUG_BLOCK
+    debug() << "sku: " << sku;
+    MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
+    databaseWorker->fetchAlbumBySku( sku, m_registry );
+    connect( databaseWorker, SIGNAL( gotAlbumBySku( Meta::MagnatuneAlbum * ) ), this, SLOT( purchase( Meta::MagnatuneAlbum * ) ) );
+
+    ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
+}
+
+void MagnatuneStore::addToFavorites( const QString &sku )
+{
+    DEBUG_BLOCK
+    MagnatuneConfig config;
+
+    if( !config.isMember() )
+        return;
+
+    QString url = "http://%1:%2@%3.magnatune.com/member/favorites?action=add_api&sku=%4";
+    url = url.arg( config.username(), config.password(), config.membershipPrefix(), sku );
+
+    debug() << "favorites url: " << url;
+
+    m_favoritesJob = KIO::storedGet( KUrl( url ), KIO::Reload, KIO::HideProgressInfo );
+    connect( m_favoritesJob, SIGNAL( result( KJob * ) ), SLOT( favoritesResult( KJob *  ) ) );
+}
+
+void MagnatuneStore::removeFromFavorites( const QString &sku )
+{
+    DEBUG_BLOCK
+    MagnatuneConfig config;
+
+    if( !config.isMember() )
+        return;
+
+    QString url = "http://%1:%2@%3.magnatune.com/member/favorites?action=remove_api&sku=%4";
+    url = url.arg( config.username(), config.password(), config.membershipPrefix(), sku );
+
+    debug() << "favorites url: " << url;
+
+    m_favoritesJob = KIO::storedGet( KUrl( url ), KIO::Reload, KIO::HideProgressInfo );
+    connect( m_favoritesJob, SIGNAL( result( KJob * ) ), SLOT( favoritesResult( KJob *  ) ) );
+}
+
+void MagnatuneStore::favoritesResult( KJob* addToFavoritesJob )
+{
+    if( addToFavoritesJob != m_favoritesJob )
+        return;
+
+    QString result = m_favoritesJob->data();
+
+    The::statusBar()->longMessage( result );
+
+    //show the favorites page
+    showFavoritesPage();
+}
+
 
 
 #include "MagnatuneStore.moc"
